/****************************************************************************
** Meta object code from reading C++ file 'qitemmodelsurfacedataproxy.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kde-qtdatavis3d/src/datavisualization/data/qitemmodelsurfacedataproxy.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qitemmodelsurfacedataproxy.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QtDataVisualization__QItemModelSurfaceDataProxy_t {
    QByteArrayData data[61];
    char stringdata0[1046];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QtDataVisualization__QItemModelSurfaceDataProxy_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QtDataVisualization__QItemModelSurfaceDataProxy_t qt_meta_stringdata_QtDataVisualization__QItemModelSurfaceDataProxy = {
    {
QT_MOC_LITERAL(0, 0, 47), // "QtDataVisualization::QItemMod..."
QT_MOC_LITERAL(1, 48, 16), // "itemModelChanged"
QT_MOC_LITERAL(2, 65, 0), // ""
QT_MOC_LITERAL(3, 66, 25), // "const QAbstractItemModel*"
QT_MOC_LITERAL(4, 92, 9), // "itemModel"
QT_MOC_LITERAL(5, 102, 14), // "rowRoleChanged"
QT_MOC_LITERAL(6, 117, 4), // "role"
QT_MOC_LITERAL(7, 122, 17), // "columnRoleChanged"
QT_MOC_LITERAL(8, 140, 15), // "xPosRoleChanged"
QT_MOC_LITERAL(9, 156, 15), // "yPosRoleChanged"
QT_MOC_LITERAL(10, 172, 15), // "zPosRoleChanged"
QT_MOC_LITERAL(11, 188, 20), // "rowCategoriesChanged"
QT_MOC_LITERAL(12, 209, 23), // "columnCategoriesChanged"
QT_MOC_LITERAL(13, 233, 25), // "useModelCategoriesChanged"
QT_MOC_LITERAL(14, 259, 6), // "enable"
QT_MOC_LITERAL(15, 266, 24), // "autoRowCategoriesChanged"
QT_MOC_LITERAL(16, 291, 27), // "autoColumnCategoriesChanged"
QT_MOC_LITERAL(17, 319, 21), // "rowRolePatternChanged"
QT_MOC_LITERAL(18, 341, 7), // "pattern"
QT_MOC_LITERAL(19, 349, 24), // "columnRolePatternChanged"
QT_MOC_LITERAL(20, 374, 22), // "xPosRolePatternChanged"
QT_MOC_LITERAL(21, 397, 22), // "yPosRolePatternChanged"
QT_MOC_LITERAL(22, 420, 22), // "zPosRolePatternChanged"
QT_MOC_LITERAL(23, 443, 21), // "rowRoleReplaceChanged"
QT_MOC_LITERAL(24, 465, 7), // "replace"
QT_MOC_LITERAL(25, 473, 24), // "columnRoleReplaceChanged"
QT_MOC_LITERAL(26, 498, 22), // "xPosRoleReplaceChanged"
QT_MOC_LITERAL(27, 521, 22), // "yPosRoleReplaceChanged"
QT_MOC_LITERAL(28, 544, 22), // "zPosRoleReplaceChanged"
QT_MOC_LITERAL(29, 567, 25), // "multiMatchBehaviorChanged"
QT_MOC_LITERAL(30, 593, 18), // "MultiMatchBehavior"
QT_MOC_LITERAL(31, 612, 8), // "behavior"
QT_MOC_LITERAL(32, 621, 16), // "rowCategoryIndex"
QT_MOC_LITERAL(33, 638, 8), // "category"
QT_MOC_LITERAL(34, 647, 19), // "columnCategoryIndex"
QT_MOC_LITERAL(35, 667, 19), // "QAbstractItemModel*"
QT_MOC_LITERAL(36, 687, 7), // "rowRole"
QT_MOC_LITERAL(37, 695, 10), // "columnRole"
QT_MOC_LITERAL(38, 706, 8), // "xPosRole"
QT_MOC_LITERAL(39, 715, 8), // "yPosRole"
QT_MOC_LITERAL(40, 724, 8), // "zPosRole"
QT_MOC_LITERAL(41, 733, 13), // "rowCategories"
QT_MOC_LITERAL(42, 747, 16), // "columnCategories"
QT_MOC_LITERAL(43, 764, 18), // "useModelCategories"
QT_MOC_LITERAL(44, 783, 17), // "autoRowCategories"
QT_MOC_LITERAL(45, 801, 20), // "autoColumnCategories"
QT_MOC_LITERAL(46, 822, 14), // "rowRolePattern"
QT_MOC_LITERAL(47, 837, 17), // "columnRolePattern"
QT_MOC_LITERAL(48, 855, 15), // "xPosRolePattern"
QT_MOC_LITERAL(49, 871, 15), // "yPosRolePattern"
QT_MOC_LITERAL(50, 887, 15), // "zPosRolePattern"
QT_MOC_LITERAL(51, 903, 14), // "rowRoleReplace"
QT_MOC_LITERAL(52, 918, 17), // "columnRoleReplace"
QT_MOC_LITERAL(53, 936, 15), // "xPosRoleReplace"
QT_MOC_LITERAL(54, 952, 15), // "yPosRoleReplace"
QT_MOC_LITERAL(55, 968, 15), // "zPosRoleReplace"
QT_MOC_LITERAL(56, 984, 18), // "multiMatchBehavior"
QT_MOC_LITERAL(57, 1003, 8), // "MMBFirst"
QT_MOC_LITERAL(58, 1012, 7), // "MMBLast"
QT_MOC_LITERAL(59, 1020, 10), // "MMBAverage"
QT_MOC_LITERAL(60, 1031, 14) // "MMBCumulativeY"

    },
    "QtDataVisualization::QItemModelSurfaceDataProxy\0"
    "itemModelChanged\0\0const QAbstractItemModel*\0"
    "itemModel\0rowRoleChanged\0role\0"
    "columnRoleChanged\0xPosRoleChanged\0"
    "yPosRoleChanged\0zPosRoleChanged\0"
    "rowCategoriesChanged\0columnCategoriesChanged\0"
    "useModelCategoriesChanged\0enable\0"
    "autoRowCategoriesChanged\0"
    "autoColumnCategoriesChanged\0"
    "rowRolePatternChanged\0pattern\0"
    "columnRolePatternChanged\0"
    "xPosRolePatternChanged\0yPosRolePatternChanged\0"
    "zPosRolePatternChanged\0rowRoleReplaceChanged\0"
    "replace\0columnRoleReplaceChanged\0"
    "xPosRoleReplaceChanged\0yPosRoleReplaceChanged\0"
    "zPosRoleReplaceChanged\0multiMatchBehaviorChanged\0"
    "MultiMatchBehavior\0behavior\0"
    "rowCategoryIndex\0category\0columnCategoryIndex\0"
    "QAbstractItemModel*\0rowRole\0columnRole\0"
    "xPosRole\0yPosRole\0zPosRole\0rowCategories\0"
    "columnCategories\0useModelCategories\0"
    "autoRowCategories\0autoColumnCategories\0"
    "rowRolePattern\0columnRolePattern\0"
    "xPosRolePattern\0yPosRolePattern\0"
    "zPosRolePattern\0rowRoleReplace\0"
    "columnRoleReplace\0xPosRoleReplace\0"
    "yPosRoleReplace\0zPosRoleReplace\0"
    "multiMatchBehavior\0MMBFirst\0MMBLast\0"
    "MMBAverage\0MMBCumulativeY"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QtDataVisualization__QItemModelSurfaceDataProxy[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      24,   14, // methods
      22,  226, // properties
       1,  336, // enums/sets
       0,    0, // constructors
       0,       // flags
      22,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  158,    2, 0x06 /* Public */,
       5,    1,  161,    2, 0x06 /* Public */,
       7,    1,  164,    2, 0x06 /* Public */,
       8,    1,  167,    2, 0x06 /* Public */,
       9,    1,  170,    2, 0x06 /* Public */,
      10,    1,  173,    2, 0x06 /* Public */,
      11,    0,  176,    2, 0x06 /* Public */,
      12,    0,  177,    2, 0x06 /* Public */,
      13,    1,  178,    2, 0x06 /* Public */,
      15,    1,  181,    2, 0x06 /* Public */,
      16,    1,  184,    2, 0x06 /* Public */,
      17,    1,  187,    2, 0x86 /* Public | MethodRevisioned */,
      19,    1,  190,    2, 0x86 /* Public | MethodRevisioned */,
      20,    1,  193,    2, 0x86 /* Public | MethodRevisioned */,
      21,    1,  196,    2, 0x86 /* Public | MethodRevisioned */,
      22,    1,  199,    2, 0x86 /* Public | MethodRevisioned */,
      23,    1,  202,    2, 0x86 /* Public | MethodRevisioned */,
      25,    1,  205,    2, 0x86 /* Public | MethodRevisioned */,
      26,    1,  208,    2, 0x86 /* Public | MethodRevisioned */,
      27,    1,  211,    2, 0x86 /* Public | MethodRevisioned */,
      28,    1,  214,    2, 0x86 /* Public | MethodRevisioned */,
      29,    1,  217,    2, 0x86 /* Public | MethodRevisioned */,

 // methods: name, argc, parameters, tag, flags
      32,    1,  220,    2, 0x02 /* Public */,
      34,    1,  223,    2, 0x02 /* Public */,

 // signals: revision
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,

 // methods: revision
       0,
       0,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, QMetaType::QString,    6,
    QMetaType::Void, QMetaType::QString,    6,
    QMetaType::Void, QMetaType::QString,    6,
    QMetaType::Void, QMetaType::QString,    6,
    QMetaType::Void, QMetaType::QString,    6,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   14,
    QMetaType::Void, QMetaType::Bool,   14,
    QMetaType::Void, QMetaType::Bool,   14,
    QMetaType::Void, QMetaType::QRegExp,   18,
    QMetaType::Void, QMetaType::QRegExp,   18,
    QMetaType::Void, QMetaType::QRegExp,   18,
    QMetaType::Void, QMetaType::QRegExp,   18,
    QMetaType::Void, QMetaType::QRegExp,   18,
    QMetaType::Void, QMetaType::QString,   24,
    QMetaType::Void, QMetaType::QString,   24,
    QMetaType::Void, QMetaType::QString,   24,
    QMetaType::Void, QMetaType::QString,   24,
    QMetaType::Void, QMetaType::QString,   24,
    QMetaType::Void, 0x80000000 | 30,   31,

 // methods: parameters
    QMetaType::Int, QMetaType::QString,   33,
    QMetaType::Int, QMetaType::QString,   33,

 // properties: name, type, flags
       4, 0x80000000 | 35, 0x0049510b,
      36, QMetaType::QString, 0x00495103,
      37, QMetaType::QString, 0x00495103,
      38, QMetaType::QString, 0x00495103,
      39, QMetaType::QString, 0x00495103,
      40, QMetaType::QString, 0x00495103,
      41, QMetaType::QStringList, 0x00495103,
      42, QMetaType::QStringList, 0x00495103,
      43, QMetaType::Bool, 0x00495103,
      44, QMetaType::Bool, 0x00495103,
      45, QMetaType::Bool, 0x00495103,
      46, QMetaType::QRegExp, 0x00c95103,
      47, QMetaType::QRegExp, 0x00c95103,
      48, QMetaType::QRegExp, 0x00c95103,
      49, QMetaType::QRegExp, 0x00c95103,
      50, QMetaType::QRegExp, 0x00c95103,
      51, QMetaType::QString, 0x00c95103,
      52, QMetaType::QString, 0x00c95103,
      53, QMetaType::QString, 0x00c95103,
      54, QMetaType::QString, 0x00c95103,
      55, QMetaType::QString, 0x00c95103,
      56, 0x80000000 | 30, 0x00c9510b,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,
       4,
       5,
       6,
       7,
       8,
       9,
      10,
      11,
      12,
      13,
      14,
      15,
      16,
      17,
      18,
      19,
      20,
      21,

 // properties: revision
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,

 // enums: name, alias, flags, count, data
      30,   30, 0x0,    4,  341,

 // enum data: key, value
      57, uint(QtDataVisualization::QItemModelSurfaceDataProxy::MMBFirst),
      58, uint(QtDataVisualization::QItemModelSurfaceDataProxy::MMBLast),
      59, uint(QtDataVisualization::QItemModelSurfaceDataProxy::MMBAverage),
      60, uint(QtDataVisualization::QItemModelSurfaceDataProxy::MMBCumulativeY),

       0        // eod
};

void QtDataVisualization::QItemModelSurfaceDataProxy::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QItemModelSurfaceDataProxy *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->itemModelChanged((*reinterpret_cast< const QAbstractItemModel*(*)>(_a[1]))); break;
        case 1: _t->rowRoleChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->columnRoleChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 3: _t->xPosRoleChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 4: _t->yPosRoleChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->zPosRoleChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 6: _t->rowCategoriesChanged(); break;
        case 7: _t->columnCategoriesChanged(); break;
        case 8: _t->useModelCategoriesChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 9: _t->autoRowCategoriesChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 10: _t->autoColumnCategoriesChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 11: _t->rowRolePatternChanged((*reinterpret_cast< const QRegExp(*)>(_a[1]))); break;
        case 12: _t->columnRolePatternChanged((*reinterpret_cast< const QRegExp(*)>(_a[1]))); break;
        case 13: _t->xPosRolePatternChanged((*reinterpret_cast< const QRegExp(*)>(_a[1]))); break;
        case 14: _t->yPosRolePatternChanged((*reinterpret_cast< const QRegExp(*)>(_a[1]))); break;
        case 15: _t->zPosRolePatternChanged((*reinterpret_cast< const QRegExp(*)>(_a[1]))); break;
        case 16: _t->rowRoleReplaceChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 17: _t->columnRoleReplaceChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 18: _t->xPosRoleReplaceChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 19: _t->yPosRoleReplaceChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 20: _t->zPosRoleReplaceChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 21: _t->multiMatchBehaviorChanged((*reinterpret_cast< MultiMatchBehavior(*)>(_a[1]))); break;
        case 22: { int _r = _t->rowCategoryIndex((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 23: { int _r = _t->columnCategoryIndex((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QItemModelSurfaceDataProxy::*)(const QAbstractItemModel * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QItemModelSurfaceDataProxy::itemModelChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QItemModelSurfaceDataProxy::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QItemModelSurfaceDataProxy::rowRoleChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QItemModelSurfaceDataProxy::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QItemModelSurfaceDataProxy::columnRoleChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QItemModelSurfaceDataProxy::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QItemModelSurfaceDataProxy::xPosRoleChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QItemModelSurfaceDataProxy::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QItemModelSurfaceDataProxy::yPosRoleChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QItemModelSurfaceDataProxy::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QItemModelSurfaceDataProxy::zPosRoleChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QItemModelSurfaceDataProxy::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QItemModelSurfaceDataProxy::rowCategoriesChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QItemModelSurfaceDataProxy::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QItemModelSurfaceDataProxy::columnCategoriesChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QItemModelSurfaceDataProxy::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QItemModelSurfaceDataProxy::useModelCategoriesChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QItemModelSurfaceDataProxy::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QItemModelSurfaceDataProxy::autoRowCategoriesChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (QItemModelSurfaceDataProxy::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QItemModelSurfaceDataProxy::autoColumnCategoriesChanged)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (QItemModelSurfaceDataProxy::*)(const QRegExp & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QItemModelSurfaceDataProxy::rowRolePatternChanged)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (QItemModelSurfaceDataProxy::*)(const QRegExp & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QItemModelSurfaceDataProxy::columnRolePatternChanged)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (QItemModelSurfaceDataProxy::*)(const QRegExp & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QItemModelSurfaceDataProxy::xPosRolePatternChanged)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (QItemModelSurfaceDataProxy::*)(const QRegExp & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QItemModelSurfaceDataProxy::yPosRolePatternChanged)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (QItemModelSurfaceDataProxy::*)(const QRegExp & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QItemModelSurfaceDataProxy::zPosRolePatternChanged)) {
                *result = 15;
                return;
            }
        }
        {
            using _t = void (QItemModelSurfaceDataProxy::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QItemModelSurfaceDataProxy::rowRoleReplaceChanged)) {
                *result = 16;
                return;
            }
        }
        {
            using _t = void (QItemModelSurfaceDataProxy::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QItemModelSurfaceDataProxy::columnRoleReplaceChanged)) {
                *result = 17;
                return;
            }
        }
        {
            using _t = void (QItemModelSurfaceDataProxy::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QItemModelSurfaceDataProxy::xPosRoleReplaceChanged)) {
                *result = 18;
                return;
            }
        }
        {
            using _t = void (QItemModelSurfaceDataProxy::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QItemModelSurfaceDataProxy::yPosRoleReplaceChanged)) {
                *result = 19;
                return;
            }
        }
        {
            using _t = void (QItemModelSurfaceDataProxy::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QItemModelSurfaceDataProxy::zPosRoleReplaceChanged)) {
                *result = 20;
                return;
            }
        }
        {
            using _t = void (QItemModelSurfaceDataProxy::*)(MultiMatchBehavior );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QItemModelSurfaceDataProxy::multiMatchBehaviorChanged)) {
                *result = 21;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAbstractItemModel* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QItemModelSurfaceDataProxy *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QAbstractItemModel**>(_v) = _t->itemModel(); break;
        case 1: *reinterpret_cast< QString*>(_v) = _t->rowRole(); break;
        case 2: *reinterpret_cast< QString*>(_v) = _t->columnRole(); break;
        case 3: *reinterpret_cast< QString*>(_v) = _t->xPosRole(); break;
        case 4: *reinterpret_cast< QString*>(_v) = _t->yPosRole(); break;
        case 5: *reinterpret_cast< QString*>(_v) = _t->zPosRole(); break;
        case 6: *reinterpret_cast< QStringList*>(_v) = _t->rowCategories(); break;
        case 7: *reinterpret_cast< QStringList*>(_v) = _t->columnCategories(); break;
        case 8: *reinterpret_cast< bool*>(_v) = _t->useModelCategories(); break;
        case 9: *reinterpret_cast< bool*>(_v) = _t->autoRowCategories(); break;
        case 10: *reinterpret_cast< bool*>(_v) = _t->autoColumnCategories(); break;
        case 11: *reinterpret_cast< QRegExp*>(_v) = _t->rowRolePattern(); break;
        case 12: *reinterpret_cast< QRegExp*>(_v) = _t->columnRolePattern(); break;
        case 13: *reinterpret_cast< QRegExp*>(_v) = _t->xPosRolePattern(); break;
        case 14: *reinterpret_cast< QRegExp*>(_v) = _t->yPosRolePattern(); break;
        case 15: *reinterpret_cast< QRegExp*>(_v) = _t->zPosRolePattern(); break;
        case 16: *reinterpret_cast< QString*>(_v) = _t->rowRoleReplace(); break;
        case 17: *reinterpret_cast< QString*>(_v) = _t->columnRoleReplace(); break;
        case 18: *reinterpret_cast< QString*>(_v) = _t->xPosRoleReplace(); break;
        case 19: *reinterpret_cast< QString*>(_v) = _t->yPosRoleReplace(); break;
        case 20: *reinterpret_cast< QString*>(_v) = _t->zPosRoleReplace(); break;
        case 21: *reinterpret_cast< MultiMatchBehavior*>(_v) = _t->multiMatchBehavior(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QItemModelSurfaceDataProxy *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setItemModel(*reinterpret_cast< QAbstractItemModel**>(_v)); break;
        case 1: _t->setRowRole(*reinterpret_cast< QString*>(_v)); break;
        case 2: _t->setColumnRole(*reinterpret_cast< QString*>(_v)); break;
        case 3: _t->setXPosRole(*reinterpret_cast< QString*>(_v)); break;
        case 4: _t->setYPosRole(*reinterpret_cast< QString*>(_v)); break;
        case 5: _t->setZPosRole(*reinterpret_cast< QString*>(_v)); break;
        case 6: _t->setRowCategories(*reinterpret_cast< QStringList*>(_v)); break;
        case 7: _t->setColumnCategories(*reinterpret_cast< QStringList*>(_v)); break;
        case 8: _t->setUseModelCategories(*reinterpret_cast< bool*>(_v)); break;
        case 9: _t->setAutoRowCategories(*reinterpret_cast< bool*>(_v)); break;
        case 10: _t->setAutoColumnCategories(*reinterpret_cast< bool*>(_v)); break;
        case 11: _t->setRowRolePattern(*reinterpret_cast< QRegExp*>(_v)); break;
        case 12: _t->setColumnRolePattern(*reinterpret_cast< QRegExp*>(_v)); break;
        case 13: _t->setXPosRolePattern(*reinterpret_cast< QRegExp*>(_v)); break;
        case 14: _t->setYPosRolePattern(*reinterpret_cast< QRegExp*>(_v)); break;
        case 15: _t->setZPosRolePattern(*reinterpret_cast< QRegExp*>(_v)); break;
        case 16: _t->setRowRoleReplace(*reinterpret_cast< QString*>(_v)); break;
        case 17: _t->setColumnRoleReplace(*reinterpret_cast< QString*>(_v)); break;
        case 18: _t->setXPosRoleReplace(*reinterpret_cast< QString*>(_v)); break;
        case 19: _t->setYPosRoleReplace(*reinterpret_cast< QString*>(_v)); break;
        case 20: _t->setZPosRoleReplace(*reinterpret_cast< QString*>(_v)); break;
        case 21: _t->setMultiMatchBehavior(*reinterpret_cast< MultiMatchBehavior*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QtDataVisualization::QItemModelSurfaceDataProxy::staticMetaObject = { {
    QMetaObject::SuperData::link<QSurfaceDataProxy::staticMetaObject>(),
    qt_meta_stringdata_QtDataVisualization__QItemModelSurfaceDataProxy.data,
    qt_meta_data_QtDataVisualization__QItemModelSurfaceDataProxy,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QtDataVisualization::QItemModelSurfaceDataProxy::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtDataVisualization::QItemModelSurfaceDataProxy::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QtDataVisualization__QItemModelSurfaceDataProxy.stringdata0))
        return static_cast<void*>(this);
    return QSurfaceDataProxy::qt_metacast(_clname);
}

int QtDataVisualization::QItemModelSurfaceDataProxy::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QSurfaceDataProxy::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 24)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 24;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 24)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 24;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 22;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 22;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 22;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 22;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 22;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QtDataVisualization::QItemModelSurfaceDataProxy::itemModelChanged(const QAbstractItemModel * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QtDataVisualization::QItemModelSurfaceDataProxy::rowRoleChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QtDataVisualization::QItemModelSurfaceDataProxy::columnRoleChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QtDataVisualization::QItemModelSurfaceDataProxy::xPosRoleChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QtDataVisualization::QItemModelSurfaceDataProxy::yPosRoleChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void QtDataVisualization::QItemModelSurfaceDataProxy::zPosRoleChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void QtDataVisualization::QItemModelSurfaceDataProxy::rowCategoriesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QtDataVisualization::QItemModelSurfaceDataProxy::columnCategoriesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QtDataVisualization::QItemModelSurfaceDataProxy::useModelCategoriesChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void QtDataVisualization::QItemModelSurfaceDataProxy::autoRowCategoriesChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void QtDataVisualization::QItemModelSurfaceDataProxy::autoColumnCategoriesChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 10, _a);
}

// SIGNAL 11
void QtDataVisualization::QItemModelSurfaceDataProxy::rowRolePatternChanged(const QRegExp & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 11, _a);
}

// SIGNAL 12
void QtDataVisualization::QItemModelSurfaceDataProxy::columnRolePatternChanged(const QRegExp & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 12, _a);
}

// SIGNAL 13
void QtDataVisualization::QItemModelSurfaceDataProxy::xPosRolePatternChanged(const QRegExp & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 13, _a);
}

// SIGNAL 14
void QtDataVisualization::QItemModelSurfaceDataProxy::yPosRolePatternChanged(const QRegExp & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 14, _a);
}

// SIGNAL 15
void QtDataVisualization::QItemModelSurfaceDataProxy::zPosRolePatternChanged(const QRegExp & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 15, _a);
}

// SIGNAL 16
void QtDataVisualization::QItemModelSurfaceDataProxy::rowRoleReplaceChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 16, _a);
}

// SIGNAL 17
void QtDataVisualization::QItemModelSurfaceDataProxy::columnRoleReplaceChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 17, _a);
}

// SIGNAL 18
void QtDataVisualization::QItemModelSurfaceDataProxy::xPosRoleReplaceChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 18, _a);
}

// SIGNAL 19
void QtDataVisualization::QItemModelSurfaceDataProxy::yPosRoleReplaceChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 19, _a);
}

// SIGNAL 20
void QtDataVisualization::QItemModelSurfaceDataProxy::zPosRoleReplaceChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 20, _a);
}

// SIGNAL 21
void QtDataVisualization::QItemModelSurfaceDataProxy::multiMatchBehaviorChanged(MultiMatchBehavior _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 21, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
