/****************************************************************************
** Meta object code from reading C++ file 'qabstract3daxis.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kde-qtdatavis3d/src/datavisualization/axis/qabstract3daxis.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qabstract3daxis.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QtDataVisualization__QAbstract3DAxis_t {
    QByteArrayData data[38];
    char stringdata0[523];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QtDataVisualization__QAbstract3DAxis_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QtDataVisualization__QAbstract3DAxis_t qt_meta_stringdata_QtDataVisualization__QAbstract3DAxis = {
    {
QT_MOC_LITERAL(0, 0, 36), // "QtDataVisualization::QAbstrac..."
QT_MOC_LITERAL(1, 37, 12), // "titleChanged"
QT_MOC_LITERAL(2, 50, 0), // ""
QT_MOC_LITERAL(3, 51, 8), // "newTitle"
QT_MOC_LITERAL(4, 60, 13), // "labelsChanged"
QT_MOC_LITERAL(5, 74, 18), // "orientationChanged"
QT_MOC_LITERAL(6, 93, 32), // "QAbstract3DAxis::AxisOrientation"
QT_MOC_LITERAL(7, 126, 11), // "orientation"
QT_MOC_LITERAL(8, 138, 10), // "minChanged"
QT_MOC_LITERAL(9, 149, 5), // "value"
QT_MOC_LITERAL(10, 155, 10), // "maxChanged"
QT_MOC_LITERAL(11, 166, 12), // "rangeChanged"
QT_MOC_LITERAL(12, 179, 3), // "min"
QT_MOC_LITERAL(13, 183, 3), // "max"
QT_MOC_LITERAL(14, 187, 22), // "autoAdjustRangeChanged"
QT_MOC_LITERAL(15, 210, 10), // "autoAdjust"
QT_MOC_LITERAL(16, 221, 24), // "labelAutoRotationChanged"
QT_MOC_LITERAL(17, 246, 5), // "angle"
QT_MOC_LITERAL(18, 252, 22), // "titleVisibilityChanged"
QT_MOC_LITERAL(19, 275, 7), // "visible"
QT_MOC_LITERAL(20, 283, 17), // "titleFixedChanged"
QT_MOC_LITERAL(21, 301, 5), // "fixed"
QT_MOC_LITERAL(22, 307, 5), // "title"
QT_MOC_LITERAL(23, 313, 6), // "labels"
QT_MOC_LITERAL(24, 320, 15), // "AxisOrientation"
QT_MOC_LITERAL(25, 336, 4), // "type"
QT_MOC_LITERAL(26, 341, 8), // "AxisType"
QT_MOC_LITERAL(27, 350, 15), // "autoAdjustRange"
QT_MOC_LITERAL(28, 366, 17), // "labelAutoRotation"
QT_MOC_LITERAL(29, 384, 12), // "titleVisible"
QT_MOC_LITERAL(30, 397, 10), // "titleFixed"
QT_MOC_LITERAL(31, 408, 19), // "AxisOrientationNone"
QT_MOC_LITERAL(32, 428, 16), // "AxisOrientationX"
QT_MOC_LITERAL(33, 445, 16), // "AxisOrientationY"
QT_MOC_LITERAL(34, 462, 16), // "AxisOrientationZ"
QT_MOC_LITERAL(35, 479, 12), // "AxisTypeNone"
QT_MOC_LITERAL(36, 492, 16), // "AxisTypeCategory"
QT_MOC_LITERAL(37, 509, 13) // "AxisTypeValue"

    },
    "QtDataVisualization::QAbstract3DAxis\0"
    "titleChanged\0\0newTitle\0labelsChanged\0"
    "orientationChanged\0QAbstract3DAxis::AxisOrientation\0"
    "orientation\0minChanged\0value\0maxChanged\0"
    "rangeChanged\0min\0max\0autoAdjustRangeChanged\0"
    "autoAdjust\0labelAutoRotationChanged\0"
    "angle\0titleVisibilityChanged\0visible\0"
    "titleFixedChanged\0fixed\0title\0labels\0"
    "AxisOrientation\0type\0AxisType\0"
    "autoAdjustRange\0labelAutoRotation\0"
    "titleVisible\0titleFixed\0AxisOrientationNone\0"
    "AxisOrientationX\0AxisOrientationY\0"
    "AxisOrientationZ\0AxisTypeNone\0"
    "AxisTypeCategory\0AxisTypeValue"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QtDataVisualization__QAbstract3DAxis[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      10,   14, // methods
      10,  104, // properties
       2,  154, // enums/sets
       0,    0, // constructors
       0,       // flags
      10,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   74,    2, 0x06 /* Public */,
       4,    0,   77,    2, 0x06 /* Public */,
       5,    1,   78,    2, 0x06 /* Public */,
       8,    1,   81,    2, 0x06 /* Public */,
      10,    1,   84,    2, 0x06 /* Public */,
      11,    2,   87,    2, 0x06 /* Public */,
      14,    1,   92,    2, 0x06 /* Public */,
      16,    1,   95,    2, 0x86 /* Public | MethodRevisioned */,
      18,    1,   98,    2, 0x86 /* Public | MethodRevisioned */,
      20,    1,  101,    2, 0x86 /* Public | MethodRevisioned */,

 // signals: revision
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       1,
       1,
       1,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void, QMetaType::Float,    9,
    QMetaType::Void, QMetaType::Float,    9,
    QMetaType::Void, QMetaType::Float, QMetaType::Float,   12,   13,
    QMetaType::Void, QMetaType::Bool,   15,
    QMetaType::Void, QMetaType::Float,   17,
    QMetaType::Void, QMetaType::Bool,   19,
    QMetaType::Void, QMetaType::Bool,   21,

 // properties: name, type, flags
      22, QMetaType::QString, 0x00495103,
      23, QMetaType::QStringList, 0x00495103,
       7, 0x80000000 | 24, 0x00495009,
      25, 0x80000000 | 26, 0x00095409,
      12, QMetaType::Float, 0x00495103,
      13, QMetaType::Float, 0x00495103,
      27, QMetaType::Bool, 0x00495103,
      28, QMetaType::Float, 0x00c95103,
      29, QMetaType::Bool, 0x00c95103,
      30, QMetaType::Bool, 0x00c95103,

 // properties: notify_signal_id
       0,
       1,
       2,
       0,
       3,
       4,
       6,
       7,
       8,
       9,

 // properties: revision
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       1,
       1,
       1,

 // enums: name, alias, flags, count, data
      24,   24, 0x0,    4,  164,
      26,   26, 0x0,    3,  172,

 // enum data: key, value
      31, uint(QtDataVisualization::QAbstract3DAxis::AxisOrientationNone),
      32, uint(QtDataVisualization::QAbstract3DAxis::AxisOrientationX),
      33, uint(QtDataVisualization::QAbstract3DAxis::AxisOrientationY),
      34, uint(QtDataVisualization::QAbstract3DAxis::AxisOrientationZ),
      35, uint(QtDataVisualization::QAbstract3DAxis::AxisTypeNone),
      36, uint(QtDataVisualization::QAbstract3DAxis::AxisTypeCategory),
      37, uint(QtDataVisualization::QAbstract3DAxis::AxisTypeValue),

       0        // eod
};

void QtDataVisualization::QAbstract3DAxis::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QAbstract3DAxis *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->titleChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 1: _t->labelsChanged(); break;
        case 2: _t->orientationChanged((*reinterpret_cast< QAbstract3DAxis::AxisOrientation(*)>(_a[1]))); break;
        case 3: _t->minChanged((*reinterpret_cast< float(*)>(_a[1]))); break;
        case 4: _t->maxChanged((*reinterpret_cast< float(*)>(_a[1]))); break;
        case 5: _t->rangeChanged((*reinterpret_cast< float(*)>(_a[1])),(*reinterpret_cast< float(*)>(_a[2]))); break;
        case 6: _t->autoAdjustRangeChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 7: _t->labelAutoRotationChanged((*reinterpret_cast< float(*)>(_a[1]))); break;
        case 8: _t->titleVisibilityChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 9: _t->titleFixedChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QAbstract3DAxis::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QAbstract3DAxis::titleChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QAbstract3DAxis::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QAbstract3DAxis::labelsChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QAbstract3DAxis::*)(QAbstract3DAxis::AxisOrientation );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QAbstract3DAxis::orientationChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QAbstract3DAxis::*)(float );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QAbstract3DAxis::minChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QAbstract3DAxis::*)(float );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QAbstract3DAxis::maxChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QAbstract3DAxis::*)(float , float );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QAbstract3DAxis::rangeChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QAbstract3DAxis::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QAbstract3DAxis::autoAdjustRangeChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QAbstract3DAxis::*)(float );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QAbstract3DAxis::labelAutoRotationChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QAbstract3DAxis::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QAbstract3DAxis::titleVisibilityChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QAbstract3DAxis::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QAbstract3DAxis::titleFixedChanged)) {
                *result = 9;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QAbstract3DAxis *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QString*>(_v) = _t->title(); break;
        case 1: *reinterpret_cast< QStringList*>(_v) = _t->labels(); break;
        case 2: *reinterpret_cast< AxisOrientation*>(_v) = _t->orientation(); break;
        case 3: *reinterpret_cast< AxisType*>(_v) = _t->type(); break;
        case 4: *reinterpret_cast< float*>(_v) = _t->min(); break;
        case 5: *reinterpret_cast< float*>(_v) = _t->max(); break;
        case 6: *reinterpret_cast< bool*>(_v) = _t->isAutoAdjustRange(); break;
        case 7: *reinterpret_cast< float*>(_v) = _t->labelAutoRotation(); break;
        case 8: *reinterpret_cast< bool*>(_v) = _t->isTitleVisible(); break;
        case 9: *reinterpret_cast< bool*>(_v) = _t->isTitleFixed(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QAbstract3DAxis *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setTitle(*reinterpret_cast< QString*>(_v)); break;
        case 1: _t->setLabels(*reinterpret_cast< QStringList*>(_v)); break;
        case 4: _t->setMin(*reinterpret_cast< float*>(_v)); break;
        case 5: _t->setMax(*reinterpret_cast< float*>(_v)); break;
        case 6: _t->setAutoAdjustRange(*reinterpret_cast< bool*>(_v)); break;
        case 7: _t->setLabelAutoRotation(*reinterpret_cast< float*>(_v)); break;
        case 8: _t->setTitleVisible(*reinterpret_cast< bool*>(_v)); break;
        case 9: _t->setTitleFixed(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QtDataVisualization::QAbstract3DAxis::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QtDataVisualization__QAbstract3DAxis.data,
    qt_meta_data_QtDataVisualization__QAbstract3DAxis,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QtDataVisualization::QAbstract3DAxis::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtDataVisualization::QAbstract3DAxis::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QtDataVisualization__QAbstract3DAxis.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QtDataVisualization::QAbstract3DAxis::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 10;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 10;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QtDataVisualization::QAbstract3DAxis::titleChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QtDataVisualization::QAbstract3DAxis::labelsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QtDataVisualization::QAbstract3DAxis::orientationChanged(QAbstract3DAxis::AxisOrientation _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QtDataVisualization::QAbstract3DAxis::minChanged(float _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QtDataVisualization::QAbstract3DAxis::maxChanged(float _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void QtDataVisualization::QAbstract3DAxis::rangeChanged(float _t1, float _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void QtDataVisualization::QAbstract3DAxis::autoAdjustRangeChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void QtDataVisualization::QAbstract3DAxis::labelAutoRotationChanged(float _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void QtDataVisualization::QAbstract3DAxis::titleVisibilityChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void QtDataVisualization::QAbstract3DAxis::titleFixedChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
