/****************************************************************************
** Meta object code from reading C++ file 'qabstract3dseries.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kde-qtdatavis3d/src/datavisualization/data/qabstract3dseries.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qabstract3dseries.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QtDataVisualization__QAbstract3DSeries_t {
    QByteArrayData data[68];
    char stringdata0[1021];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QtDataVisualization__QAbstract3DSeries_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QtDataVisualization__QAbstract3DSeries_t qt_meta_stringdata_QtDataVisualization__QAbstract3DSeries = {
    {
QT_MOC_LITERAL(0, 0, 38), // "QtDataVisualization::QAbstrac..."
QT_MOC_LITERAL(1, 39, 22), // "itemLabelFormatChanged"
QT_MOC_LITERAL(2, 62, 0), // ""
QT_MOC_LITERAL(3, 63, 6), // "format"
QT_MOC_LITERAL(4, 70, 17), // "visibilityChanged"
QT_MOC_LITERAL(5, 88, 7), // "visible"
QT_MOC_LITERAL(6, 96, 11), // "meshChanged"
QT_MOC_LITERAL(7, 108, 23), // "QAbstract3DSeries::Mesh"
QT_MOC_LITERAL(8, 132, 4), // "mesh"
QT_MOC_LITERAL(9, 137, 17), // "meshSmoothChanged"
QT_MOC_LITERAL(10, 155, 7), // "enabled"
QT_MOC_LITERAL(11, 163, 19), // "meshRotationChanged"
QT_MOC_LITERAL(12, 183, 8), // "rotation"
QT_MOC_LITERAL(13, 192, 22), // "userDefinedMeshChanged"
QT_MOC_LITERAL(14, 215, 8), // "fileName"
QT_MOC_LITERAL(15, 224, 17), // "colorStyleChanged"
QT_MOC_LITERAL(16, 242, 20), // "Q3DTheme::ColorStyle"
QT_MOC_LITERAL(17, 263, 5), // "style"
QT_MOC_LITERAL(18, 269, 16), // "baseColorChanged"
QT_MOC_LITERAL(19, 286, 5), // "color"
QT_MOC_LITERAL(20, 292, 19), // "baseGradientChanged"
QT_MOC_LITERAL(21, 312, 15), // "QLinearGradient"
QT_MOC_LITERAL(22, 328, 8), // "gradient"
QT_MOC_LITERAL(23, 337, 27), // "singleHighlightColorChanged"
QT_MOC_LITERAL(24, 365, 30), // "singleHighlightGradientChanged"
QT_MOC_LITERAL(25, 396, 26), // "multiHighlightColorChanged"
QT_MOC_LITERAL(26, 423, 29), // "multiHighlightGradientChanged"
QT_MOC_LITERAL(27, 453, 11), // "nameChanged"
QT_MOC_LITERAL(28, 465, 4), // "name"
QT_MOC_LITERAL(29, 470, 16), // "itemLabelChanged"
QT_MOC_LITERAL(30, 487, 5), // "label"
QT_MOC_LITERAL(31, 493, 26), // "itemLabelVisibilityChanged"
QT_MOC_LITERAL(32, 520, 19), // "setMeshAxisAndAngle"
QT_MOC_LITERAL(33, 540, 4), // "axis"
QT_MOC_LITERAL(34, 545, 5), // "angle"
QT_MOC_LITERAL(35, 551, 4), // "type"
QT_MOC_LITERAL(36, 556, 10), // "SeriesType"
QT_MOC_LITERAL(37, 567, 15), // "itemLabelFormat"
QT_MOC_LITERAL(38, 583, 4), // "Mesh"
QT_MOC_LITERAL(39, 588, 10), // "meshSmooth"
QT_MOC_LITERAL(40, 599, 12), // "meshRotation"
QT_MOC_LITERAL(41, 612, 15), // "userDefinedMesh"
QT_MOC_LITERAL(42, 628, 10), // "colorStyle"
QT_MOC_LITERAL(43, 639, 41), // "QtDataVisualization::Q3DTheme..."
QT_MOC_LITERAL(44, 681, 9), // "baseColor"
QT_MOC_LITERAL(45, 691, 12), // "baseGradient"
QT_MOC_LITERAL(46, 704, 20), // "singleHighlightColor"
QT_MOC_LITERAL(47, 725, 23), // "singleHighlightGradient"
QT_MOC_LITERAL(48, 749, 19), // "multiHighlightColor"
QT_MOC_LITERAL(49, 769, 22), // "multiHighlightGradient"
QT_MOC_LITERAL(50, 792, 9), // "itemLabel"
QT_MOC_LITERAL(51, 802, 16), // "itemLabelVisible"
QT_MOC_LITERAL(52, 819, 14), // "SeriesTypeNone"
QT_MOC_LITERAL(53, 834, 13), // "SeriesTypeBar"
QT_MOC_LITERAL(54, 848, 17), // "SeriesTypeScatter"
QT_MOC_LITERAL(55, 866, 17), // "SeriesTypeSurface"
QT_MOC_LITERAL(56, 884, 15), // "MeshUserDefined"
QT_MOC_LITERAL(57, 900, 7), // "MeshBar"
QT_MOC_LITERAL(58, 908, 8), // "MeshCube"
QT_MOC_LITERAL(59, 917, 11), // "MeshPyramid"
QT_MOC_LITERAL(60, 929, 8), // "MeshCone"
QT_MOC_LITERAL(61, 938, 12), // "MeshCylinder"
QT_MOC_LITERAL(62, 951, 12), // "MeshBevelBar"
QT_MOC_LITERAL(63, 964, 13), // "MeshBevelCube"
QT_MOC_LITERAL(64, 978, 10), // "MeshSphere"
QT_MOC_LITERAL(65, 989, 11), // "MeshMinimal"
QT_MOC_LITERAL(66, 1001, 9), // "MeshArrow"
QT_MOC_LITERAL(67, 1011, 9) // "MeshPoint"

    },
    "QtDataVisualization::QAbstract3DSeries\0"
    "itemLabelFormatChanged\0\0format\0"
    "visibilityChanged\0visible\0meshChanged\0"
    "QAbstract3DSeries::Mesh\0mesh\0"
    "meshSmoothChanged\0enabled\0meshRotationChanged\0"
    "rotation\0userDefinedMeshChanged\0"
    "fileName\0colorStyleChanged\0"
    "Q3DTheme::ColorStyle\0style\0baseColorChanged\0"
    "color\0baseGradientChanged\0QLinearGradient\0"
    "gradient\0singleHighlightColorChanged\0"
    "singleHighlightGradientChanged\0"
    "multiHighlightColorChanged\0"
    "multiHighlightGradientChanged\0nameChanged\0"
    "name\0itemLabelChanged\0label\0"
    "itemLabelVisibilityChanged\0"
    "setMeshAxisAndAngle\0axis\0angle\0type\0"
    "SeriesType\0itemLabelFormat\0Mesh\0"
    "meshSmooth\0meshRotation\0userDefinedMesh\0"
    "colorStyle\0QtDataVisualization::Q3DTheme::ColorStyle\0"
    "baseColor\0baseGradient\0singleHighlightColor\0"
    "singleHighlightGradient\0multiHighlightColor\0"
    "multiHighlightGradient\0itemLabel\0"
    "itemLabelVisible\0SeriesTypeNone\0"
    "SeriesTypeBar\0SeriesTypeScatter\0"
    "SeriesTypeSurface\0MeshUserDefined\0"
    "MeshBar\0MeshCube\0MeshPyramid\0MeshCone\0"
    "MeshCylinder\0MeshBevelBar\0MeshBevelCube\0"
    "MeshSphere\0MeshMinimal\0MeshArrow\0"
    "MeshPoint"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QtDataVisualization__QAbstract3DSeries[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      17,   14, // methods
      17,  169, // properties
       2,  254, // enums/sets
       0,    0, // constructors
       0,       // flags
      16,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  116,    2, 0x06 /* Public */,
       4,    1,  119,    2, 0x06 /* Public */,
       6,    1,  122,    2, 0x06 /* Public */,
       9,    1,  125,    2, 0x06 /* Public */,
      11,    1,  128,    2, 0x06 /* Public */,
      13,    1,  131,    2, 0x06 /* Public */,
      15,    1,  134,    2, 0x06 /* Public */,
      18,    1,  137,    2, 0x06 /* Public */,
      20,    1,  140,    2, 0x06 /* Public */,
      23,    1,  143,    2, 0x06 /* Public */,
      24,    1,  146,    2, 0x06 /* Public */,
      25,    1,  149,    2, 0x06 /* Public */,
      26,    1,  152,    2, 0x06 /* Public */,
      27,    1,  155,    2, 0x06 /* Public */,
      29,    1,  158,    2, 0x86 /* Public | MethodRevisioned */,
      31,    1,  161,    2, 0x86 /* Public | MethodRevisioned */,

 // methods: name, argc, parameters, tag, flags
      32,    2,  164,    2, 0x02 /* Public */,

 // signals: revision
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       1,
       1,

 // methods: revision
       0,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::Bool,    5,
    QMetaType::Void, 0x80000000 | 7,    8,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Void, QMetaType::QQuaternion,   12,
    QMetaType::Void, QMetaType::QString,   14,
    QMetaType::Void, 0x80000000 | 16,   17,
    QMetaType::Void, QMetaType::QColor,   19,
    QMetaType::Void, 0x80000000 | 21,   22,
    QMetaType::Void, QMetaType::QColor,   19,
    QMetaType::Void, 0x80000000 | 21,   22,
    QMetaType::Void, QMetaType::QColor,   19,
    QMetaType::Void, 0x80000000 | 21,   22,
    QMetaType::Void, QMetaType::QString,   28,
    QMetaType::Void, QMetaType::QString,   30,
    QMetaType::Void, QMetaType::Bool,    5,

 // methods: parameters
    QMetaType::Void, QMetaType::QVector3D, QMetaType::Float,   33,   34,

 // properties: name, type, flags
      35, 0x80000000 | 36, 0x00095409,
      37, QMetaType::QString, 0x00495103,
       5, QMetaType::Bool, 0x00495103,
       8, 0x80000000 | 38, 0x0049510b,
      39, QMetaType::Bool, 0x00495103,
      40, QMetaType::QQuaternion, 0x00495103,
      41, QMetaType::QString, 0x00495103,
      42, 0x80000000 | 43, 0x0049510b,
      44, QMetaType::QColor, 0x00495103,
      45, 0x80000000 | 21, 0x0049510b,
      46, QMetaType::QColor, 0x00495103,
      47, 0x80000000 | 21, 0x0049510b,
      48, QMetaType::QColor, 0x00495103,
      49, 0x80000000 | 21, 0x0049510b,
      28, QMetaType::QString, 0x00495103,
      50, QMetaType::QString, 0x00c95001,
      51, QMetaType::Bool, 0x00c95103,

 // properties: notify_signal_id
       0,
       0,
       1,
       2,
       3,
       4,
       5,
       6,
       7,
       8,
       9,
      10,
      11,
      12,
      13,
      14,
      15,

 // properties: revision
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       1,
       1,

 // enums: name, alias, flags, count, data
      36,   36, 0x0,    4,  264,
      38,   38, 0x0,   12,  272,

 // enum data: key, value
      52, uint(QtDataVisualization::QAbstract3DSeries::SeriesTypeNone),
      53, uint(QtDataVisualization::QAbstract3DSeries::SeriesTypeBar),
      54, uint(QtDataVisualization::QAbstract3DSeries::SeriesTypeScatter),
      55, uint(QtDataVisualization::QAbstract3DSeries::SeriesTypeSurface),
      56, uint(QtDataVisualization::QAbstract3DSeries::MeshUserDefined),
      57, uint(QtDataVisualization::QAbstract3DSeries::MeshBar),
      58, uint(QtDataVisualization::QAbstract3DSeries::MeshCube),
      59, uint(QtDataVisualization::QAbstract3DSeries::MeshPyramid),
      60, uint(QtDataVisualization::QAbstract3DSeries::MeshCone),
      61, uint(QtDataVisualization::QAbstract3DSeries::MeshCylinder),
      62, uint(QtDataVisualization::QAbstract3DSeries::MeshBevelBar),
      63, uint(QtDataVisualization::QAbstract3DSeries::MeshBevelCube),
      64, uint(QtDataVisualization::QAbstract3DSeries::MeshSphere),
      65, uint(QtDataVisualization::QAbstract3DSeries::MeshMinimal),
      66, uint(QtDataVisualization::QAbstract3DSeries::MeshArrow),
      67, uint(QtDataVisualization::QAbstract3DSeries::MeshPoint),

       0        // eod
};

void QtDataVisualization::QAbstract3DSeries::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QAbstract3DSeries *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->itemLabelFormatChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 1: _t->visibilityChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 2: _t->meshChanged((*reinterpret_cast< QAbstract3DSeries::Mesh(*)>(_a[1]))); break;
        case 3: _t->meshSmoothChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 4: _t->meshRotationChanged((*reinterpret_cast< const QQuaternion(*)>(_a[1]))); break;
        case 5: _t->userDefinedMeshChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 6: _t->colorStyleChanged((*reinterpret_cast< Q3DTheme::ColorStyle(*)>(_a[1]))); break;
        case 7: _t->baseColorChanged((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 8: _t->baseGradientChanged((*reinterpret_cast< const QLinearGradient(*)>(_a[1]))); break;
        case 9: _t->singleHighlightColorChanged((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 10: _t->singleHighlightGradientChanged((*reinterpret_cast< const QLinearGradient(*)>(_a[1]))); break;
        case 11: _t->multiHighlightColorChanged((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 12: _t->multiHighlightGradientChanged((*reinterpret_cast< const QLinearGradient(*)>(_a[1]))); break;
        case 13: _t->nameChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 14: _t->itemLabelChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 15: _t->itemLabelVisibilityChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 16: _t->setMeshAxisAndAngle((*reinterpret_cast< const QVector3D(*)>(_a[1])),(*reinterpret_cast< float(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QAbstract3DSeries::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QAbstract3DSeries::itemLabelFormatChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QAbstract3DSeries::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QAbstract3DSeries::visibilityChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QAbstract3DSeries::*)(QAbstract3DSeries::Mesh );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QAbstract3DSeries::meshChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QAbstract3DSeries::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QAbstract3DSeries::meshSmoothChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QAbstract3DSeries::*)(const QQuaternion & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QAbstract3DSeries::meshRotationChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QAbstract3DSeries::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QAbstract3DSeries::userDefinedMeshChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QAbstract3DSeries::*)(Q3DTheme::ColorStyle );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QAbstract3DSeries::colorStyleChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QAbstract3DSeries::*)(const QColor & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QAbstract3DSeries::baseColorChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QAbstract3DSeries::*)(const QLinearGradient & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QAbstract3DSeries::baseGradientChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QAbstract3DSeries::*)(const QColor & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QAbstract3DSeries::singleHighlightColorChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (QAbstract3DSeries::*)(const QLinearGradient & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QAbstract3DSeries::singleHighlightGradientChanged)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (QAbstract3DSeries::*)(const QColor & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QAbstract3DSeries::multiHighlightColorChanged)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (QAbstract3DSeries::*)(const QLinearGradient & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QAbstract3DSeries::multiHighlightGradientChanged)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (QAbstract3DSeries::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QAbstract3DSeries::nameChanged)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (QAbstract3DSeries::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QAbstract3DSeries::itemLabelChanged)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (QAbstract3DSeries::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QAbstract3DSeries::itemLabelVisibilityChanged)) {
                *result = 15;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QAbstract3DSeries *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< SeriesType*>(_v) = _t->type(); break;
        case 1: *reinterpret_cast< QString*>(_v) = _t->itemLabelFormat(); break;
        case 2: *reinterpret_cast< bool*>(_v) = _t->isVisible(); break;
        case 3: *reinterpret_cast< Mesh*>(_v) = _t->mesh(); break;
        case 4: *reinterpret_cast< bool*>(_v) = _t->isMeshSmooth(); break;
        case 5: *reinterpret_cast< QQuaternion*>(_v) = _t->meshRotation(); break;
        case 6: *reinterpret_cast< QString*>(_v) = _t->userDefinedMesh(); break;
        case 7: *reinterpret_cast< QtDataVisualization::Q3DTheme::ColorStyle*>(_v) = _t->colorStyle(); break;
        case 8: *reinterpret_cast< QColor*>(_v) = _t->baseColor(); break;
        case 9: *reinterpret_cast< QLinearGradient*>(_v) = _t->baseGradient(); break;
        case 10: *reinterpret_cast< QColor*>(_v) = _t->singleHighlightColor(); break;
        case 11: *reinterpret_cast< QLinearGradient*>(_v) = _t->singleHighlightGradient(); break;
        case 12: *reinterpret_cast< QColor*>(_v) = _t->multiHighlightColor(); break;
        case 13: *reinterpret_cast< QLinearGradient*>(_v) = _t->multiHighlightGradient(); break;
        case 14: *reinterpret_cast< QString*>(_v) = _t->name(); break;
        case 15: *reinterpret_cast< QString*>(_v) = _t->itemLabel(); break;
        case 16: *reinterpret_cast< bool*>(_v) = _t->isItemLabelVisible(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QAbstract3DSeries *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setItemLabelFormat(*reinterpret_cast< QString*>(_v)); break;
        case 2: _t->setVisible(*reinterpret_cast< bool*>(_v)); break;
        case 3: _t->setMesh(*reinterpret_cast< Mesh*>(_v)); break;
        case 4: _t->setMeshSmooth(*reinterpret_cast< bool*>(_v)); break;
        case 5: _t->setMeshRotation(*reinterpret_cast< QQuaternion*>(_v)); break;
        case 6: _t->setUserDefinedMesh(*reinterpret_cast< QString*>(_v)); break;
        case 7: _t->setColorStyle(*reinterpret_cast< QtDataVisualization::Q3DTheme::ColorStyle*>(_v)); break;
        case 8: _t->setBaseColor(*reinterpret_cast< QColor*>(_v)); break;
        case 9: _t->setBaseGradient(*reinterpret_cast< QLinearGradient*>(_v)); break;
        case 10: _t->setSingleHighlightColor(*reinterpret_cast< QColor*>(_v)); break;
        case 11: _t->setSingleHighlightGradient(*reinterpret_cast< QLinearGradient*>(_v)); break;
        case 12: _t->setMultiHighlightColor(*reinterpret_cast< QColor*>(_v)); break;
        case 13: _t->setMultiHighlightGradient(*reinterpret_cast< QLinearGradient*>(_v)); break;
        case 14: _t->setName(*reinterpret_cast< QString*>(_v)); break;
        case 16: _t->setItemLabelVisible(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

static const QMetaObject::SuperData qt_meta_extradata_QtDataVisualization__QAbstract3DSeries[] = {
    QMetaObject::SuperData::link<QtDataVisualization::Q3DTheme::staticMetaObject>(),
    nullptr
};

QT_INIT_METAOBJECT const QMetaObject QtDataVisualization::QAbstract3DSeries::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QtDataVisualization__QAbstract3DSeries.data,
    qt_meta_data_QtDataVisualization__QAbstract3DSeries,
    qt_static_metacall,
    qt_meta_extradata_QtDataVisualization__QAbstract3DSeries,
    nullptr
} };


const QMetaObject *QtDataVisualization::QAbstract3DSeries::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtDataVisualization::QAbstract3DSeries::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QtDataVisualization__QAbstract3DSeries.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QtDataVisualization::QAbstract3DSeries::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 17;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 17;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 17;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 17;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 17;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 17;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QtDataVisualization::QAbstract3DSeries::itemLabelFormatChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QtDataVisualization::QAbstract3DSeries::visibilityChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QtDataVisualization::QAbstract3DSeries::meshChanged(QAbstract3DSeries::Mesh _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QtDataVisualization::QAbstract3DSeries::meshSmoothChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QtDataVisualization::QAbstract3DSeries::meshRotationChanged(const QQuaternion & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void QtDataVisualization::QAbstract3DSeries::userDefinedMeshChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void QtDataVisualization::QAbstract3DSeries::colorStyleChanged(Q3DTheme::ColorStyle _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void QtDataVisualization::QAbstract3DSeries::baseColorChanged(const QColor & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void QtDataVisualization::QAbstract3DSeries::baseGradientChanged(const QLinearGradient & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void QtDataVisualization::QAbstract3DSeries::singleHighlightColorChanged(const QColor & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void QtDataVisualization::QAbstract3DSeries::singleHighlightGradientChanged(const QLinearGradient & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 10, _a);
}

// SIGNAL 11
void QtDataVisualization::QAbstract3DSeries::multiHighlightColorChanged(const QColor & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 11, _a);
}

// SIGNAL 12
void QtDataVisualization::QAbstract3DSeries::multiHighlightGradientChanged(const QLinearGradient & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 12, _a);
}

// SIGNAL 13
void QtDataVisualization::QAbstract3DSeries::nameChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 13, _a);
}

// SIGNAL 14
void QtDataVisualization::QAbstract3DSeries::itemLabelChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 14, _a);
}

// SIGNAL 15
void QtDataVisualization::QAbstract3DSeries::itemLabelVisibilityChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 15, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
