/****************************************************************************
** Meta object code from reading C++ file 'abstractdeclarative_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kde-qtdatavis3d/src/datavisualizationqml2/abstractdeclarative_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'abstractdeclarative_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QtDataVisualization__AbstractDeclarative_t {
    QByteArrayData data[124];
    char stringdata0[2085];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QtDataVisualization__AbstractDeclarative_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QtDataVisualization__AbstractDeclarative_t qt_meta_stringdata_QtDataVisualization__AbstractDeclarative = {
    {
QT_MOC_LITERAL(0, 0, 40), // "QtDataVisualization::Abstract..."
QT_MOC_LITERAL(1, 41, 20), // "selectionModeChanged"
QT_MOC_LITERAL(2, 62, 0), // ""
QT_MOC_LITERAL(3, 63, 35), // "AbstractDeclarative::Selectio..."
QT_MOC_LITERAL(4, 99, 4), // "mode"
QT_MOC_LITERAL(5, 104, 20), // "shadowQualityChanged"
QT_MOC_LITERAL(6, 125, 34), // "AbstractDeclarative::ShadowQu..."
QT_MOC_LITERAL(7, 160, 7), // "quality"
QT_MOC_LITERAL(8, 168, 23), // "shadowsSupportedChanged"
QT_MOC_LITERAL(9, 192, 9), // "supported"
QT_MOC_LITERAL(10, 202, 18), // "msaaSamplesChanged"
QT_MOC_LITERAL(11, 221, 7), // "samples"
QT_MOC_LITERAL(12, 229, 12), // "sceneChanged"
QT_MOC_LITERAL(13, 242, 9), // "Q3DScene*"
QT_MOC_LITERAL(14, 252, 5), // "scene"
QT_MOC_LITERAL(15, 258, 19), // "inputHandlerChanged"
QT_MOC_LITERAL(16, 278, 24), // "QAbstract3DInputHandler*"
QT_MOC_LITERAL(17, 303, 12), // "inputHandler"
QT_MOC_LITERAL(18, 316, 12), // "themeChanged"
QT_MOC_LITERAL(19, 329, 9), // "Q3DTheme*"
QT_MOC_LITERAL(20, 339, 5), // "theme"
QT_MOC_LITERAL(21, 345, 20), // "renderingModeChanged"
QT_MOC_LITERAL(22, 366, 34), // "AbstractDeclarative::Renderin..."
QT_MOC_LITERAL(23, 401, 17), // "measureFpsChanged"
QT_MOC_LITERAL(24, 419, 7), // "enabled"
QT_MOC_LITERAL(25, 427, 17), // "currentFpsChanged"
QT_MOC_LITERAL(26, 445, 3), // "fps"
QT_MOC_LITERAL(27, 449, 22), // "selectedElementChanged"
QT_MOC_LITERAL(28, 472, 32), // "AbstractDeclarative::ElementType"
QT_MOC_LITERAL(29, 505, 4), // "type"
QT_MOC_LITERAL(30, 510, 22), // "orthoProjectionChanged"
QT_MOC_LITERAL(31, 533, 18), // "aspectRatioChanged"
QT_MOC_LITERAL(32, 552, 5), // "ratio"
QT_MOC_LITERAL(33, 558, 24), // "optimizationHintsChanged"
QT_MOC_LITERAL(34, 583, 38), // "AbstractDeclarative::Optimiza..."
QT_MOC_LITERAL(35, 622, 5), // "hints"
QT_MOC_LITERAL(36, 628, 12), // "polarChanged"
QT_MOC_LITERAL(37, 641, 24), // "radialLabelOffsetChanged"
QT_MOC_LITERAL(38, 666, 6), // "offset"
QT_MOC_LITERAL(39, 673, 28), // "horizontalAspectRatioChanged"
QT_MOC_LITERAL(40, 702, 17), // "reflectionChanged"
QT_MOC_LITERAL(41, 720, 19), // "reflectivityChanged"
QT_MOC_LITERAL(42, 740, 12), // "reflectivity"
QT_MOC_LITERAL(43, 753, 13), // "localeChanged"
QT_MOC_LITERAL(44, 767, 6), // "locale"
QT_MOC_LITERAL(45, 774, 27), // "queriedGraphPositionChanged"
QT_MOC_LITERAL(46, 802, 4), // "data"
QT_MOC_LITERAL(47, 807, 13), // "marginChanged"
QT_MOC_LITERAL(48, 821, 6), // "margin"
QT_MOC_LITERAL(49, 828, 18), // "handleAxisXChanged"
QT_MOC_LITERAL(50, 847, 16), // "QAbstract3DAxis*"
QT_MOC_LITERAL(51, 864, 4), // "axis"
QT_MOC_LITERAL(52, 869, 18), // "handleAxisYChanged"
QT_MOC_LITERAL(53, 888, 18), // "handleAxisZChanged"
QT_MOC_LITERAL(54, 907, 15), // "windowDestroyed"
QT_MOC_LITERAL(55, 923, 3), // "obj"
QT_MOC_LITERAL(56, 927, 14), // "destroyContext"
QT_MOC_LITERAL(57, 942, 14), // "clearSelection"
QT_MOC_LITERAL(58, 957, 13), // "addCustomItem"
QT_MOC_LITERAL(59, 971, 14), // "QCustom3DItem*"
QT_MOC_LITERAL(60, 986, 4), // "item"
QT_MOC_LITERAL(61, 991, 17), // "removeCustomItems"
QT_MOC_LITERAL(62, 1009, 16), // "removeCustomItem"
QT_MOC_LITERAL(63, 1026, 18), // "removeCustomItemAt"
QT_MOC_LITERAL(64, 1045, 8), // "position"
QT_MOC_LITERAL(65, 1054, 17), // "releaseCustomItem"
QT_MOC_LITERAL(66, 1072, 18), // "selectedLabelIndex"
QT_MOC_LITERAL(67, 1091, 12), // "selectedAxis"
QT_MOC_LITERAL(68, 1104, 23), // "selectedCustomItemIndex"
QT_MOC_LITERAL(69, 1128, 18), // "selectedCustomItem"
QT_MOC_LITERAL(70, 1147, 13), // "selectionMode"
QT_MOC_LITERAL(71, 1161, 14), // "SelectionFlags"
QT_MOC_LITERAL(72, 1176, 13), // "shadowQuality"
QT_MOC_LITERAL(73, 1190, 13), // "ShadowQuality"
QT_MOC_LITERAL(74, 1204, 16), // "shadowsSupported"
QT_MOC_LITERAL(75, 1221, 11), // "msaaSamples"
QT_MOC_LITERAL(76, 1233, 19), // "Declarative3DScene*"
QT_MOC_LITERAL(77, 1253, 13), // "renderingMode"
QT_MOC_LITERAL(78, 1267, 13), // "RenderingMode"
QT_MOC_LITERAL(79, 1281, 10), // "measureFps"
QT_MOC_LITERAL(80, 1292, 10), // "currentFps"
QT_MOC_LITERAL(81, 1303, 14), // "customItemList"
QT_MOC_LITERAL(82, 1318, 31), // "QQmlListProperty<QCustom3DItem>"
QT_MOC_LITERAL(83, 1350, 15), // "orthoProjection"
QT_MOC_LITERAL(84, 1366, 15), // "selectedElement"
QT_MOC_LITERAL(85, 1382, 11), // "ElementType"
QT_MOC_LITERAL(86, 1394, 11), // "aspectRatio"
QT_MOC_LITERAL(87, 1406, 17), // "optimizationHints"
QT_MOC_LITERAL(88, 1424, 17), // "OptimizationHints"
QT_MOC_LITERAL(89, 1442, 5), // "polar"
QT_MOC_LITERAL(90, 1448, 17), // "radialLabelOffset"
QT_MOC_LITERAL(91, 1466, 21), // "horizontalAspectRatio"
QT_MOC_LITERAL(92, 1488, 10), // "reflection"
QT_MOC_LITERAL(93, 1499, 20), // "queriedGraphPosition"
QT_MOC_LITERAL(94, 1520, 13), // "SelectionFlag"
QT_MOC_LITERAL(95, 1534, 13), // "SelectionNone"
QT_MOC_LITERAL(96, 1548, 13), // "SelectionItem"
QT_MOC_LITERAL(97, 1562, 12), // "SelectionRow"
QT_MOC_LITERAL(98, 1575, 19), // "SelectionItemAndRow"
QT_MOC_LITERAL(99, 1595, 15), // "SelectionColumn"
QT_MOC_LITERAL(100, 1611, 22), // "SelectionItemAndColumn"
QT_MOC_LITERAL(101, 1634, 21), // "SelectionRowAndColumn"
QT_MOC_LITERAL(102, 1656, 25), // "SelectionItemRowAndColumn"
QT_MOC_LITERAL(103, 1682, 14), // "SelectionSlice"
QT_MOC_LITERAL(104, 1697, 20), // "SelectionMultiSeries"
QT_MOC_LITERAL(105, 1718, 17), // "ShadowQualityNone"
QT_MOC_LITERAL(106, 1736, 16), // "ShadowQualityLow"
QT_MOC_LITERAL(107, 1753, 19), // "ShadowQualityMedium"
QT_MOC_LITERAL(108, 1773, 17), // "ShadowQualityHigh"
QT_MOC_LITERAL(109, 1791, 20), // "ShadowQualitySoftLow"
QT_MOC_LITERAL(110, 1812, 23), // "ShadowQualitySoftMedium"
QT_MOC_LITERAL(111, 1836, 21), // "ShadowQualitySoftHigh"
QT_MOC_LITERAL(112, 1858, 11), // "ElementNone"
QT_MOC_LITERAL(113, 1870, 13), // "ElementSeries"
QT_MOC_LITERAL(114, 1884, 17), // "ElementAxisXLabel"
QT_MOC_LITERAL(115, 1902, 17), // "ElementAxisYLabel"
QT_MOC_LITERAL(116, 1920, 17), // "ElementAxisZLabel"
QT_MOC_LITERAL(117, 1938, 17), // "ElementCustomItem"
QT_MOC_LITERAL(118, 1956, 24), // "RenderDirectToBackground"
QT_MOC_LITERAL(119, 1981, 32), // "RenderDirectToBackground_NoClear"
QT_MOC_LITERAL(120, 2014, 14), // "RenderIndirect"
QT_MOC_LITERAL(121, 2029, 16), // "OptimizationHint"
QT_MOC_LITERAL(122, 2046, 19), // "OptimizationDefault"
QT_MOC_LITERAL(123, 2066, 18) // "OptimizationStatic"

    },
    "QtDataVisualization::AbstractDeclarative\0"
    "selectionModeChanged\0\0"
    "AbstractDeclarative::SelectionFlags\0"
    "mode\0shadowQualityChanged\0"
    "AbstractDeclarative::ShadowQuality\0"
    "quality\0shadowsSupportedChanged\0"
    "supported\0msaaSamplesChanged\0samples\0"
    "sceneChanged\0Q3DScene*\0scene\0"
    "inputHandlerChanged\0QAbstract3DInputHandler*\0"
    "inputHandler\0themeChanged\0Q3DTheme*\0"
    "theme\0renderingModeChanged\0"
    "AbstractDeclarative::RenderingMode\0"
    "measureFpsChanged\0enabled\0currentFpsChanged\0"
    "fps\0selectedElementChanged\0"
    "AbstractDeclarative::ElementType\0type\0"
    "orthoProjectionChanged\0aspectRatioChanged\0"
    "ratio\0optimizationHintsChanged\0"
    "AbstractDeclarative::OptimizationHints\0"
    "hints\0polarChanged\0radialLabelOffsetChanged\0"
    "offset\0horizontalAspectRatioChanged\0"
    "reflectionChanged\0reflectivityChanged\0"
    "reflectivity\0localeChanged\0locale\0"
    "queriedGraphPositionChanged\0data\0"
    "marginChanged\0margin\0handleAxisXChanged\0"
    "QAbstract3DAxis*\0axis\0handleAxisYChanged\0"
    "handleAxisZChanged\0windowDestroyed\0"
    "obj\0destroyContext\0clearSelection\0"
    "addCustomItem\0QCustom3DItem*\0item\0"
    "removeCustomItems\0removeCustomItem\0"
    "removeCustomItemAt\0position\0"
    "releaseCustomItem\0selectedLabelIndex\0"
    "selectedAxis\0selectedCustomItemIndex\0"
    "selectedCustomItem\0selectionMode\0"
    "SelectionFlags\0shadowQuality\0ShadowQuality\0"
    "shadowsSupported\0msaaSamples\0"
    "Declarative3DScene*\0renderingMode\0"
    "RenderingMode\0measureFps\0currentFps\0"
    "customItemList\0QQmlListProperty<QCustom3DItem>\0"
    "orthoProjection\0selectedElement\0"
    "ElementType\0aspectRatio\0optimizationHints\0"
    "OptimizationHints\0polar\0radialLabelOffset\0"
    "horizontalAspectRatio\0reflection\0"
    "queriedGraphPosition\0SelectionFlag\0"
    "SelectionNone\0SelectionItem\0SelectionRow\0"
    "SelectionItemAndRow\0SelectionColumn\0"
    "SelectionItemAndColumn\0SelectionRowAndColumn\0"
    "SelectionItemRowAndColumn\0SelectionSlice\0"
    "SelectionMultiSeries\0ShadowQualityNone\0"
    "ShadowQualityLow\0ShadowQualityMedium\0"
    "ShadowQualityHigh\0ShadowQualitySoftLow\0"
    "ShadowQualitySoftMedium\0ShadowQualitySoftHigh\0"
    "ElementNone\0ElementSeries\0ElementAxisXLabel\0"
    "ElementAxisYLabel\0ElementAxisZLabel\0"
    "ElementCustomItem\0RenderDirectToBackground\0"
    "RenderDirectToBackground_NoClear\0"
    "RenderIndirect\0OptimizationHint\0"
    "OptimizationDefault\0OptimizationStatic"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QtDataVisualization__AbstractDeclarative[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      37,   14, // methods
      23,  333, // properties
       7,  448, // enums/sets
       0,    0, // constructors
       0,       // flags
      22,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  236,    2, 0x06 /* Public */,
       5,    1,  239,    2, 0x06 /* Public */,
       8,    1,  242,    2, 0x06 /* Public */,
      10,    1,  245,    2, 0x06 /* Public */,
      12,    1,  248,    2, 0x06 /* Public */,
      15,    1,  251,    2, 0x06 /* Public */,
      18,    1,  254,    2, 0x06 /* Public */,
      21,    1,  257,    2, 0x06 /* Public */,
      23,    1,  260,    2, 0x86 /* Public | MethodRevisioned */,
      25,    1,  263,    2, 0x86 /* Public | MethodRevisioned */,
      27,    1,  266,    2, 0x86 /* Public | MethodRevisioned */,
      30,    1,  269,    2, 0x86 /* Public | MethodRevisioned */,
      31,    1,  272,    2, 0x86 /* Public | MethodRevisioned */,
      33,    1,  275,    2, 0x86 /* Public | MethodRevisioned */,
      36,    1,  278,    2, 0x86 /* Public | MethodRevisioned */,
      37,    1,  281,    2, 0x86 /* Public | MethodRevisioned */,
      39,    1,  284,    2, 0x86 /* Public | MethodRevisioned */,
      40,    1,  287,    2, 0x86 /* Public | MethodRevisioned */,
      41,    1,  290,    2, 0x86 /* Public | MethodRevisioned */,
      43,    1,  293,    2, 0x86 /* Public | MethodRevisioned */,
      45,    1,  296,    2, 0x86 /* Public | MethodRevisioned */,
      47,    1,  299,    2, 0x86 /* Public | MethodRevisioned */,

 // slots: name, argc, parameters, tag, flags
      49,    1,  302,    2, 0x0a /* Public */,
      52,    1,  305,    2, 0x0a /* Public */,
      53,    1,  308,    2, 0x0a /* Public */,
      54,    1,  311,    2, 0x0a /* Public */,
      56,    0,  314,    2, 0x0a /* Public */,

 // methods: name, argc, parameters, tag, flags
      57,    0,  315,    2, 0x02 /* Public */,
      58,    1,  316,    2, 0x82 /* Public | MethodRevisioned */,
      61,    0,  319,    2, 0x82 /* Public | MethodRevisioned */,
      62,    1,  320,    2, 0x82 /* Public | MethodRevisioned */,
      63,    1,  323,    2, 0x82 /* Public | MethodRevisioned */,
      65,    1,  326,    2, 0x82 /* Public | MethodRevisioned */,
      66,    0,  329,    2, 0x82 /* Public | MethodRevisioned */,
      67,    0,  330,    2, 0x82 /* Public | MethodRevisioned */,
      68,    0,  331,    2, 0x82 /* Public | MethodRevisioned */,
      69,    0,  332,    2, 0x82 /* Public | MethodRevisioned */,

 // signals: revision
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       1,
       1,
       1,
       1,
       1,
       1,
       2,
       2,
       2,
       2,
       2,
       2,
       2,
       2,

 // slots: revision
       0,
       0,
       0,
       0,
       0,

 // methods: revision
       0,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void, QMetaType::Bool,    9,
    QMetaType::Void, QMetaType::Int,   11,
    QMetaType::Void, 0x80000000 | 13,   14,
    QMetaType::Void, 0x80000000 | 16,   17,
    QMetaType::Void, 0x80000000 | 19,   20,
    QMetaType::Void, 0x80000000 | 22,    4,
    QMetaType::Void, QMetaType::Bool,   24,
    QMetaType::Void, QMetaType::QReal,   26,
    QMetaType::Void, 0x80000000 | 28,   29,
    QMetaType::Void, QMetaType::Bool,   24,
    QMetaType::Void, QMetaType::QReal,   32,
    QMetaType::Void, 0x80000000 | 34,   35,
    QMetaType::Void, QMetaType::Bool,   24,
    QMetaType::Void, QMetaType::Float,   38,
    QMetaType::Void, QMetaType::QReal,   32,
    QMetaType::Void, QMetaType::Bool,   24,
    QMetaType::Void, QMetaType::QReal,   42,
    QMetaType::Void, QMetaType::QLocale,   44,
    QMetaType::Void, QMetaType::QVector3D,   46,
    QMetaType::Void, QMetaType::QReal,   48,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 50,   51,
    QMetaType::Void, 0x80000000 | 50,   51,
    QMetaType::Void, 0x80000000 | 50,   51,
    QMetaType::Void, QMetaType::QObjectStar,   55,
    QMetaType::Void,

 // methods: parameters
    QMetaType::Void,
    QMetaType::Int, 0x80000000 | 59,   60,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 59,   60,
    QMetaType::Void, QMetaType::QVector3D,   64,
    QMetaType::Void, 0x80000000 | 59,   60,
    QMetaType::Int,
    0x80000000 | 50,
    QMetaType::Int,
    0x80000000 | 59,

 // properties: name, type, flags
      70, 0x80000000 | 71, 0x0049510b,
      72, 0x80000000 | 73, 0x0049510b,
      74, QMetaType::Bool, 0x00495001,
      75, QMetaType::Int, 0x00495103,
      14, 0x80000000 | 76, 0x00495009,
      17, 0x80000000 | 16, 0x0049510b,
      20, 0x80000000 | 19, 0x0049510b,
      77, 0x80000000 | 78, 0x0049510b,
      79, QMetaType::Bool, 0x00c95103,
      80, QMetaType::QReal, 0x00c95001,
      81, 0x80000000 | 82, 0x00895009,
      83, QMetaType::Bool, 0x00c95103,
      84, 0x80000000 | 85, 0x00c95009,
      86, QMetaType::QReal, 0x00c95103,
      87, 0x80000000 | 88, 0x00c9510b,
      89, QMetaType::Bool, 0x00c95103,
      90, QMetaType::Float, 0x00c95103,
      91, QMetaType::QReal, 0x00c95103,
      92, QMetaType::Bool, 0x00c95103,
      42, QMetaType::QReal, 0x00c95103,
      44, QMetaType::QLocale, 0x00c95103,
      93, QMetaType::QVector3D, 0x00c95001,
      48, QMetaType::QReal, 0x00c95103,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,
       4,
       5,
       6,
       7,
       8,
       9,
       0,
      11,
      10,
      12,
      13,
      14,
      15,
      16,
      17,
      18,
      19,
      20,
      21,

 // properties: revision
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       2,
       2,
       2,
       2,
       2,
       2,
       2,
       2,

 // enums: name, alias, flags, count, data
      94,   94, 0x1,   10,  483,
      71,   94, 0x1,   10,  503,
      73,   73, 0x0,    7,  523,
      85,   85, 0x0,    6,  537,
      78,   78, 0x0,    3,  549,
     121,  121, 0x1,    2,  555,
      88,  121, 0x1,    2,  559,

 // enum data: key, value
      95, uint(QtDataVisualization::AbstractDeclarative::SelectionNone),
      96, uint(QtDataVisualization::AbstractDeclarative::SelectionItem),
      97, uint(QtDataVisualization::AbstractDeclarative::SelectionRow),
      98, uint(QtDataVisualization::AbstractDeclarative::SelectionItemAndRow),
      99, uint(QtDataVisualization::AbstractDeclarative::SelectionColumn),
     100, uint(QtDataVisualization::AbstractDeclarative::SelectionItemAndColumn),
     101, uint(QtDataVisualization::AbstractDeclarative::SelectionRowAndColumn),
     102, uint(QtDataVisualization::AbstractDeclarative::SelectionItemRowAndColumn),
     103, uint(QtDataVisualization::AbstractDeclarative::SelectionSlice),
     104, uint(QtDataVisualization::AbstractDeclarative::SelectionMultiSeries),
      95, uint(QtDataVisualization::AbstractDeclarative::SelectionNone),
      96, uint(QtDataVisualization::AbstractDeclarative::SelectionItem),
      97, uint(QtDataVisualization::AbstractDeclarative::SelectionRow),
      98, uint(QtDataVisualization::AbstractDeclarative::SelectionItemAndRow),
      99, uint(QtDataVisualization::AbstractDeclarative::SelectionColumn),
     100, uint(QtDataVisualization::AbstractDeclarative::SelectionItemAndColumn),
     101, uint(QtDataVisualization::AbstractDeclarative::SelectionRowAndColumn),
     102, uint(QtDataVisualization::AbstractDeclarative::SelectionItemRowAndColumn),
     103, uint(QtDataVisualization::AbstractDeclarative::SelectionSlice),
     104, uint(QtDataVisualization::AbstractDeclarative::SelectionMultiSeries),
     105, uint(QtDataVisualization::AbstractDeclarative::ShadowQualityNone),
     106, uint(QtDataVisualization::AbstractDeclarative::ShadowQualityLow),
     107, uint(QtDataVisualization::AbstractDeclarative::ShadowQualityMedium),
     108, uint(QtDataVisualization::AbstractDeclarative::ShadowQualityHigh),
     109, uint(QtDataVisualization::AbstractDeclarative::ShadowQualitySoftLow),
     110, uint(QtDataVisualization::AbstractDeclarative::ShadowQualitySoftMedium),
     111, uint(QtDataVisualization::AbstractDeclarative::ShadowQualitySoftHigh),
     112, uint(QtDataVisualization::AbstractDeclarative::ElementNone),
     113, uint(QtDataVisualization::AbstractDeclarative::ElementSeries),
     114, uint(QtDataVisualization::AbstractDeclarative::ElementAxisXLabel),
     115, uint(QtDataVisualization::AbstractDeclarative::ElementAxisYLabel),
     116, uint(QtDataVisualization::AbstractDeclarative::ElementAxisZLabel),
     117, uint(QtDataVisualization::AbstractDeclarative::ElementCustomItem),
     118, uint(QtDataVisualization::AbstractDeclarative::RenderDirectToBackground),
     119, uint(QtDataVisualization::AbstractDeclarative::RenderDirectToBackground_NoClear),
     120, uint(QtDataVisualization::AbstractDeclarative::RenderIndirect),
     122, uint(QtDataVisualization::AbstractDeclarative::OptimizationDefault),
     123, uint(QtDataVisualization::AbstractDeclarative::OptimizationStatic),
     122, uint(QtDataVisualization::AbstractDeclarative::OptimizationDefault),
     123, uint(QtDataVisualization::AbstractDeclarative::OptimizationStatic),

       0        // eod
};

void QtDataVisualization::AbstractDeclarative::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<AbstractDeclarative *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->selectionModeChanged((*reinterpret_cast< AbstractDeclarative::SelectionFlags(*)>(_a[1]))); break;
        case 1: _t->shadowQualityChanged((*reinterpret_cast< AbstractDeclarative::ShadowQuality(*)>(_a[1]))); break;
        case 2: _t->shadowsSupportedChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->msaaSamplesChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->sceneChanged((*reinterpret_cast< Q3DScene*(*)>(_a[1]))); break;
        case 5: _t->inputHandlerChanged((*reinterpret_cast< QAbstract3DInputHandler*(*)>(_a[1]))); break;
        case 6: _t->themeChanged((*reinterpret_cast< Q3DTheme*(*)>(_a[1]))); break;
        case 7: _t->renderingModeChanged((*reinterpret_cast< AbstractDeclarative::RenderingMode(*)>(_a[1]))); break;
        case 8: _t->measureFpsChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 9: _t->currentFpsChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 10: _t->selectedElementChanged((*reinterpret_cast< AbstractDeclarative::ElementType(*)>(_a[1]))); break;
        case 11: _t->orthoProjectionChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 12: _t->aspectRatioChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 13: _t->optimizationHintsChanged((*reinterpret_cast< AbstractDeclarative::OptimizationHints(*)>(_a[1]))); break;
        case 14: _t->polarChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 15: _t->radialLabelOffsetChanged((*reinterpret_cast< float(*)>(_a[1]))); break;
        case 16: _t->horizontalAspectRatioChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 17: _t->reflectionChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 18: _t->reflectivityChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 19: _t->localeChanged((*reinterpret_cast< const QLocale(*)>(_a[1]))); break;
        case 20: _t->queriedGraphPositionChanged((*reinterpret_cast< const QVector3D(*)>(_a[1]))); break;
        case 21: _t->marginChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 22: _t->handleAxisXChanged((*reinterpret_cast< QAbstract3DAxis*(*)>(_a[1]))); break;
        case 23: _t->handleAxisYChanged((*reinterpret_cast< QAbstract3DAxis*(*)>(_a[1]))); break;
        case 24: _t->handleAxisZChanged((*reinterpret_cast< QAbstract3DAxis*(*)>(_a[1]))); break;
        case 25: _t->windowDestroyed((*reinterpret_cast< QObject*(*)>(_a[1]))); break;
        case 26: _t->destroyContext(); break;
        case 27: _t->clearSelection(); break;
        case 28: { int _r = _t->addCustomItem((*reinterpret_cast< QCustom3DItem*(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 29: _t->removeCustomItems(); break;
        case 30: _t->removeCustomItem((*reinterpret_cast< QCustom3DItem*(*)>(_a[1]))); break;
        case 31: _t->removeCustomItemAt((*reinterpret_cast< const QVector3D(*)>(_a[1]))); break;
        case 32: _t->releaseCustomItem((*reinterpret_cast< QCustom3DItem*(*)>(_a[1]))); break;
        case 33: { int _r = _t->selectedLabelIndex();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 34: { QAbstract3DAxis* _r = _t->selectedAxis();
            if (_a[0]) *reinterpret_cast< QAbstract3DAxis**>(_a[0]) = std::move(_r); }  break;
        case 35: { int _r = _t->selectedCustomItemIndex();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 36: { QCustom3DItem* _r = _t->selectedCustomItem();
            if (_a[0]) *reinterpret_cast< QCustom3DItem**>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Q3DScene* >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAbstract3DInputHandler* >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Q3DTheme* >(); break;
            }
            break;
        case 22:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAbstract3DAxis* >(); break;
            }
            break;
        case 23:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAbstract3DAxis* >(); break;
            }
            break;
        case 24:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAbstract3DAxis* >(); break;
            }
            break;
        case 28:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QCustom3DItem* >(); break;
            }
            break;
        case 30:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QCustom3DItem* >(); break;
            }
            break;
        case 32:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QCustom3DItem* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (AbstractDeclarative::*)(AbstractDeclarative::SelectionFlags );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractDeclarative::selectionModeChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (AbstractDeclarative::*)(AbstractDeclarative::ShadowQuality );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractDeclarative::shadowQualityChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (AbstractDeclarative::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractDeclarative::shadowsSupportedChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (AbstractDeclarative::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractDeclarative::msaaSamplesChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (AbstractDeclarative::*)(Q3DScene * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractDeclarative::sceneChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (AbstractDeclarative::*)(QAbstract3DInputHandler * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractDeclarative::inputHandlerChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (AbstractDeclarative::*)(Q3DTheme * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractDeclarative::themeChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (AbstractDeclarative::*)(AbstractDeclarative::RenderingMode );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractDeclarative::renderingModeChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (AbstractDeclarative::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractDeclarative::measureFpsChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (AbstractDeclarative::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractDeclarative::currentFpsChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (AbstractDeclarative::*)(AbstractDeclarative::ElementType );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractDeclarative::selectedElementChanged)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (AbstractDeclarative::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractDeclarative::orthoProjectionChanged)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (AbstractDeclarative::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractDeclarative::aspectRatioChanged)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (AbstractDeclarative::*)(AbstractDeclarative::OptimizationHints );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractDeclarative::optimizationHintsChanged)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (AbstractDeclarative::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractDeclarative::polarChanged)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (AbstractDeclarative::*)(float );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractDeclarative::radialLabelOffsetChanged)) {
                *result = 15;
                return;
            }
        }
        {
            using _t = void (AbstractDeclarative::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractDeclarative::horizontalAspectRatioChanged)) {
                *result = 16;
                return;
            }
        }
        {
            using _t = void (AbstractDeclarative::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractDeclarative::reflectionChanged)) {
                *result = 17;
                return;
            }
        }
        {
            using _t = void (AbstractDeclarative::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractDeclarative::reflectivityChanged)) {
                *result = 18;
                return;
            }
        }
        {
            using _t = void (AbstractDeclarative::*)(const QLocale & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractDeclarative::localeChanged)) {
                *result = 19;
                return;
            }
        }
        {
            using _t = void (AbstractDeclarative::*)(const QVector3D & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractDeclarative::queriedGraphPositionChanged)) {
                *result = 20;
                return;
            }
        }
        {
            using _t = void (AbstractDeclarative::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractDeclarative::marginChanged)) {
                *result = 21;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 4:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Declarative3DScene* >(); break;
        case 6:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Q3DTheme* >(); break;
        case 5:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAbstract3DInputHandler* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<AbstractDeclarative *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = QFlag(_t->selectionMode()); break;
        case 1: *reinterpret_cast< ShadowQuality*>(_v) = _t->shadowQuality(); break;
        case 2: *reinterpret_cast< bool*>(_v) = _t->shadowsSupported(); break;
        case 3: *reinterpret_cast< int*>(_v) = _t->msaaSamples(); break;
        case 4: *reinterpret_cast< Declarative3DScene**>(_v) = _t->scene(); break;
        case 5: *reinterpret_cast< QAbstract3DInputHandler**>(_v) = _t->inputHandler(); break;
        case 6: *reinterpret_cast< Q3DTheme**>(_v) = _t->theme(); break;
        case 7: *reinterpret_cast< RenderingMode*>(_v) = _t->renderingMode(); break;
        case 8: *reinterpret_cast< bool*>(_v) = _t->measureFps(); break;
        case 9: *reinterpret_cast< qreal*>(_v) = _t->currentFps(); break;
        case 10: *reinterpret_cast< QQmlListProperty<QCustom3DItem>*>(_v) = _t->customItemList(); break;
        case 11: *reinterpret_cast< bool*>(_v) = _t->isOrthoProjection(); break;
        case 12: *reinterpret_cast< ElementType*>(_v) = _t->selectedElement(); break;
        case 13: *reinterpret_cast< qreal*>(_v) = _t->aspectRatio(); break;
        case 14: *reinterpret_cast<int*>(_v) = QFlag(_t->optimizationHints()); break;
        case 15: *reinterpret_cast< bool*>(_v) = _t->isPolar(); break;
        case 16: *reinterpret_cast< float*>(_v) = _t->radialLabelOffset(); break;
        case 17: *reinterpret_cast< qreal*>(_v) = _t->horizontalAspectRatio(); break;
        case 18: *reinterpret_cast< bool*>(_v) = _t->isReflection(); break;
        case 19: *reinterpret_cast< qreal*>(_v) = _t->reflectivity(); break;
        case 20: *reinterpret_cast< QLocale*>(_v) = _t->locale(); break;
        case 21: *reinterpret_cast< QVector3D*>(_v) = _t->queriedGraphPosition(); break;
        case 22: *reinterpret_cast< qreal*>(_v) = _t->margin(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<AbstractDeclarative *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setSelectionMode(QFlag(*reinterpret_cast<int*>(_v))); break;
        case 1: _t->setShadowQuality(*reinterpret_cast< ShadowQuality*>(_v)); break;
        case 3: _t->setMsaaSamples(*reinterpret_cast< int*>(_v)); break;
        case 5: _t->setInputHandler(*reinterpret_cast< QAbstract3DInputHandler**>(_v)); break;
        case 6: _t->setTheme(*reinterpret_cast< Q3DTheme**>(_v)); break;
        case 7: _t->setRenderingMode(*reinterpret_cast< RenderingMode*>(_v)); break;
        case 8: _t->setMeasureFps(*reinterpret_cast< bool*>(_v)); break;
        case 11: _t->setOrthoProjection(*reinterpret_cast< bool*>(_v)); break;
        case 13: _t->setAspectRatio(*reinterpret_cast< qreal*>(_v)); break;
        case 14: _t->setOptimizationHints(QFlag(*reinterpret_cast<int*>(_v))); break;
        case 15: _t->setPolar(*reinterpret_cast< bool*>(_v)); break;
        case 16: _t->setRadialLabelOffset(*reinterpret_cast< float*>(_v)); break;
        case 17: _t->setHorizontalAspectRatio(*reinterpret_cast< qreal*>(_v)); break;
        case 18: _t->setReflection(*reinterpret_cast< bool*>(_v)); break;
        case 19: _t->setReflectivity(*reinterpret_cast< qreal*>(_v)); break;
        case 20: _t->setLocale(*reinterpret_cast< QLocale*>(_v)); break;
        case 22: _t->setMargin(*reinterpret_cast< qreal*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QtDataVisualization::AbstractDeclarative::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickItem::staticMetaObject>(),
    qt_meta_stringdata_QtDataVisualization__AbstractDeclarative.data,
    qt_meta_data_QtDataVisualization__AbstractDeclarative,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QtDataVisualization::AbstractDeclarative::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtDataVisualization::AbstractDeclarative::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QtDataVisualization__AbstractDeclarative.stringdata0))
        return static_cast<void*>(this);
    return QQuickItem::qt_metacast(_clname);
}

int QtDataVisualization::AbstractDeclarative::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 37)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 37;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 37)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 37;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 23;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 23;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 23;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 23;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 23;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QtDataVisualization::AbstractDeclarative::selectionModeChanged(AbstractDeclarative::SelectionFlags _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QtDataVisualization::AbstractDeclarative::shadowQualityChanged(AbstractDeclarative::ShadowQuality _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QtDataVisualization::AbstractDeclarative::shadowsSupportedChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QtDataVisualization::AbstractDeclarative::msaaSamplesChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QtDataVisualization::AbstractDeclarative::sceneChanged(Q3DScene * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void QtDataVisualization::AbstractDeclarative::inputHandlerChanged(QAbstract3DInputHandler * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void QtDataVisualization::AbstractDeclarative::themeChanged(Q3DTheme * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void QtDataVisualization::AbstractDeclarative::renderingModeChanged(AbstractDeclarative::RenderingMode _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void QtDataVisualization::AbstractDeclarative::measureFpsChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void QtDataVisualization::AbstractDeclarative::currentFpsChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void QtDataVisualization::AbstractDeclarative::selectedElementChanged(AbstractDeclarative::ElementType _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 10, _a);
}

// SIGNAL 11
void QtDataVisualization::AbstractDeclarative::orthoProjectionChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 11, _a);
}

// SIGNAL 12
void QtDataVisualization::AbstractDeclarative::aspectRatioChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 12, _a);
}

// SIGNAL 13
void QtDataVisualization::AbstractDeclarative::optimizationHintsChanged(AbstractDeclarative::OptimizationHints _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 13, _a);
}

// SIGNAL 14
void QtDataVisualization::AbstractDeclarative::polarChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 14, _a);
}

// SIGNAL 15
void QtDataVisualization::AbstractDeclarative::radialLabelOffsetChanged(float _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 15, _a);
}

// SIGNAL 16
void QtDataVisualization::AbstractDeclarative::horizontalAspectRatioChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 16, _a);
}

// SIGNAL 17
void QtDataVisualization::AbstractDeclarative::reflectionChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 17, _a);
}

// SIGNAL 18
void QtDataVisualization::AbstractDeclarative::reflectivityChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 18, _a);
}

// SIGNAL 19
void QtDataVisualization::AbstractDeclarative::localeChanged(const QLocale & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 19, _a);
}

// SIGNAL 20
void QtDataVisualization::AbstractDeclarative::queriedGraphPositionChanged(const QVector3D & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 20, _a);
}

// SIGNAL 21
void QtDataVisualization::AbstractDeclarative::marginChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 21, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
