/****************************************************************************
** Meta object code from reading C++ file 'declarativesurface_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kde-qtdatavis3d/src/datavisualizationqml2/declarativesurface_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'declarativesurface_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QtDataVisualization__DeclarativeSurface_t {
    QByteArrayData data[26];
    char stringdata0[388];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QtDataVisualization__DeclarativeSurface_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QtDataVisualization__DeclarativeSurface_t qt_meta_stringdata_QtDataVisualization__DeclarativeSurface = {
    {
QT_MOC_LITERAL(0, 0, 39), // "QtDataVisualization::Declarat..."
QT_MOC_LITERAL(1, 40, 15), // "DefaultProperty"
QT_MOC_LITERAL(2, 56, 10), // "seriesList"
QT_MOC_LITERAL(3, 67, 12), // "axisXChanged"
QT_MOC_LITERAL(4, 80, 0), // ""
QT_MOC_LITERAL(5, 81, 13), // "QValue3DAxis*"
QT_MOC_LITERAL(6, 95, 4), // "axis"
QT_MOC_LITERAL(7, 100, 12), // "axisYChanged"
QT_MOC_LITERAL(8, 113, 12), // "axisZChanged"
QT_MOC_LITERAL(9, 126, 21), // "selectedSeriesChanged"
QT_MOC_LITERAL(10, 148, 17), // "QSurface3DSeries*"
QT_MOC_LITERAL(11, 166, 6), // "series"
QT_MOC_LITERAL(12, 173, 25), // "flipHorizontalGridChanged"
QT_MOC_LITERAL(13, 199, 4), // "flip"
QT_MOC_LITERAL(14, 204, 18), // "handleAxisXChanged"
QT_MOC_LITERAL(15, 223, 16), // "QAbstract3DAxis*"
QT_MOC_LITERAL(16, 240, 18), // "handleAxisYChanged"
QT_MOC_LITERAL(17, 259, 18), // "handleAxisZChanged"
QT_MOC_LITERAL(18, 278, 9), // "addSeries"
QT_MOC_LITERAL(19, 288, 12), // "removeSeries"
QT_MOC_LITERAL(20, 301, 5), // "axisX"
QT_MOC_LITERAL(21, 307, 5), // "axisY"
QT_MOC_LITERAL(22, 313, 5), // "axisZ"
QT_MOC_LITERAL(23, 319, 14), // "selectedSeries"
QT_MOC_LITERAL(24, 334, 34), // "QQmlListProperty<QSurface3DSe..."
QT_MOC_LITERAL(25, 369, 18) // "flipHorizontalGrid"

    },
    "QtDataVisualization::DeclarativeSurface\0"
    "DefaultProperty\0seriesList\0axisXChanged\0"
    "\0QValue3DAxis*\0axis\0axisYChanged\0"
    "axisZChanged\0selectedSeriesChanged\0"
    "QSurface3DSeries*\0series\0"
    "flipHorizontalGridChanged\0flip\0"
    "handleAxisXChanged\0QAbstract3DAxis*\0"
    "handleAxisYChanged\0handleAxisZChanged\0"
    "addSeries\0removeSeries\0axisX\0axisY\0"
    "axisZ\0selectedSeries\0"
    "QQmlListProperty<QSurface3DSeries>\0"
    "flipHorizontalGrid"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QtDataVisualization__DeclarativeSurface[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
      10,   16, // methods
       6,  106, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    1,   76,    4, 0x06 /* Public */,
       7,    1,   79,    4, 0x06 /* Public */,
       8,    1,   82,    4, 0x06 /* Public */,
       9,    1,   85,    4, 0x06 /* Public */,
      12,    1,   88,    4, 0x86 /* Public | MethodRevisioned */,

 // slots: name, argc, parameters, tag, flags
      14,    1,   91,    4, 0x0a /* Public */,
      16,    1,   94,    4, 0x0a /* Public */,
      17,    1,   97,    4, 0x0a /* Public */,

 // methods: name, argc, parameters, tag, flags
      18,    1,  100,    4, 0x02 /* Public */,
      19,    1,  103,    4, 0x02 /* Public */,

 // signals: revision
       0,
       0,
       0,
       0,
       1,

 // slots: revision
       0,
       0,
       0,

 // methods: revision
       0,
       0,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 5,    6,
    QMetaType::Void, 0x80000000 | 5,    6,
    QMetaType::Void, 0x80000000 | 5,    6,
    QMetaType::Void, 0x80000000 | 10,   11,
    QMetaType::Void, QMetaType::Bool,   13,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 15,    6,
    QMetaType::Void, 0x80000000 | 15,    6,
    QMetaType::Void, 0x80000000 | 15,    6,

 // methods: parameters
    QMetaType::Void, 0x80000000 | 10,   11,
    QMetaType::Void, 0x80000000 | 10,   11,

 // properties: name, type, flags
      20, 0x80000000 | 5, 0x0049510b,
      21, 0x80000000 | 5, 0x0049510b,
      22, 0x80000000 | 5, 0x0049510b,
      23, 0x80000000 | 10, 0x00495009,
       2, 0x80000000 | 24, 0x00095009,
      25, QMetaType::Bool, 0x00c95103,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,
       0,
       4,

 // properties: revision
       0,
       0,
       0,
       0,
       0,
       1,

       0        // eod
};

void QtDataVisualization::DeclarativeSurface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DeclarativeSurface *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->axisXChanged((*reinterpret_cast< QValue3DAxis*(*)>(_a[1]))); break;
        case 1: _t->axisYChanged((*reinterpret_cast< QValue3DAxis*(*)>(_a[1]))); break;
        case 2: _t->axisZChanged((*reinterpret_cast< QValue3DAxis*(*)>(_a[1]))); break;
        case 3: _t->selectedSeriesChanged((*reinterpret_cast< QSurface3DSeries*(*)>(_a[1]))); break;
        case 4: _t->flipHorizontalGridChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: _t->handleAxisXChanged((*reinterpret_cast< QAbstract3DAxis*(*)>(_a[1]))); break;
        case 6: _t->handleAxisYChanged((*reinterpret_cast< QAbstract3DAxis*(*)>(_a[1]))); break;
        case 7: _t->handleAxisZChanged((*reinterpret_cast< QAbstract3DAxis*(*)>(_a[1]))); break;
        case 8: _t->addSeries((*reinterpret_cast< QSurface3DSeries*(*)>(_a[1]))); break;
        case 9: _t->removeSeries((*reinterpret_cast< QSurface3DSeries*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QValue3DAxis* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QValue3DAxis* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QValue3DAxis* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QSurface3DSeries* >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAbstract3DAxis* >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAbstract3DAxis* >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAbstract3DAxis* >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QSurface3DSeries* >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QSurface3DSeries* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DeclarativeSurface::*)(QValue3DAxis * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeclarativeSurface::axisXChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (DeclarativeSurface::*)(QValue3DAxis * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeclarativeSurface::axisYChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (DeclarativeSurface::*)(QValue3DAxis * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeclarativeSurface::axisZChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (DeclarativeSurface::*)(QSurface3DSeries * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeclarativeSurface::selectedSeriesChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (DeclarativeSurface::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeclarativeSurface::flipHorizontalGridChanged)) {
                *result = 4;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QSurface3DSeries* >(); break;
        case 2:
        case 1:
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QValue3DAxis* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<DeclarativeSurface *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QValue3DAxis**>(_v) = _t->axisX(); break;
        case 1: *reinterpret_cast< QValue3DAxis**>(_v) = _t->axisY(); break;
        case 2: *reinterpret_cast< QValue3DAxis**>(_v) = _t->axisZ(); break;
        case 3: *reinterpret_cast< QSurface3DSeries**>(_v) = _t->selectedSeries(); break;
        case 4: *reinterpret_cast< QQmlListProperty<QSurface3DSeries>*>(_v) = _t->seriesList(); break;
        case 5: *reinterpret_cast< bool*>(_v) = _t->flipHorizontalGrid(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<DeclarativeSurface *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setAxisX(*reinterpret_cast< QValue3DAxis**>(_v)); break;
        case 1: _t->setAxisY(*reinterpret_cast< QValue3DAxis**>(_v)); break;
        case 2: _t->setAxisZ(*reinterpret_cast< QValue3DAxis**>(_v)); break;
        case 5: _t->setFlipHorizontalGrid(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QtDataVisualization::DeclarativeSurface::staticMetaObject = { {
    QMetaObject::SuperData::link<AbstractDeclarative::staticMetaObject>(),
    qt_meta_stringdata_QtDataVisualization__DeclarativeSurface.data,
    qt_meta_data_QtDataVisualization__DeclarativeSurface,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QtDataVisualization::DeclarativeSurface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtDataVisualization::DeclarativeSurface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QtDataVisualization__DeclarativeSurface.stringdata0))
        return static_cast<void*>(this);
    return AbstractDeclarative::qt_metacast(_clname);
}

int QtDataVisualization::DeclarativeSurface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AbstractDeclarative::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 6;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QtDataVisualization::DeclarativeSurface::axisXChanged(QValue3DAxis * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QtDataVisualization::DeclarativeSurface::axisYChanged(QValue3DAxis * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QtDataVisualization::DeclarativeSurface::axisZChanged(QValue3DAxis * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QtDataVisualization::DeclarativeSurface::selectedSeriesChanged(QSurface3DSeries * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QtDataVisualization::DeclarativeSurface::flipHorizontalGridChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
