/****************************************************************************
** Meta object code from reading C++ file 'qsurface3dseries.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kde-qtdatavis3d/src/datavisualization/data/qsurface3dseries.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qsurface3dseries.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QtDataVisualization__QSurface3DSeries_t {
    QByteArrayData data[29];
    char stringdata0[432];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QtDataVisualization__QSurface3DSeries_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QtDataVisualization__QSurface3DSeries_t qt_meta_stringdata_QtDataVisualization__QSurface3DSeries = {
    {
QT_MOC_LITERAL(0, 0, 37), // "QtDataVisualization::QSurface..."
QT_MOC_LITERAL(1, 38, 16), // "dataProxyChanged"
QT_MOC_LITERAL(2, 55, 0), // ""
QT_MOC_LITERAL(3, 56, 18), // "QSurfaceDataProxy*"
QT_MOC_LITERAL(4, 75, 5), // "proxy"
QT_MOC_LITERAL(5, 81, 20), // "selectedPointChanged"
QT_MOC_LITERAL(6, 102, 8), // "position"
QT_MOC_LITERAL(7, 111, 25), // "flatShadingEnabledChanged"
QT_MOC_LITERAL(8, 137, 6), // "enable"
QT_MOC_LITERAL(9, 144, 27), // "flatShadingSupportedChanged"
QT_MOC_LITERAL(10, 172, 15), // "drawModeChanged"
QT_MOC_LITERAL(11, 188, 27), // "QSurface3DSeries::DrawFlags"
QT_MOC_LITERAL(12, 216, 4), // "mode"
QT_MOC_LITERAL(13, 221, 14), // "textureChanged"
QT_MOC_LITERAL(14, 236, 5), // "image"
QT_MOC_LITERAL(15, 242, 18), // "textureFileChanged"
QT_MOC_LITERAL(16, 261, 8), // "filename"
QT_MOC_LITERAL(17, 270, 9), // "dataProxy"
QT_MOC_LITERAL(18, 280, 13), // "selectedPoint"
QT_MOC_LITERAL(19, 294, 18), // "flatShadingEnabled"
QT_MOC_LITERAL(20, 313, 20), // "flatShadingSupported"
QT_MOC_LITERAL(21, 334, 8), // "drawMode"
QT_MOC_LITERAL(22, 343, 9), // "DrawFlags"
QT_MOC_LITERAL(23, 353, 7), // "texture"
QT_MOC_LITERAL(24, 361, 11), // "textureFile"
QT_MOC_LITERAL(25, 373, 8), // "DrawFlag"
QT_MOC_LITERAL(26, 382, 13), // "DrawWireframe"
QT_MOC_LITERAL(27, 396, 11), // "DrawSurface"
QT_MOC_LITERAL(28, 408, 23) // "DrawSurfaceAndWireframe"

    },
    "QtDataVisualization::QSurface3DSeries\0"
    "dataProxyChanged\0\0QSurfaceDataProxy*\0"
    "proxy\0selectedPointChanged\0position\0"
    "flatShadingEnabledChanged\0enable\0"
    "flatShadingSupportedChanged\0drawModeChanged\0"
    "QSurface3DSeries::DrawFlags\0mode\0"
    "textureChanged\0image\0textureFileChanged\0"
    "filename\0dataProxy\0selectedPoint\0"
    "flatShadingEnabled\0flatShadingSupported\0"
    "drawMode\0DrawFlags\0texture\0textureFile\0"
    "DrawFlag\0DrawWireframe\0DrawSurface\0"
    "DrawSurfaceAndWireframe"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QtDataVisualization__QSurface3DSeries[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       7,   70, // properties
       2,   98, // enums/sets
       0,    0, // constructors
       0,       // flags
       7,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   49,    2, 0x06 /* Public */,
       5,    1,   52,    2, 0x06 /* Public */,
       7,    1,   55,    2, 0x06 /* Public */,
       9,    1,   58,    2, 0x06 /* Public */,
      10,    1,   61,    2, 0x06 /* Public */,
      13,    1,   64,    2, 0x06 /* Public */,
      15,    1,   67,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, QMetaType::QPoint,    6,
    QMetaType::Void, QMetaType::Bool,    8,
    QMetaType::Void, QMetaType::Bool,    8,
    QMetaType::Void, 0x80000000 | 11,   12,
    QMetaType::Void, QMetaType::QImage,   14,
    QMetaType::Void, QMetaType::QString,   16,

 // properties: name, type, flags
      17, 0x80000000 | 3, 0x0049510b,
      18, QMetaType::QPoint, 0x00495103,
      19, QMetaType::Bool, 0x00495103,
      20, QMetaType::Bool, 0x00495001,
      21, 0x80000000 | 22, 0x0049510b,
      23, QMetaType::QImage, 0x00495103,
      24, QMetaType::QString, 0x00495103,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,
       4,
       5,
       6,

 // enums: name, alias, flags, count, data
      25,   25, 0x1,    3,  108,
      22,   25, 0x1,    3,  114,

 // enum data: key, value
      26, uint(QtDataVisualization::QSurface3DSeries::DrawWireframe),
      27, uint(QtDataVisualization::QSurface3DSeries::DrawSurface),
      28, uint(QtDataVisualization::QSurface3DSeries::DrawSurfaceAndWireframe),
      26, uint(QtDataVisualization::QSurface3DSeries::DrawWireframe),
      27, uint(QtDataVisualization::QSurface3DSeries::DrawSurface),
      28, uint(QtDataVisualization::QSurface3DSeries::DrawSurfaceAndWireframe),

       0        // eod
};

void QtDataVisualization::QSurface3DSeries::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QSurface3DSeries *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->dataProxyChanged((*reinterpret_cast< QSurfaceDataProxy*(*)>(_a[1]))); break;
        case 1: _t->selectedPointChanged((*reinterpret_cast< const QPoint(*)>(_a[1]))); break;
        case 2: _t->flatShadingEnabledChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->flatShadingSupportedChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 4: _t->drawModeChanged((*reinterpret_cast< QSurface3DSeries::DrawFlags(*)>(_a[1]))); break;
        case 5: _t->textureChanged((*reinterpret_cast< const QImage(*)>(_a[1]))); break;
        case 6: _t->textureFileChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QSurfaceDataProxy* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QSurface3DSeries::*)(QSurfaceDataProxy * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QSurface3DSeries::dataProxyChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QSurface3DSeries::*)(const QPoint & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QSurface3DSeries::selectedPointChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QSurface3DSeries::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QSurface3DSeries::flatShadingEnabledChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QSurface3DSeries::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QSurface3DSeries::flatShadingSupportedChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QSurface3DSeries::*)(QSurface3DSeries::DrawFlags );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QSurface3DSeries::drawModeChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QSurface3DSeries::*)(const QImage & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QSurface3DSeries::textureChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QSurface3DSeries::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QSurface3DSeries::textureFileChanged)) {
                *result = 6;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QSurfaceDataProxy* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QSurface3DSeries *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QSurfaceDataProxy**>(_v) = _t->dataProxy(); break;
        case 1: *reinterpret_cast< QPoint*>(_v) = _t->selectedPoint(); break;
        case 2: *reinterpret_cast< bool*>(_v) = _t->isFlatShadingEnabled(); break;
        case 3: *reinterpret_cast< bool*>(_v) = _t->isFlatShadingSupported(); break;
        case 4: *reinterpret_cast<int*>(_v) = QFlag(_t->drawMode()); break;
        case 5: *reinterpret_cast< QImage*>(_v) = _t->texture(); break;
        case 6: *reinterpret_cast< QString*>(_v) = _t->textureFile(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QSurface3DSeries *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setDataProxy(*reinterpret_cast< QSurfaceDataProxy**>(_v)); break;
        case 1: _t->setSelectedPoint(*reinterpret_cast< QPoint*>(_v)); break;
        case 2: _t->setFlatShadingEnabled(*reinterpret_cast< bool*>(_v)); break;
        case 4: _t->setDrawMode(QFlag(*reinterpret_cast<int*>(_v))); break;
        case 5: _t->setTexture(*reinterpret_cast< QImage*>(_v)); break;
        case 6: _t->setTextureFile(*reinterpret_cast< QString*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QtDataVisualization::QSurface3DSeries::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstract3DSeries::staticMetaObject>(),
    qt_meta_stringdata_QtDataVisualization__QSurface3DSeries.data,
    qt_meta_data_QtDataVisualization__QSurface3DSeries,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QtDataVisualization::QSurface3DSeries::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtDataVisualization::QSurface3DSeries::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QtDataVisualization__QSurface3DSeries.stringdata0))
        return static_cast<void*>(this);
    return QAbstract3DSeries::qt_metacast(_clname);
}

int QtDataVisualization::QSurface3DSeries::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstract3DSeries::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 7;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QtDataVisualization::QSurface3DSeries::dataProxyChanged(QSurfaceDataProxy * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QtDataVisualization::QSurface3DSeries::selectedPointChanged(const QPoint & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QtDataVisualization::QSurface3DSeries::flatShadingEnabledChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QtDataVisualization::QSurface3DSeries::flatShadingSupportedChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QtDataVisualization::QSurface3DSeries::drawModeChanged(QSurface3DSeries::DrawFlags _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void QtDataVisualization::QSurface3DSeries::textureChanged(const QImage & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void QtDataVisualization::QSurface3DSeries::textureFileChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
