/****************************************************************************
** Meta object code from reading C++ file 'qitemmodelscatterdataproxy.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kde-qtdatavis3d/src/datavisualization/data/qitemmodelscatterdataproxy.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qitemmodelscatterdataproxy.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QtDataVisualization__QItemModelScatterDataProxy_t {
    QByteArrayData data[33];
    char stringdata0[579];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QtDataVisualization__QItemModelScatterDataProxy_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QtDataVisualization__QItemModelScatterDataProxy_t qt_meta_stringdata_QtDataVisualization__QItemModelScatterDataProxy = {
    {
QT_MOC_LITERAL(0, 0, 47), // "QtDataVisualization::QItemMod..."
QT_MOC_LITERAL(1, 48, 16), // "itemModelChanged"
QT_MOC_LITERAL(2, 65, 0), // ""
QT_MOC_LITERAL(3, 66, 25), // "const QAbstractItemModel*"
QT_MOC_LITERAL(4, 92, 9), // "itemModel"
QT_MOC_LITERAL(5, 102, 15), // "xPosRoleChanged"
QT_MOC_LITERAL(6, 118, 4), // "role"
QT_MOC_LITERAL(7, 123, 15), // "yPosRoleChanged"
QT_MOC_LITERAL(8, 139, 15), // "zPosRoleChanged"
QT_MOC_LITERAL(9, 155, 19), // "rotationRoleChanged"
QT_MOC_LITERAL(10, 175, 22), // "xPosRolePatternChanged"
QT_MOC_LITERAL(11, 198, 7), // "pattern"
QT_MOC_LITERAL(12, 206, 22), // "yPosRolePatternChanged"
QT_MOC_LITERAL(13, 229, 22), // "zPosRolePatternChanged"
QT_MOC_LITERAL(14, 252, 26), // "rotationRolePatternChanged"
QT_MOC_LITERAL(15, 279, 26), // "rotationRoleReplaceChanged"
QT_MOC_LITERAL(16, 306, 7), // "replace"
QT_MOC_LITERAL(17, 314, 22), // "xPosRoleReplaceChanged"
QT_MOC_LITERAL(18, 337, 22), // "yPosRoleReplaceChanged"
QT_MOC_LITERAL(19, 360, 22), // "zPosRoleReplaceChanged"
QT_MOC_LITERAL(20, 383, 19), // "QAbstractItemModel*"
QT_MOC_LITERAL(21, 403, 8), // "xPosRole"
QT_MOC_LITERAL(22, 412, 8), // "yPosRole"
QT_MOC_LITERAL(23, 421, 8), // "zPosRole"
QT_MOC_LITERAL(24, 430, 12), // "rotationRole"
QT_MOC_LITERAL(25, 443, 15), // "xPosRolePattern"
QT_MOC_LITERAL(26, 459, 15), // "yPosRolePattern"
QT_MOC_LITERAL(27, 475, 15), // "zPosRolePattern"
QT_MOC_LITERAL(28, 491, 19), // "rotationRolePattern"
QT_MOC_LITERAL(29, 511, 15), // "xPosRoleReplace"
QT_MOC_LITERAL(30, 527, 15), // "yPosRoleReplace"
QT_MOC_LITERAL(31, 543, 15), // "zPosRoleReplace"
QT_MOC_LITERAL(32, 559, 19) // "rotationRoleReplace"

    },
    "QtDataVisualization::QItemModelScatterDataProxy\0"
    "itemModelChanged\0\0const QAbstractItemModel*\0"
    "itemModel\0xPosRoleChanged\0role\0"
    "yPosRoleChanged\0zPosRoleChanged\0"
    "rotationRoleChanged\0xPosRolePatternChanged\0"
    "pattern\0yPosRolePatternChanged\0"
    "zPosRolePatternChanged\0"
    "rotationRolePatternChanged\0"
    "rotationRoleReplaceChanged\0replace\0"
    "xPosRoleReplaceChanged\0yPosRoleReplaceChanged\0"
    "zPosRoleReplaceChanged\0QAbstractItemModel*\0"
    "xPosRole\0yPosRole\0zPosRole\0rotationRole\0"
    "xPosRolePattern\0yPosRolePattern\0"
    "zPosRolePattern\0rotationRolePattern\0"
    "xPosRoleReplace\0yPosRoleReplace\0"
    "zPosRoleReplace\0rotationRoleReplace"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QtDataVisualization__QItemModelScatterDataProxy[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      13,   14, // methods
      13,  131, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      13,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   92,    2, 0x06 /* Public */,
       5,    1,   95,    2, 0x06 /* Public */,
       7,    1,   98,    2, 0x06 /* Public */,
       8,    1,  101,    2, 0x06 /* Public */,
       9,    1,  104,    2, 0x06 /* Public */,
      10,    1,  107,    2, 0x86 /* Public | MethodRevisioned */,
      12,    1,  110,    2, 0x86 /* Public | MethodRevisioned */,
      13,    1,  113,    2, 0x86 /* Public | MethodRevisioned */,
      14,    1,  116,    2, 0x86 /* Public | MethodRevisioned */,
      15,    1,  119,    2, 0x86 /* Public | MethodRevisioned */,
      17,    1,  122,    2, 0x86 /* Public | MethodRevisioned */,
      18,    1,  125,    2, 0x86 /* Public | MethodRevisioned */,
      19,    1,  128,    2, 0x86 /* Public | MethodRevisioned */,

 // signals: revision
       0,
       0,
       0,
       0,
       0,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, QMetaType::QString,    6,
    QMetaType::Void, QMetaType::QString,    6,
    QMetaType::Void, QMetaType::QString,    6,
    QMetaType::Void, QMetaType::QString,    6,
    QMetaType::Void, QMetaType::QRegExp,   11,
    QMetaType::Void, QMetaType::QRegExp,   11,
    QMetaType::Void, QMetaType::QRegExp,   11,
    QMetaType::Void, QMetaType::QRegExp,   11,
    QMetaType::Void, QMetaType::QString,   16,
    QMetaType::Void, QMetaType::QString,   16,
    QMetaType::Void, QMetaType::QString,   16,
    QMetaType::Void, QMetaType::QString,   16,

 // properties: name, type, flags
       4, 0x80000000 | 20, 0x0049510b,
      21, QMetaType::QString, 0x00495103,
      22, QMetaType::QString, 0x00495103,
      23, QMetaType::QString, 0x00495103,
      24, QMetaType::QString, 0x00495103,
      25, QMetaType::QRegExp, 0x00c95103,
      26, QMetaType::QRegExp, 0x00c95103,
      27, QMetaType::QRegExp, 0x00c95103,
      28, QMetaType::QRegExp, 0x00c95103,
      29, QMetaType::QString, 0x00c95103,
      30, QMetaType::QString, 0x00c95103,
      31, QMetaType::QString, 0x00c95103,
      32, QMetaType::QString, 0x00c95103,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,
       4,
       5,
       6,
       7,
       8,
      10,
      11,
      12,
       9,

 // properties: revision
       0,
       0,
       0,
       0,
       0,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,

       0        // eod
};

void QtDataVisualization::QItemModelScatterDataProxy::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QItemModelScatterDataProxy *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->itemModelChanged((*reinterpret_cast< const QAbstractItemModel*(*)>(_a[1]))); break;
        case 1: _t->xPosRoleChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->yPosRoleChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 3: _t->zPosRoleChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 4: _t->rotationRoleChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->xPosRolePatternChanged((*reinterpret_cast< const QRegExp(*)>(_a[1]))); break;
        case 6: _t->yPosRolePatternChanged((*reinterpret_cast< const QRegExp(*)>(_a[1]))); break;
        case 7: _t->zPosRolePatternChanged((*reinterpret_cast< const QRegExp(*)>(_a[1]))); break;
        case 8: _t->rotationRolePatternChanged((*reinterpret_cast< const QRegExp(*)>(_a[1]))); break;
        case 9: _t->rotationRoleReplaceChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 10: _t->xPosRoleReplaceChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 11: _t->yPosRoleReplaceChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 12: _t->zPosRoleReplaceChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QItemModelScatterDataProxy::*)(const QAbstractItemModel * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QItemModelScatterDataProxy::itemModelChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QItemModelScatterDataProxy::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QItemModelScatterDataProxy::xPosRoleChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QItemModelScatterDataProxy::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QItemModelScatterDataProxy::yPosRoleChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QItemModelScatterDataProxy::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QItemModelScatterDataProxy::zPosRoleChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QItemModelScatterDataProxy::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QItemModelScatterDataProxy::rotationRoleChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QItemModelScatterDataProxy::*)(const QRegExp & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QItemModelScatterDataProxy::xPosRolePatternChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QItemModelScatterDataProxy::*)(const QRegExp & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QItemModelScatterDataProxy::yPosRolePatternChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QItemModelScatterDataProxy::*)(const QRegExp & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QItemModelScatterDataProxy::zPosRolePatternChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QItemModelScatterDataProxy::*)(const QRegExp & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QItemModelScatterDataProxy::rotationRolePatternChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QItemModelScatterDataProxy::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QItemModelScatterDataProxy::rotationRoleReplaceChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (QItemModelScatterDataProxy::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QItemModelScatterDataProxy::xPosRoleReplaceChanged)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (QItemModelScatterDataProxy::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QItemModelScatterDataProxy::yPosRoleReplaceChanged)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (QItemModelScatterDataProxy::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QItemModelScatterDataProxy::zPosRoleReplaceChanged)) {
                *result = 12;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAbstractItemModel* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QItemModelScatterDataProxy *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QAbstractItemModel**>(_v) = _t->itemModel(); break;
        case 1: *reinterpret_cast< QString*>(_v) = _t->xPosRole(); break;
        case 2: *reinterpret_cast< QString*>(_v) = _t->yPosRole(); break;
        case 3: *reinterpret_cast< QString*>(_v) = _t->zPosRole(); break;
        case 4: *reinterpret_cast< QString*>(_v) = _t->rotationRole(); break;
        case 5: *reinterpret_cast< QRegExp*>(_v) = _t->xPosRolePattern(); break;
        case 6: *reinterpret_cast< QRegExp*>(_v) = _t->yPosRolePattern(); break;
        case 7: *reinterpret_cast< QRegExp*>(_v) = _t->zPosRolePattern(); break;
        case 8: *reinterpret_cast< QRegExp*>(_v) = _t->rotationRolePattern(); break;
        case 9: *reinterpret_cast< QString*>(_v) = _t->xPosRoleReplace(); break;
        case 10: *reinterpret_cast< QString*>(_v) = _t->yPosRoleReplace(); break;
        case 11: *reinterpret_cast< QString*>(_v) = _t->zPosRoleReplace(); break;
        case 12: *reinterpret_cast< QString*>(_v) = _t->rotationRoleReplace(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QItemModelScatterDataProxy *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setItemModel(*reinterpret_cast< QAbstractItemModel**>(_v)); break;
        case 1: _t->setXPosRole(*reinterpret_cast< QString*>(_v)); break;
        case 2: _t->setYPosRole(*reinterpret_cast< QString*>(_v)); break;
        case 3: _t->setZPosRole(*reinterpret_cast< QString*>(_v)); break;
        case 4: _t->setRotationRole(*reinterpret_cast< QString*>(_v)); break;
        case 5: _t->setXPosRolePattern(*reinterpret_cast< QRegExp*>(_v)); break;
        case 6: _t->setYPosRolePattern(*reinterpret_cast< QRegExp*>(_v)); break;
        case 7: _t->setZPosRolePattern(*reinterpret_cast< QRegExp*>(_v)); break;
        case 8: _t->setRotationRolePattern(*reinterpret_cast< QRegExp*>(_v)); break;
        case 9: _t->setXPosRoleReplace(*reinterpret_cast< QString*>(_v)); break;
        case 10: _t->setYPosRoleReplace(*reinterpret_cast< QString*>(_v)); break;
        case 11: _t->setZPosRoleReplace(*reinterpret_cast< QString*>(_v)); break;
        case 12: _t->setRotationRoleReplace(*reinterpret_cast< QString*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QtDataVisualization::QItemModelScatterDataProxy::staticMetaObject = { {
    QMetaObject::SuperData::link<QScatterDataProxy::staticMetaObject>(),
    qt_meta_stringdata_QtDataVisualization__QItemModelScatterDataProxy.data,
    qt_meta_data_QtDataVisualization__QItemModelScatterDataProxy,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QtDataVisualization::QItemModelScatterDataProxy::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtDataVisualization::QItemModelScatterDataProxy::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QtDataVisualization__QItemModelScatterDataProxy.stringdata0))
        return static_cast<void*>(this);
    return QScatterDataProxy::qt_metacast(_clname);
}

int QtDataVisualization::QItemModelScatterDataProxy::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QScatterDataProxy::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 13;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 13;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 13;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 13;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 13;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 13;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QtDataVisualization::QItemModelScatterDataProxy::itemModelChanged(const QAbstractItemModel * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QtDataVisualization::QItemModelScatterDataProxy::xPosRoleChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QtDataVisualization::QItemModelScatterDataProxy::yPosRoleChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QtDataVisualization::QItemModelScatterDataProxy::zPosRoleChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QtDataVisualization::QItemModelScatterDataProxy::rotationRoleChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void QtDataVisualization::QItemModelScatterDataProxy::xPosRolePatternChanged(const QRegExp & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void QtDataVisualization::QItemModelScatterDataProxy::yPosRolePatternChanged(const QRegExp & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void QtDataVisualization::QItemModelScatterDataProxy::zPosRolePatternChanged(const QRegExp & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void QtDataVisualization::QItemModelScatterDataProxy::rotationRolePatternChanged(const QRegExp & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void QtDataVisualization::QItemModelScatterDataProxy::rotationRoleReplaceChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void QtDataVisualization::QItemModelScatterDataProxy::xPosRoleReplaceChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 10, _a);
}

// SIGNAL 11
void QtDataVisualization::QItemModelScatterDataProxy::yPosRoleReplaceChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 11, _a);
}

// SIGNAL 12
void QtDataVisualization::QItemModelScatterDataProxy::zPosRoleReplaceChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 12, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
