/****************************************************************************
** Meta object code from reading C++ file 'qcustom3dvolume.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kde-qtdatavis3d/src/datavisualization/data/qcustom3dvolume.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QVector>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qcustom3dvolume.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QtDataVisualization__QCustom3DVolume_t {
    QByteArrayData data[47];
    char stringdata0[757];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QtDataVisualization__QCustom3DVolume_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QtDataVisualization__QCustom3DVolume_t qt_meta_stringdata_QtDataVisualization__QCustom3DVolume = {
    {
QT_MOC_LITERAL(0, 0, 36), // "QtDataVisualization::QCustom3..."
QT_MOC_LITERAL(1, 37, 19), // "textureWidthChanged"
QT_MOC_LITERAL(2, 57, 0), // ""
QT_MOC_LITERAL(3, 58, 5), // "value"
QT_MOC_LITERAL(4, 64, 20), // "textureHeightChanged"
QT_MOC_LITERAL(5, 85, 19), // "textureDepthChanged"
QT_MOC_LITERAL(6, 105, 18), // "sliceIndexXChanged"
QT_MOC_LITERAL(7, 124, 18), // "sliceIndexYChanged"
QT_MOC_LITERAL(8, 143, 18), // "sliceIndexZChanged"
QT_MOC_LITERAL(9, 162, 17), // "colorTableChanged"
QT_MOC_LITERAL(10, 180, 18), // "textureDataChanged"
QT_MOC_LITERAL(11, 199, 15), // "QVector<uchar>*"
QT_MOC_LITERAL(12, 215, 4), // "data"
QT_MOC_LITERAL(13, 220, 20), // "textureFormatChanged"
QT_MOC_LITERAL(14, 241, 14), // "QImage::Format"
QT_MOC_LITERAL(15, 256, 6), // "format"
QT_MOC_LITERAL(16, 263, 22), // "alphaMultiplierChanged"
QT_MOC_LITERAL(17, 286, 4), // "mult"
QT_MOC_LITERAL(18, 291, 22), // "preserveOpacityChanged"
QT_MOC_LITERAL(19, 314, 7), // "enabled"
QT_MOC_LITERAL(20, 322, 23), // "useHighDefShaderChanged"
QT_MOC_LITERAL(21, 346, 17), // "drawSlicesChanged"
QT_MOC_LITERAL(22, 364, 22), // "drawSliceFramesChanged"
QT_MOC_LITERAL(23, 387, 22), // "sliceFrameColorChanged"
QT_MOC_LITERAL(24, 410, 5), // "color"
QT_MOC_LITERAL(25, 416, 23), // "sliceFrameWidthsChanged"
QT_MOC_LITERAL(26, 440, 6), // "values"
QT_MOC_LITERAL(27, 447, 21), // "sliceFrameGapsChanged"
QT_MOC_LITERAL(28, 469, 28), // "sliceFrameThicknessesChanged"
QT_MOC_LITERAL(29, 498, 12), // "textureWidth"
QT_MOC_LITERAL(30, 511, 13), // "textureHeight"
QT_MOC_LITERAL(31, 525, 12), // "textureDepth"
QT_MOC_LITERAL(32, 538, 11), // "sliceIndexX"
QT_MOC_LITERAL(33, 550, 11), // "sliceIndexY"
QT_MOC_LITERAL(34, 562, 11), // "sliceIndexZ"
QT_MOC_LITERAL(35, 574, 10), // "colorTable"
QT_MOC_LITERAL(36, 585, 13), // "QVector<QRgb>"
QT_MOC_LITERAL(37, 599, 11), // "textureData"
QT_MOC_LITERAL(38, 611, 15), // "alphaMultiplier"
QT_MOC_LITERAL(39, 627, 15), // "preserveOpacity"
QT_MOC_LITERAL(40, 643, 16), // "useHighDefShader"
QT_MOC_LITERAL(41, 660, 10), // "drawSlices"
QT_MOC_LITERAL(42, 671, 15), // "drawSliceFrames"
QT_MOC_LITERAL(43, 687, 15), // "sliceFrameColor"
QT_MOC_LITERAL(44, 703, 16), // "sliceFrameWidths"
QT_MOC_LITERAL(45, 720, 14), // "sliceFrameGaps"
QT_MOC_LITERAL(46, 735, 21) // "sliceFrameThicknesses"

    },
    "QtDataVisualization::QCustom3DVolume\0"
    "textureWidthChanged\0\0value\0"
    "textureHeightChanged\0textureDepthChanged\0"
    "sliceIndexXChanged\0sliceIndexYChanged\0"
    "sliceIndexZChanged\0colorTableChanged\0"
    "textureDataChanged\0QVector<uchar>*\0"
    "data\0textureFormatChanged\0QImage::Format\0"
    "format\0alphaMultiplierChanged\0mult\0"
    "preserveOpacityChanged\0enabled\0"
    "useHighDefShaderChanged\0drawSlicesChanged\0"
    "drawSliceFramesChanged\0sliceFrameColorChanged\0"
    "color\0sliceFrameWidthsChanged\0values\0"
    "sliceFrameGapsChanged\0"
    "sliceFrameThicknessesChanged\0textureWidth\0"
    "textureHeight\0textureDepth\0sliceIndexX\0"
    "sliceIndexY\0sliceIndexZ\0colorTable\0"
    "QVector<QRgb>\0textureData\0alphaMultiplier\0"
    "preserveOpacity\0useHighDefShader\0"
    "drawSlices\0drawSliceFrames\0sliceFrameColor\0"
    "sliceFrameWidths\0sliceFrameGaps\0"
    "sliceFrameThicknesses"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QtDataVisualization__QCustom3DVolume[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      18,   14, // methods
      17,  156, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      18,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  104,    2, 0x06 /* Public */,
       4,    1,  107,    2, 0x06 /* Public */,
       5,    1,  110,    2, 0x06 /* Public */,
       6,    1,  113,    2, 0x06 /* Public */,
       7,    1,  116,    2, 0x06 /* Public */,
       8,    1,  119,    2, 0x06 /* Public */,
       9,    0,  122,    2, 0x06 /* Public */,
      10,    1,  123,    2, 0x06 /* Public */,
      13,    1,  126,    2, 0x06 /* Public */,
      16,    1,  129,    2, 0x06 /* Public */,
      18,    1,  132,    2, 0x06 /* Public */,
      20,    1,  135,    2, 0x06 /* Public */,
      21,    1,  138,    2, 0x06 /* Public */,
      22,    1,  141,    2, 0x06 /* Public */,
      23,    1,  144,    2, 0x06 /* Public */,
      25,    1,  147,    2, 0x06 /* Public */,
      27,    1,  150,    2, 0x06 /* Public */,
      28,    1,  153,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 11,   12,
    QMetaType::Void, 0x80000000 | 14,   15,
    QMetaType::Void, QMetaType::Float,   17,
    QMetaType::Void, QMetaType::Bool,   19,
    QMetaType::Void, QMetaType::Bool,   19,
    QMetaType::Void, QMetaType::Bool,   19,
    QMetaType::Void, QMetaType::Bool,   19,
    QMetaType::Void, QMetaType::QColor,   24,
    QMetaType::Void, QMetaType::QVector3D,   26,
    QMetaType::Void, QMetaType::QVector3D,   26,
    QMetaType::Void, QMetaType::QVector3D,   26,

 // properties: name, type, flags
      29, QMetaType::Int, 0x00495103,
      30, QMetaType::Int, 0x00495103,
      31, QMetaType::Int, 0x00495103,
      32, QMetaType::Int, 0x00495103,
      33, QMetaType::Int, 0x00495103,
      34, QMetaType::Int, 0x00495103,
      35, 0x80000000 | 36, 0x0049510b,
      37, 0x80000000 | 11, 0x0049510b,
      38, QMetaType::Float, 0x00495103,
      39, QMetaType::Bool, 0x00495103,
      40, QMetaType::Bool, 0x00495103,
      41, QMetaType::Bool, 0x00495103,
      42, QMetaType::Bool, 0x00495103,
      43, QMetaType::QColor, 0x00495103,
      44, QMetaType::QVector3D, 0x00495103,
      45, QMetaType::QVector3D, 0x00495103,
      46, QMetaType::QVector3D, 0x00495103,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,
       4,
       5,
       6,
       7,
       9,
      10,
      11,
      12,
      13,
      14,
      15,
      16,
      17,

       0        // eod
};

void QtDataVisualization::QCustom3DVolume::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QCustom3DVolume *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->textureWidthChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->textureHeightChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->textureDepthChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->sliceIndexXChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->sliceIndexYChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->sliceIndexZChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 6: _t->colorTableChanged(); break;
        case 7: _t->textureDataChanged((*reinterpret_cast< QVector<uchar>*(*)>(_a[1]))); break;
        case 8: _t->textureFormatChanged((*reinterpret_cast< QImage::Format(*)>(_a[1]))); break;
        case 9: _t->alphaMultiplierChanged((*reinterpret_cast< float(*)>(_a[1]))); break;
        case 10: _t->preserveOpacityChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 11: _t->useHighDefShaderChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 12: _t->drawSlicesChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 13: _t->drawSliceFramesChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 14: _t->sliceFrameColorChanged((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 15: _t->sliceFrameWidthsChanged((*reinterpret_cast< const QVector3D(*)>(_a[1]))); break;
        case 16: _t->sliceFrameGapsChanged((*reinterpret_cast< const QVector3D(*)>(_a[1]))); break;
        case 17: _t->sliceFrameThicknessesChanged((*reinterpret_cast< const QVector3D(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QCustom3DVolume::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QCustom3DVolume::textureWidthChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QCustom3DVolume::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QCustom3DVolume::textureHeightChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QCustom3DVolume::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QCustom3DVolume::textureDepthChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QCustom3DVolume::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QCustom3DVolume::sliceIndexXChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QCustom3DVolume::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QCustom3DVolume::sliceIndexYChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QCustom3DVolume::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QCustom3DVolume::sliceIndexZChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QCustom3DVolume::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QCustom3DVolume::colorTableChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QCustom3DVolume::*)(QVector<uchar> * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QCustom3DVolume::textureDataChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QCustom3DVolume::*)(QImage::Format );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QCustom3DVolume::textureFormatChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QCustom3DVolume::*)(float );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QCustom3DVolume::alphaMultiplierChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (QCustom3DVolume::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QCustom3DVolume::preserveOpacityChanged)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (QCustom3DVolume::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QCustom3DVolume::useHighDefShaderChanged)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (QCustom3DVolume::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QCustom3DVolume::drawSlicesChanged)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (QCustom3DVolume::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QCustom3DVolume::drawSliceFramesChanged)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (QCustom3DVolume::*)(const QColor & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QCustom3DVolume::sliceFrameColorChanged)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (QCustom3DVolume::*)(const QVector3D & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QCustom3DVolume::sliceFrameWidthsChanged)) {
                *result = 15;
                return;
            }
        }
        {
            using _t = void (QCustom3DVolume::*)(const QVector3D & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QCustom3DVolume::sliceFrameGapsChanged)) {
                *result = 16;
                return;
            }
        }
        {
            using _t = void (QCustom3DVolume::*)(const QVector3D & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QCustom3DVolume::sliceFrameThicknessesChanged)) {
                *result = 17;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QCustom3DVolume *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< int*>(_v) = _t->textureWidth(); break;
        case 1: *reinterpret_cast< int*>(_v) = _t->textureHeight(); break;
        case 2: *reinterpret_cast< int*>(_v) = _t->textureDepth(); break;
        case 3: *reinterpret_cast< int*>(_v) = _t->sliceIndexX(); break;
        case 4: *reinterpret_cast< int*>(_v) = _t->sliceIndexY(); break;
        case 5: *reinterpret_cast< int*>(_v) = _t->sliceIndexZ(); break;
        case 6: *reinterpret_cast< QVector<QRgb>*>(_v) = _t->colorTable(); break;
        case 7: *reinterpret_cast< QVector<uchar>**>(_v) = _t->textureData(); break;
        case 8: *reinterpret_cast< float*>(_v) = _t->alphaMultiplier(); break;
        case 9: *reinterpret_cast< bool*>(_v) = _t->preserveOpacity(); break;
        case 10: *reinterpret_cast< bool*>(_v) = _t->useHighDefShader(); break;
        case 11: *reinterpret_cast< bool*>(_v) = _t->drawSlices(); break;
        case 12: *reinterpret_cast< bool*>(_v) = _t->drawSliceFrames(); break;
        case 13: *reinterpret_cast< QColor*>(_v) = _t->sliceFrameColor(); break;
        case 14: *reinterpret_cast< QVector3D*>(_v) = _t->sliceFrameWidths(); break;
        case 15: *reinterpret_cast< QVector3D*>(_v) = _t->sliceFrameGaps(); break;
        case 16: *reinterpret_cast< QVector3D*>(_v) = _t->sliceFrameThicknesses(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QCustom3DVolume *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setTextureWidth(*reinterpret_cast< int*>(_v)); break;
        case 1: _t->setTextureHeight(*reinterpret_cast< int*>(_v)); break;
        case 2: _t->setTextureDepth(*reinterpret_cast< int*>(_v)); break;
        case 3: _t->setSliceIndexX(*reinterpret_cast< int*>(_v)); break;
        case 4: _t->setSliceIndexY(*reinterpret_cast< int*>(_v)); break;
        case 5: _t->setSliceIndexZ(*reinterpret_cast< int*>(_v)); break;
        case 6: _t->setColorTable(*reinterpret_cast< QVector<QRgb>*>(_v)); break;
        case 7: _t->setTextureData(*reinterpret_cast< QVector<uchar>**>(_v)); break;
        case 8: _t->setAlphaMultiplier(*reinterpret_cast< float*>(_v)); break;
        case 9: _t->setPreserveOpacity(*reinterpret_cast< bool*>(_v)); break;
        case 10: _t->setUseHighDefShader(*reinterpret_cast< bool*>(_v)); break;
        case 11: _t->setDrawSlices(*reinterpret_cast< bool*>(_v)); break;
        case 12: _t->setDrawSliceFrames(*reinterpret_cast< bool*>(_v)); break;
        case 13: _t->setSliceFrameColor(*reinterpret_cast< QColor*>(_v)); break;
        case 14: _t->setSliceFrameWidths(*reinterpret_cast< QVector3D*>(_v)); break;
        case 15: _t->setSliceFrameGaps(*reinterpret_cast< QVector3D*>(_v)); break;
        case 16: _t->setSliceFrameThicknesses(*reinterpret_cast< QVector3D*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QtDataVisualization::QCustom3DVolume::staticMetaObject = { {
    QMetaObject::SuperData::link<QCustom3DItem::staticMetaObject>(),
    qt_meta_stringdata_QtDataVisualization__QCustom3DVolume.data,
    qt_meta_data_QtDataVisualization__QCustom3DVolume,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QtDataVisualization::QCustom3DVolume::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtDataVisualization::QCustom3DVolume::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QtDataVisualization__QCustom3DVolume.stringdata0))
        return static_cast<void*>(this);
    return QCustom3DItem::qt_metacast(_clname);
}

int QtDataVisualization::QCustom3DVolume::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QCustom3DItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 18;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 17;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 17;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 17;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 17;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 17;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QtDataVisualization::QCustom3DVolume::textureWidthChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QtDataVisualization::QCustom3DVolume::textureHeightChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QtDataVisualization::QCustom3DVolume::textureDepthChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QtDataVisualization::QCustom3DVolume::sliceIndexXChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QtDataVisualization::QCustom3DVolume::sliceIndexYChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void QtDataVisualization::QCustom3DVolume::sliceIndexZChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void QtDataVisualization::QCustom3DVolume::colorTableChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QtDataVisualization::QCustom3DVolume::textureDataChanged(QVector<uchar> * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void QtDataVisualization::QCustom3DVolume::textureFormatChanged(QImage::Format _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void QtDataVisualization::QCustom3DVolume::alphaMultiplierChanged(float _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void QtDataVisualization::QCustom3DVolume::preserveOpacityChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 10, _a);
}

// SIGNAL 11
void QtDataVisualization::QCustom3DVolume::useHighDefShaderChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 11, _a);
}

// SIGNAL 12
void QtDataVisualization::QCustom3DVolume::drawSlicesChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 12, _a);
}

// SIGNAL 13
void QtDataVisualization::QCustom3DVolume::drawSliceFramesChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 13, _a);
}

// SIGNAL 14
void QtDataVisualization::QCustom3DVolume::sliceFrameColorChanged(const QColor & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 14, _a);
}

// SIGNAL 15
void QtDataVisualization::QCustom3DVolume::sliceFrameWidthsChanged(const QVector3D & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 15, _a);
}

// SIGNAL 16
void QtDataVisualization::QCustom3DVolume::sliceFrameGapsChanged(const QVector3D & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 16, _a);
}

// SIGNAL 17
void QtDataVisualization::QCustom3DVolume::sliceFrameThicknessesChanged(const QVector3D & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 17, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
