/****************************************************************************
** Meta object code from reading C++ file 'qabstract3dgraph.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kde-qtdatavis3d/src/datavisualization/engine/qabstract3dgraph.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qabstract3dgraph.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QtDataVisualization__QAbstract3DGraph_t {
    QByteArrayData data[88];
    char stringdata0[1467];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QtDataVisualization__QAbstract3DGraph_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QtDataVisualization__QAbstract3DGraph_t qt_meta_stringdata_QtDataVisualization__QAbstract3DGraph = {
    {
QT_MOC_LITERAL(0, 0, 37), // "QtDataVisualization::QAbstrac..."
QT_MOC_LITERAL(1, 38, 25), // "activeInputHandlerChanged"
QT_MOC_LITERAL(2, 64, 0), // ""
QT_MOC_LITERAL(3, 65, 24), // "QAbstract3DInputHandler*"
QT_MOC_LITERAL(4, 90, 12), // "inputHandler"
QT_MOC_LITERAL(5, 103, 18), // "activeThemeChanged"
QT_MOC_LITERAL(6, 122, 9), // "Q3DTheme*"
QT_MOC_LITERAL(7, 132, 5), // "theme"
QT_MOC_LITERAL(8, 138, 20), // "selectionModeChanged"
QT_MOC_LITERAL(9, 159, 32), // "QAbstract3DGraph::SelectionFlags"
QT_MOC_LITERAL(10, 192, 4), // "mode"
QT_MOC_LITERAL(11, 197, 20), // "shadowQualityChanged"
QT_MOC_LITERAL(12, 218, 31), // "QAbstract3DGraph::ShadowQuality"
QT_MOC_LITERAL(13, 250, 7), // "quality"
QT_MOC_LITERAL(14, 258, 22), // "selectedElementChanged"
QT_MOC_LITERAL(15, 281, 29), // "QAbstract3DGraph::ElementType"
QT_MOC_LITERAL(16, 311, 4), // "type"
QT_MOC_LITERAL(17, 316, 17), // "measureFpsChanged"
QT_MOC_LITERAL(18, 334, 7), // "enabled"
QT_MOC_LITERAL(19, 342, 17), // "currentFpsChanged"
QT_MOC_LITERAL(20, 360, 3), // "fps"
QT_MOC_LITERAL(21, 364, 22), // "orthoProjectionChanged"
QT_MOC_LITERAL(22, 387, 18), // "aspectRatioChanged"
QT_MOC_LITERAL(23, 406, 5), // "ratio"
QT_MOC_LITERAL(24, 412, 24), // "optimizationHintsChanged"
QT_MOC_LITERAL(25, 437, 35), // "QAbstract3DGraph::Optimizatio..."
QT_MOC_LITERAL(26, 473, 5), // "hints"
QT_MOC_LITERAL(27, 479, 12), // "polarChanged"
QT_MOC_LITERAL(28, 492, 24), // "radialLabelOffsetChanged"
QT_MOC_LITERAL(29, 517, 6), // "offset"
QT_MOC_LITERAL(30, 524, 28), // "horizontalAspectRatioChanged"
QT_MOC_LITERAL(31, 553, 17), // "reflectionChanged"
QT_MOC_LITERAL(32, 571, 19), // "reflectivityChanged"
QT_MOC_LITERAL(33, 591, 12), // "reflectivity"
QT_MOC_LITERAL(34, 604, 13), // "localeChanged"
QT_MOC_LITERAL(35, 618, 6), // "locale"
QT_MOC_LITERAL(36, 625, 27), // "queriedGraphPositionChanged"
QT_MOC_LITERAL(37, 653, 4), // "data"
QT_MOC_LITERAL(38, 658, 13), // "marginChanged"
QT_MOC_LITERAL(39, 672, 6), // "margin"
QT_MOC_LITERAL(40, 679, 18), // "activeInputHandler"
QT_MOC_LITERAL(41, 698, 11), // "activeTheme"
QT_MOC_LITERAL(42, 710, 13), // "selectionMode"
QT_MOC_LITERAL(43, 724, 14), // "SelectionFlags"
QT_MOC_LITERAL(44, 739, 13), // "shadowQuality"
QT_MOC_LITERAL(45, 753, 13), // "ShadowQuality"
QT_MOC_LITERAL(46, 767, 5), // "scene"
QT_MOC_LITERAL(47, 773, 9), // "Q3DScene*"
QT_MOC_LITERAL(48, 783, 10), // "measureFps"
QT_MOC_LITERAL(49, 794, 10), // "currentFps"
QT_MOC_LITERAL(50, 805, 15), // "orthoProjection"
QT_MOC_LITERAL(51, 821, 15), // "selectedElement"
QT_MOC_LITERAL(52, 837, 11), // "ElementType"
QT_MOC_LITERAL(53, 849, 11), // "aspectRatio"
QT_MOC_LITERAL(54, 861, 17), // "optimizationHints"
QT_MOC_LITERAL(55, 879, 17), // "OptimizationHints"
QT_MOC_LITERAL(56, 897, 5), // "polar"
QT_MOC_LITERAL(57, 903, 17), // "radialLabelOffset"
QT_MOC_LITERAL(58, 921, 21), // "horizontalAspectRatio"
QT_MOC_LITERAL(59, 943, 10), // "reflection"
QT_MOC_LITERAL(60, 954, 20), // "queriedGraphPosition"
QT_MOC_LITERAL(61, 975, 13), // "SelectionFlag"
QT_MOC_LITERAL(62, 989, 13), // "SelectionNone"
QT_MOC_LITERAL(63, 1003, 13), // "SelectionItem"
QT_MOC_LITERAL(64, 1017, 12), // "SelectionRow"
QT_MOC_LITERAL(65, 1030, 19), // "SelectionItemAndRow"
QT_MOC_LITERAL(66, 1050, 15), // "SelectionColumn"
QT_MOC_LITERAL(67, 1066, 22), // "SelectionItemAndColumn"
QT_MOC_LITERAL(68, 1089, 21), // "SelectionRowAndColumn"
QT_MOC_LITERAL(69, 1111, 25), // "SelectionItemRowAndColumn"
QT_MOC_LITERAL(70, 1137, 14), // "SelectionSlice"
QT_MOC_LITERAL(71, 1152, 20), // "SelectionMultiSeries"
QT_MOC_LITERAL(72, 1173, 17), // "ShadowQualityNone"
QT_MOC_LITERAL(73, 1191, 16), // "ShadowQualityLow"
QT_MOC_LITERAL(74, 1208, 19), // "ShadowQualityMedium"
QT_MOC_LITERAL(75, 1228, 17), // "ShadowQualityHigh"
QT_MOC_LITERAL(76, 1246, 20), // "ShadowQualitySoftLow"
QT_MOC_LITERAL(77, 1267, 23), // "ShadowQualitySoftMedium"
QT_MOC_LITERAL(78, 1291, 21), // "ShadowQualitySoftHigh"
QT_MOC_LITERAL(79, 1313, 11), // "ElementNone"
QT_MOC_LITERAL(80, 1325, 13), // "ElementSeries"
QT_MOC_LITERAL(81, 1339, 17), // "ElementAxisXLabel"
QT_MOC_LITERAL(82, 1357, 17), // "ElementAxisYLabel"
QT_MOC_LITERAL(83, 1375, 17), // "ElementAxisZLabel"
QT_MOC_LITERAL(84, 1393, 17), // "ElementCustomItem"
QT_MOC_LITERAL(85, 1411, 16), // "OptimizationHint"
QT_MOC_LITERAL(86, 1428, 19), // "OptimizationDefault"
QT_MOC_LITERAL(87, 1448, 18) // "OptimizationStatic"

    },
    "QtDataVisualization::QAbstract3DGraph\0"
    "activeInputHandlerChanged\0\0"
    "QAbstract3DInputHandler*\0inputHandler\0"
    "activeThemeChanged\0Q3DTheme*\0theme\0"
    "selectionModeChanged\0"
    "QAbstract3DGraph::SelectionFlags\0mode\0"
    "shadowQualityChanged\0"
    "QAbstract3DGraph::ShadowQuality\0quality\0"
    "selectedElementChanged\0"
    "QAbstract3DGraph::ElementType\0type\0"
    "measureFpsChanged\0enabled\0currentFpsChanged\0"
    "fps\0orthoProjectionChanged\0"
    "aspectRatioChanged\0ratio\0"
    "optimizationHintsChanged\0"
    "QAbstract3DGraph::OptimizationHints\0"
    "hints\0polarChanged\0radialLabelOffsetChanged\0"
    "offset\0horizontalAspectRatioChanged\0"
    "reflectionChanged\0reflectivityChanged\0"
    "reflectivity\0localeChanged\0locale\0"
    "queriedGraphPositionChanged\0data\0"
    "marginChanged\0margin\0activeInputHandler\0"
    "activeTheme\0selectionMode\0SelectionFlags\0"
    "shadowQuality\0ShadowQuality\0scene\0"
    "Q3DScene*\0measureFps\0currentFps\0"
    "orthoProjection\0selectedElement\0"
    "ElementType\0aspectRatio\0optimizationHints\0"
    "OptimizationHints\0polar\0radialLabelOffset\0"
    "horizontalAspectRatio\0reflection\0"
    "queriedGraphPosition\0SelectionFlag\0"
    "SelectionNone\0SelectionItem\0SelectionRow\0"
    "SelectionItemAndRow\0SelectionColumn\0"
    "SelectionItemAndColumn\0SelectionRowAndColumn\0"
    "SelectionItemRowAndColumn\0SelectionSlice\0"
    "SelectionMultiSeries\0ShadowQualityNone\0"
    "ShadowQualityLow\0ShadowQualityMedium\0"
    "ShadowQualityHigh\0ShadowQualitySoftLow\0"
    "ShadowQualitySoftMedium\0ShadowQualitySoftHigh\0"
    "ElementNone\0ElementSeries\0ElementAxisXLabel\0"
    "ElementAxisYLabel\0ElementAxisZLabel\0"
    "ElementCustomItem\0OptimizationHint\0"
    "OptimizationDefault\0OptimizationStatic"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QtDataVisualization__QAbstract3DGraph[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      18,   14, // methods
      19,  158, // properties
       6,  234, // enums/sets
       0,    0, // constructors
       0,       // flags
      18,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  104,    2, 0x06 /* Public */,
       5,    1,  107,    2, 0x06 /* Public */,
       8,    1,  110,    2, 0x06 /* Public */,
      11,    1,  113,    2, 0x06 /* Public */,
      14,    1,  116,    2, 0x06 /* Public */,
      17,    1,  119,    2, 0x06 /* Public */,
      19,    1,  122,    2, 0x06 /* Public */,
      21,    1,  125,    2, 0x06 /* Public */,
      22,    1,  128,    2, 0x06 /* Public */,
      24,    1,  131,    2, 0x06 /* Public */,
      27,    1,  134,    2, 0x06 /* Public */,
      28,    1,  137,    2, 0x06 /* Public */,
      30,    1,  140,    2, 0x06 /* Public */,
      31,    1,  143,    2, 0x06 /* Public */,
      32,    1,  146,    2, 0x06 /* Public */,
      34,    1,  149,    2, 0x06 /* Public */,
      36,    1,  152,    2, 0x06 /* Public */,
      38,    1,  155,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 15,   16,
    QMetaType::Void, QMetaType::Bool,   18,
    QMetaType::Void, QMetaType::QReal,   20,
    QMetaType::Void, QMetaType::Bool,   18,
    QMetaType::Void, QMetaType::QReal,   23,
    QMetaType::Void, 0x80000000 | 25,   26,
    QMetaType::Void, QMetaType::Bool,   18,
    QMetaType::Void, QMetaType::Float,   29,
    QMetaType::Void, QMetaType::QReal,   23,
    QMetaType::Void, QMetaType::Bool,   18,
    QMetaType::Void, QMetaType::QReal,   33,
    QMetaType::Void, QMetaType::QLocale,   35,
    QMetaType::Void, QMetaType::QVector3D,   37,
    QMetaType::Void, QMetaType::QReal,   39,

 // properties: name, type, flags
      40, 0x80000000 | 3, 0x0049510b,
      41, 0x80000000 | 6, 0x0049510b,
      42, 0x80000000 | 43, 0x0049510b,
      44, 0x80000000 | 45, 0x0049510b,
      46, 0x80000000 | 47, 0x00095009,
      48, QMetaType::Bool, 0x00495103,
      49, QMetaType::QReal, 0x00495001,
      50, QMetaType::Bool, 0x00495103,
      51, 0x80000000 | 52, 0x00495009,
      53, QMetaType::QReal, 0x00495103,
      54, 0x80000000 | 55, 0x0049510b,
      56, QMetaType::Bool, 0x00495103,
      57, QMetaType::Float, 0x00495103,
      58, QMetaType::QReal, 0x00495103,
      59, QMetaType::Bool, 0x00495103,
      33, QMetaType::QReal, 0x00495103,
      35, QMetaType::QLocale, 0x00495103,
      60, QMetaType::QVector3D, 0x00495001,
      39, QMetaType::QReal, 0x00495103,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,
       0,
       5,
       6,
       7,
       4,
       8,
       9,
      10,
      11,
      12,
      13,
      14,
      15,
      16,
      17,

 // enums: name, alias, flags, count, data
      61,   61, 0x1,   10,  264,
      43,   61, 0x1,   10,  284,
      45,   45, 0x0,    7,  304,
      52,   52, 0x0,    6,  318,
      85,   85, 0x1,    2,  330,
      55,   85, 0x1,    2,  334,

 // enum data: key, value
      62, uint(QtDataVisualization::QAbstract3DGraph::SelectionNone),
      63, uint(QtDataVisualization::QAbstract3DGraph::SelectionItem),
      64, uint(QtDataVisualization::QAbstract3DGraph::SelectionRow),
      65, uint(QtDataVisualization::QAbstract3DGraph::SelectionItemAndRow),
      66, uint(QtDataVisualization::QAbstract3DGraph::SelectionColumn),
      67, uint(QtDataVisualization::QAbstract3DGraph::SelectionItemAndColumn),
      68, uint(QtDataVisualization::QAbstract3DGraph::SelectionRowAndColumn),
      69, uint(QtDataVisualization::QAbstract3DGraph::SelectionItemRowAndColumn),
      70, uint(QtDataVisualization::QAbstract3DGraph::SelectionSlice),
      71, uint(QtDataVisualization::QAbstract3DGraph::SelectionMultiSeries),
      62, uint(QtDataVisualization::QAbstract3DGraph::SelectionNone),
      63, uint(QtDataVisualization::QAbstract3DGraph::SelectionItem),
      64, uint(QtDataVisualization::QAbstract3DGraph::SelectionRow),
      65, uint(QtDataVisualization::QAbstract3DGraph::SelectionItemAndRow),
      66, uint(QtDataVisualization::QAbstract3DGraph::SelectionColumn),
      67, uint(QtDataVisualization::QAbstract3DGraph::SelectionItemAndColumn),
      68, uint(QtDataVisualization::QAbstract3DGraph::SelectionRowAndColumn),
      69, uint(QtDataVisualization::QAbstract3DGraph::SelectionItemRowAndColumn),
      70, uint(QtDataVisualization::QAbstract3DGraph::SelectionSlice),
      71, uint(QtDataVisualization::QAbstract3DGraph::SelectionMultiSeries),
      72, uint(QtDataVisualization::QAbstract3DGraph::ShadowQualityNone),
      73, uint(QtDataVisualization::QAbstract3DGraph::ShadowQualityLow),
      74, uint(QtDataVisualization::QAbstract3DGraph::ShadowQualityMedium),
      75, uint(QtDataVisualization::QAbstract3DGraph::ShadowQualityHigh),
      76, uint(QtDataVisualization::QAbstract3DGraph::ShadowQualitySoftLow),
      77, uint(QtDataVisualization::QAbstract3DGraph::ShadowQualitySoftMedium),
      78, uint(QtDataVisualization::QAbstract3DGraph::ShadowQualitySoftHigh),
      79, uint(QtDataVisualization::QAbstract3DGraph::ElementNone),
      80, uint(QtDataVisualization::QAbstract3DGraph::ElementSeries),
      81, uint(QtDataVisualization::QAbstract3DGraph::ElementAxisXLabel),
      82, uint(QtDataVisualization::QAbstract3DGraph::ElementAxisYLabel),
      83, uint(QtDataVisualization::QAbstract3DGraph::ElementAxisZLabel),
      84, uint(QtDataVisualization::QAbstract3DGraph::ElementCustomItem),
      86, uint(QtDataVisualization::QAbstract3DGraph::OptimizationDefault),
      87, uint(QtDataVisualization::QAbstract3DGraph::OptimizationStatic),
      86, uint(QtDataVisualization::QAbstract3DGraph::OptimizationDefault),
      87, uint(QtDataVisualization::QAbstract3DGraph::OptimizationStatic),

       0        // eod
};

void QtDataVisualization::QAbstract3DGraph::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QAbstract3DGraph *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->activeInputHandlerChanged((*reinterpret_cast< QAbstract3DInputHandler*(*)>(_a[1]))); break;
        case 1: _t->activeThemeChanged((*reinterpret_cast< Q3DTheme*(*)>(_a[1]))); break;
        case 2: _t->selectionModeChanged((*reinterpret_cast< QAbstract3DGraph::SelectionFlags(*)>(_a[1]))); break;
        case 3: _t->shadowQualityChanged((*reinterpret_cast< QAbstract3DGraph::ShadowQuality(*)>(_a[1]))); break;
        case 4: _t->selectedElementChanged((*reinterpret_cast< QAbstract3DGraph::ElementType(*)>(_a[1]))); break;
        case 5: _t->measureFpsChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 6: _t->currentFpsChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 7: _t->orthoProjectionChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 8: _t->aspectRatioChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 9: _t->optimizationHintsChanged((*reinterpret_cast< QAbstract3DGraph::OptimizationHints(*)>(_a[1]))); break;
        case 10: _t->polarChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 11: _t->radialLabelOffsetChanged((*reinterpret_cast< float(*)>(_a[1]))); break;
        case 12: _t->horizontalAspectRatioChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 13: _t->reflectionChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 14: _t->reflectivityChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 15: _t->localeChanged((*reinterpret_cast< const QLocale(*)>(_a[1]))); break;
        case 16: _t->queriedGraphPositionChanged((*reinterpret_cast< const QVector3D(*)>(_a[1]))); break;
        case 17: _t->marginChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAbstract3DInputHandler* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Q3DTheme* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QAbstract3DGraph::*)(QAbstract3DInputHandler * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QAbstract3DGraph::activeInputHandlerChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QAbstract3DGraph::*)(Q3DTheme * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QAbstract3DGraph::activeThemeChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QAbstract3DGraph::*)(QAbstract3DGraph::SelectionFlags );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QAbstract3DGraph::selectionModeChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QAbstract3DGraph::*)(QAbstract3DGraph::ShadowQuality );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QAbstract3DGraph::shadowQualityChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QAbstract3DGraph::*)(QAbstract3DGraph::ElementType );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QAbstract3DGraph::selectedElementChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QAbstract3DGraph::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QAbstract3DGraph::measureFpsChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QAbstract3DGraph::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QAbstract3DGraph::currentFpsChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QAbstract3DGraph::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QAbstract3DGraph::orthoProjectionChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QAbstract3DGraph::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QAbstract3DGraph::aspectRatioChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QAbstract3DGraph::*)(QAbstract3DGraph::OptimizationHints );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QAbstract3DGraph::optimizationHintsChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (QAbstract3DGraph::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QAbstract3DGraph::polarChanged)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (QAbstract3DGraph::*)(float );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QAbstract3DGraph::radialLabelOffsetChanged)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (QAbstract3DGraph::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QAbstract3DGraph::horizontalAspectRatioChanged)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (QAbstract3DGraph::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QAbstract3DGraph::reflectionChanged)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (QAbstract3DGraph::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QAbstract3DGraph::reflectivityChanged)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (QAbstract3DGraph::*)(const QLocale & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QAbstract3DGraph::localeChanged)) {
                *result = 15;
                return;
            }
        }
        {
            using _t = void (QAbstract3DGraph::*)(const QVector3D & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QAbstract3DGraph::queriedGraphPositionChanged)) {
                *result = 16;
                return;
            }
        }
        {
            using _t = void (QAbstract3DGraph::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QAbstract3DGraph::marginChanged)) {
                *result = 17;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 4:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Q3DScene* >(); break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Q3DTheme* >(); break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAbstract3DInputHandler* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QAbstract3DGraph *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QAbstract3DInputHandler**>(_v) = _t->activeInputHandler(); break;
        case 1: *reinterpret_cast< Q3DTheme**>(_v) = _t->activeTheme(); break;
        case 2: *reinterpret_cast<int*>(_v) = QFlag(_t->selectionMode()); break;
        case 3: *reinterpret_cast< ShadowQuality*>(_v) = _t->shadowQuality(); break;
        case 4: *reinterpret_cast< Q3DScene**>(_v) = _t->scene(); break;
        case 5: *reinterpret_cast< bool*>(_v) = _t->measureFps(); break;
        case 6: *reinterpret_cast< qreal*>(_v) = _t->currentFps(); break;
        case 7: *reinterpret_cast< bool*>(_v) = _t->isOrthoProjection(); break;
        case 8: *reinterpret_cast< ElementType*>(_v) = _t->selectedElement(); break;
        case 9: *reinterpret_cast< qreal*>(_v) = _t->aspectRatio(); break;
        case 10: *reinterpret_cast<int*>(_v) = QFlag(_t->optimizationHints()); break;
        case 11: *reinterpret_cast< bool*>(_v) = _t->isPolar(); break;
        case 12: *reinterpret_cast< float*>(_v) = _t->radialLabelOffset(); break;
        case 13: *reinterpret_cast< qreal*>(_v) = _t->horizontalAspectRatio(); break;
        case 14: *reinterpret_cast< bool*>(_v) = _t->isReflection(); break;
        case 15: *reinterpret_cast< qreal*>(_v) = _t->reflectivity(); break;
        case 16: *reinterpret_cast< QLocale*>(_v) = _t->locale(); break;
        case 17: *reinterpret_cast< QVector3D*>(_v) = _t->queriedGraphPosition(); break;
        case 18: *reinterpret_cast< qreal*>(_v) = _t->margin(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QAbstract3DGraph *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setActiveInputHandler(*reinterpret_cast< QAbstract3DInputHandler**>(_v)); break;
        case 1: _t->setActiveTheme(*reinterpret_cast< Q3DTheme**>(_v)); break;
        case 2: _t->setSelectionMode(QFlag(*reinterpret_cast<int*>(_v))); break;
        case 3: _t->setShadowQuality(*reinterpret_cast< ShadowQuality*>(_v)); break;
        case 5: _t->setMeasureFps(*reinterpret_cast< bool*>(_v)); break;
        case 7: _t->setOrthoProjection(*reinterpret_cast< bool*>(_v)); break;
        case 9: _t->setAspectRatio(*reinterpret_cast< qreal*>(_v)); break;
        case 10: _t->setOptimizationHints(QFlag(*reinterpret_cast<int*>(_v))); break;
        case 11: _t->setPolar(*reinterpret_cast< bool*>(_v)); break;
        case 12: _t->setRadialLabelOffset(*reinterpret_cast< float*>(_v)); break;
        case 13: _t->setHorizontalAspectRatio(*reinterpret_cast< qreal*>(_v)); break;
        case 14: _t->setReflection(*reinterpret_cast< bool*>(_v)); break;
        case 15: _t->setReflectivity(*reinterpret_cast< qreal*>(_v)); break;
        case 16: _t->setLocale(*reinterpret_cast< QLocale*>(_v)); break;
        case 18: _t->setMargin(*reinterpret_cast< qreal*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QtDataVisualization::QAbstract3DGraph::staticMetaObject = { {
    QMetaObject::SuperData::link<QWindow::staticMetaObject>(),
    qt_meta_stringdata_QtDataVisualization__QAbstract3DGraph.data,
    qt_meta_data_QtDataVisualization__QAbstract3DGraph,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QtDataVisualization::QAbstract3DGraph::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtDataVisualization::QAbstract3DGraph::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QtDataVisualization__QAbstract3DGraph.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QOpenGLFunctions"))
        return static_cast< QOpenGLFunctions*>(this);
    return QWindow::qt_metacast(_clname);
}

int QtDataVisualization::QAbstract3DGraph::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 19;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 19;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 19;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 19;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 19;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QtDataVisualization::QAbstract3DGraph::activeInputHandlerChanged(QAbstract3DInputHandler * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QtDataVisualization::QAbstract3DGraph::activeThemeChanged(Q3DTheme * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QtDataVisualization::QAbstract3DGraph::selectionModeChanged(QAbstract3DGraph::SelectionFlags _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QtDataVisualization::QAbstract3DGraph::shadowQualityChanged(QAbstract3DGraph::ShadowQuality _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QtDataVisualization::QAbstract3DGraph::selectedElementChanged(QAbstract3DGraph::ElementType _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void QtDataVisualization::QAbstract3DGraph::measureFpsChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void QtDataVisualization::QAbstract3DGraph::currentFpsChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void QtDataVisualization::QAbstract3DGraph::orthoProjectionChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void QtDataVisualization::QAbstract3DGraph::aspectRatioChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void QtDataVisualization::QAbstract3DGraph::optimizationHintsChanged(QAbstract3DGraph::OptimizationHints _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void QtDataVisualization::QAbstract3DGraph::polarChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 10, _a);
}

// SIGNAL 11
void QtDataVisualization::QAbstract3DGraph::radialLabelOffsetChanged(float _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 11, _a);
}

// SIGNAL 12
void QtDataVisualization::QAbstract3DGraph::horizontalAspectRatioChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 12, _a);
}

// SIGNAL 13
void QtDataVisualization::QAbstract3DGraph::reflectionChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 13, _a);
}

// SIGNAL 14
void QtDataVisualization::QAbstract3DGraph::reflectivityChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 14, _a);
}

// SIGNAL 15
void QtDataVisualization::QAbstract3DGraph::localeChanged(const QLocale & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 15, _a);
}

// SIGNAL 16
void QtDataVisualization::QAbstract3DGraph::queriedGraphPositionChanged(const QVector3D & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 16, _a);
}

// SIGNAL 17
void QtDataVisualization::QAbstract3DGraph::marginChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 17, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
