/****************************************************************************
** Meta object code from reading C++ file 'declarativebars_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kde-qtdatavis3d/src/datavisualizationqml2/declarativebars_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'declarativebars_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QtDataVisualization__DeclarativeBars_t {
    QByteArrayData data[45];
    char stringdata0[676];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QtDataVisualization__DeclarativeBars_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QtDataVisualization__DeclarativeBars_t qt_meta_stringdata_QtDataVisualization__DeclarativeBars = {
    {
QT_MOC_LITERAL(0, 0, 36), // "QtDataVisualization::Declarat..."
QT_MOC_LITERAL(1, 37, 15), // "DefaultProperty"
QT_MOC_LITERAL(2, 53, 10), // "seriesList"
QT_MOC_LITERAL(3, 64, 14), // "rowAxisChanged"
QT_MOC_LITERAL(4, 79, 0), // ""
QT_MOC_LITERAL(5, 80, 16), // "QCategory3DAxis*"
QT_MOC_LITERAL(6, 97, 4), // "axis"
QT_MOC_LITERAL(7, 102, 16), // "valueAxisChanged"
QT_MOC_LITERAL(8, 119, 13), // "QValue3DAxis*"
QT_MOC_LITERAL(9, 133, 17), // "columnAxisChanged"
QT_MOC_LITERAL(10, 151, 25), // "multiSeriesUniformChanged"
QT_MOC_LITERAL(11, 177, 7), // "uniform"
QT_MOC_LITERAL(12, 185, 19), // "barThicknessChanged"
QT_MOC_LITERAL(13, 205, 14), // "thicknessRatio"
QT_MOC_LITERAL(14, 220, 17), // "barSpacingChanged"
QT_MOC_LITERAL(15, 238, 7), // "spacing"
QT_MOC_LITERAL(16, 246, 25), // "barSpacingRelativeChanged"
QT_MOC_LITERAL(17, 272, 8), // "relative"
QT_MOC_LITERAL(18, 281, 19), // "meshFileNameChanged"
QT_MOC_LITERAL(19, 301, 8), // "filename"
QT_MOC_LITERAL(20, 310, 20), // "primarySeriesChanged"
QT_MOC_LITERAL(21, 331, 13), // "QBar3DSeries*"
QT_MOC_LITERAL(22, 345, 6), // "series"
QT_MOC_LITERAL(23, 352, 21), // "selectedSeriesChanged"
QT_MOC_LITERAL(24, 374, 17), // "floorLevelChanged"
QT_MOC_LITERAL(25, 392, 5), // "level"
QT_MOC_LITERAL(26, 398, 18), // "handleAxisXChanged"
QT_MOC_LITERAL(27, 417, 16), // "QAbstract3DAxis*"
QT_MOC_LITERAL(28, 434, 18), // "handleAxisYChanged"
QT_MOC_LITERAL(29, 453, 18), // "handleAxisZChanged"
QT_MOC_LITERAL(30, 472, 9), // "addSeries"
QT_MOC_LITERAL(31, 482, 12), // "removeSeries"
QT_MOC_LITERAL(32, 495, 12), // "insertSeries"
QT_MOC_LITERAL(33, 508, 5), // "index"
QT_MOC_LITERAL(34, 514, 7), // "rowAxis"
QT_MOC_LITERAL(35, 522, 9), // "valueAxis"
QT_MOC_LITERAL(36, 532, 10), // "columnAxis"
QT_MOC_LITERAL(37, 543, 18), // "multiSeriesUniform"
QT_MOC_LITERAL(38, 562, 12), // "barThickness"
QT_MOC_LITERAL(39, 575, 10), // "barSpacing"
QT_MOC_LITERAL(40, 586, 18), // "barSpacingRelative"
QT_MOC_LITERAL(41, 605, 30), // "QQmlListProperty<QBar3DSeries>"
QT_MOC_LITERAL(42, 636, 14), // "selectedSeries"
QT_MOC_LITERAL(43, 651, 13), // "primarySeries"
QT_MOC_LITERAL(44, 665, 10) // "floorLevel"

    },
    "QtDataVisualization::DeclarativeBars\0"
    "DefaultProperty\0seriesList\0rowAxisChanged\0"
    "\0QCategory3DAxis*\0axis\0valueAxisChanged\0"
    "QValue3DAxis*\0columnAxisChanged\0"
    "multiSeriesUniformChanged\0uniform\0"
    "barThicknessChanged\0thicknessRatio\0"
    "barSpacingChanged\0spacing\0"
    "barSpacingRelativeChanged\0relative\0"
    "meshFileNameChanged\0filename\0"
    "primarySeriesChanged\0QBar3DSeries*\0"
    "series\0selectedSeriesChanged\0"
    "floorLevelChanged\0level\0handleAxisXChanged\0"
    "QAbstract3DAxis*\0handleAxisYChanged\0"
    "handleAxisZChanged\0addSeries\0removeSeries\0"
    "insertSeries\0index\0rowAxis\0valueAxis\0"
    "columnAxis\0multiSeriesUniform\0"
    "barThickness\0barSpacing\0barSpacingRelative\0"
    "QQmlListProperty<QBar3DSeries>\0"
    "selectedSeries\0primarySeries\0floorLevel"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QtDataVisualization__DeclarativeBars[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
      17,   16, // methods
      11,  171, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      11,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    1,  118,    4, 0x06 /* Public */,
       7,    1,  121,    4, 0x06 /* Public */,
       9,    1,  124,    4, 0x06 /* Public */,
      10,    1,  127,    4, 0x06 /* Public */,
      12,    1,  130,    4, 0x06 /* Public */,
      14,    1,  133,    4, 0x06 /* Public */,
      16,    1,  136,    4, 0x06 /* Public */,
      18,    1,  139,    4, 0x06 /* Public */,
      20,    1,  142,    4, 0x06 /* Public */,
      23,    1,  145,    4, 0x06 /* Public */,
      24,    1,  148,    4, 0x86 /* Public | MethodRevisioned */,

 // slots: name, argc, parameters, tag, flags
      26,    1,  151,    4, 0x0a /* Public */,
      28,    1,  154,    4, 0x0a /* Public */,
      29,    1,  157,    4, 0x0a /* Public */,

 // methods: name, argc, parameters, tag, flags
      30,    1,  160,    4, 0x02 /* Public */,
      31,    1,  163,    4, 0x02 /* Public */,
      32,    2,  166,    4, 0x02 /* Public */,

 // signals: revision
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       1,

 // slots: revision
       0,
       0,
       0,

 // methods: revision
       0,
       0,
       0,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 5,    6,
    QMetaType::Void, 0x80000000 | 8,    6,
    QMetaType::Void, 0x80000000 | 5,    6,
    QMetaType::Void, QMetaType::Bool,   11,
    QMetaType::Void, QMetaType::Float,   13,
    QMetaType::Void, QMetaType::QSizeF,   15,
    QMetaType::Void, QMetaType::Bool,   17,
    QMetaType::Void, QMetaType::QString,   19,
    QMetaType::Void, 0x80000000 | 21,   22,
    QMetaType::Void, 0x80000000 | 21,   22,
    QMetaType::Void, QMetaType::Float,   25,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 27,    6,
    QMetaType::Void, 0x80000000 | 27,    6,
    QMetaType::Void, 0x80000000 | 27,    6,

 // methods: parameters
    QMetaType::Void, 0x80000000 | 21,   22,
    QMetaType::Void, 0x80000000 | 21,   22,
    QMetaType::Void, QMetaType::Int, 0x80000000 | 21,   33,   22,

 // properties: name, type, flags
      34, 0x80000000 | 5, 0x0049510b,
      35, 0x80000000 | 8, 0x0049510b,
      36, 0x80000000 | 5, 0x0049510b,
      37, QMetaType::Bool, 0x00495103,
      38, QMetaType::Float, 0x00495103,
      39, QMetaType::QSizeF, 0x00495103,
      40, QMetaType::Bool, 0x00495103,
       2, 0x80000000 | 41, 0x00095009,
      42, 0x80000000 | 21, 0x00495009,
      43, 0x80000000 | 21, 0x0049510b,
      44, QMetaType::Float, 0x00c95103,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,
       4,
       5,
       6,
       0,
       9,
       8,
      10,

 // properties: revision
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       1,

       0        // eod
};

void QtDataVisualization::DeclarativeBars::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DeclarativeBars *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->rowAxisChanged((*reinterpret_cast< QCategory3DAxis*(*)>(_a[1]))); break;
        case 1: _t->valueAxisChanged((*reinterpret_cast< QValue3DAxis*(*)>(_a[1]))); break;
        case 2: _t->columnAxisChanged((*reinterpret_cast< QCategory3DAxis*(*)>(_a[1]))); break;
        case 3: _t->multiSeriesUniformChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 4: _t->barThicknessChanged((*reinterpret_cast< float(*)>(_a[1]))); break;
        case 5: _t->barSpacingChanged((*reinterpret_cast< QSizeF(*)>(_a[1]))); break;
        case 6: _t->barSpacingRelativeChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 7: _t->meshFileNameChanged((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 8: _t->primarySeriesChanged((*reinterpret_cast< QBar3DSeries*(*)>(_a[1]))); break;
        case 9: _t->selectedSeriesChanged((*reinterpret_cast< QBar3DSeries*(*)>(_a[1]))); break;
        case 10: _t->floorLevelChanged((*reinterpret_cast< float(*)>(_a[1]))); break;
        case 11: _t->handleAxisXChanged((*reinterpret_cast< QAbstract3DAxis*(*)>(_a[1]))); break;
        case 12: _t->handleAxisYChanged((*reinterpret_cast< QAbstract3DAxis*(*)>(_a[1]))); break;
        case 13: _t->handleAxisZChanged((*reinterpret_cast< QAbstract3DAxis*(*)>(_a[1]))); break;
        case 14: _t->addSeries((*reinterpret_cast< QBar3DSeries*(*)>(_a[1]))); break;
        case 15: _t->removeSeries((*reinterpret_cast< QBar3DSeries*(*)>(_a[1]))); break;
        case 16: _t->insertSeries((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< QBar3DSeries*(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QCategory3DAxis* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QValue3DAxis* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QCategory3DAxis* >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QBar3DSeries* >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QBar3DSeries* >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAbstract3DAxis* >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAbstract3DAxis* >(); break;
            }
            break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAbstract3DAxis* >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QBar3DSeries* >(); break;
            }
            break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QBar3DSeries* >(); break;
            }
            break;
        case 16:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QBar3DSeries* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DeclarativeBars::*)(QCategory3DAxis * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeclarativeBars::rowAxisChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (DeclarativeBars::*)(QValue3DAxis * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeclarativeBars::valueAxisChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (DeclarativeBars::*)(QCategory3DAxis * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeclarativeBars::columnAxisChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (DeclarativeBars::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeclarativeBars::multiSeriesUniformChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (DeclarativeBars::*)(float );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeclarativeBars::barThicknessChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (DeclarativeBars::*)(QSizeF );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeclarativeBars::barSpacingChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (DeclarativeBars::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeclarativeBars::barSpacingRelativeChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (DeclarativeBars::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeclarativeBars::meshFileNameChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (DeclarativeBars::*)(QBar3DSeries * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeclarativeBars::primarySeriesChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (DeclarativeBars::*)(QBar3DSeries * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeclarativeBars::selectedSeriesChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (DeclarativeBars::*)(float );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeclarativeBars::floorLevelChanged)) {
                *result = 10;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 9:
        case 8:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QBar3DSeries* >(); break;
        case 2:
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QCategory3DAxis* >(); break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QValue3DAxis* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<DeclarativeBars *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QCategory3DAxis**>(_v) = _t->rowAxis(); break;
        case 1: *reinterpret_cast< QValue3DAxis**>(_v) = _t->valueAxis(); break;
        case 2: *reinterpret_cast< QCategory3DAxis**>(_v) = _t->columnAxis(); break;
        case 3: *reinterpret_cast< bool*>(_v) = _t->isMultiSeriesUniform(); break;
        case 4: *reinterpret_cast< float*>(_v) = _t->barThickness(); break;
        case 5: *reinterpret_cast< QSizeF*>(_v) = _t->barSpacing(); break;
        case 6: *reinterpret_cast< bool*>(_v) = _t->isBarSpacingRelative(); break;
        case 7: *reinterpret_cast< QQmlListProperty<QBar3DSeries>*>(_v) = _t->seriesList(); break;
        case 8: *reinterpret_cast< QBar3DSeries**>(_v) = _t->selectedSeries(); break;
        case 9: *reinterpret_cast< QBar3DSeries**>(_v) = _t->primarySeries(); break;
        case 10: *reinterpret_cast< float*>(_v) = _t->floorLevel(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<DeclarativeBars *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setRowAxis(*reinterpret_cast< QCategory3DAxis**>(_v)); break;
        case 1: _t->setValueAxis(*reinterpret_cast< QValue3DAxis**>(_v)); break;
        case 2: _t->setColumnAxis(*reinterpret_cast< QCategory3DAxis**>(_v)); break;
        case 3: _t->setMultiSeriesUniform(*reinterpret_cast< bool*>(_v)); break;
        case 4: _t->setBarThickness(*reinterpret_cast< float*>(_v)); break;
        case 5: _t->setBarSpacing(*reinterpret_cast< QSizeF*>(_v)); break;
        case 6: _t->setBarSpacingRelative(*reinterpret_cast< bool*>(_v)); break;
        case 9: _t->setPrimarySeries(*reinterpret_cast< QBar3DSeries**>(_v)); break;
        case 10: _t->setFloorLevel(*reinterpret_cast< float*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QtDataVisualization::DeclarativeBars::staticMetaObject = { {
    QMetaObject::SuperData::link<AbstractDeclarative::staticMetaObject>(),
    qt_meta_stringdata_QtDataVisualization__DeclarativeBars.data,
    qt_meta_data_QtDataVisualization__DeclarativeBars,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QtDataVisualization::DeclarativeBars::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtDataVisualization::DeclarativeBars::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QtDataVisualization__DeclarativeBars.stringdata0))
        return static_cast<void*>(this);
    return AbstractDeclarative::qt_metacast(_clname);
}

int QtDataVisualization::DeclarativeBars::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AbstractDeclarative::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 11;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 11;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 11;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 11;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 11;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QtDataVisualization::DeclarativeBars::rowAxisChanged(QCategory3DAxis * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QtDataVisualization::DeclarativeBars::valueAxisChanged(QValue3DAxis * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QtDataVisualization::DeclarativeBars::columnAxisChanged(QCategory3DAxis * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QtDataVisualization::DeclarativeBars::multiSeriesUniformChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QtDataVisualization::DeclarativeBars::barThicknessChanged(float _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void QtDataVisualization::DeclarativeBars::barSpacingChanged(QSizeF _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void QtDataVisualization::DeclarativeBars::barSpacingRelativeChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void QtDataVisualization::DeclarativeBars::meshFileNameChanged(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void QtDataVisualization::DeclarativeBars::primarySeriesChanged(QBar3DSeries * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void QtDataVisualization::DeclarativeBars::selectedSeriesChanged(QBar3DSeries * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void QtDataVisualization::DeclarativeBars::floorLevelChanged(float _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 10, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
