/****************************************************************************
** Meta object code from reading C++ file 'q3dcamera.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kde-qtdatavis3d/src/datavisualization/engine/q3dcamera.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'q3dcamera.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QtDataVisualization__Q3DCamera_t {
    QByteArrayData data[50];
    char stringdata0[946];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QtDataVisualization__Q3DCamera_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QtDataVisualization__Q3DCamera_t qt_meta_stringdata_QtDataVisualization__Q3DCamera = {
    {
QT_MOC_LITERAL(0, 0, 30), // "QtDataVisualization::Q3DCamera"
QT_MOC_LITERAL(1, 31, 16), // "xRotationChanged"
QT_MOC_LITERAL(2, 48, 0), // ""
QT_MOC_LITERAL(3, 49, 8), // "rotation"
QT_MOC_LITERAL(4, 58, 16), // "yRotationChanged"
QT_MOC_LITERAL(5, 75, 16), // "zoomLevelChanged"
QT_MOC_LITERAL(6, 92, 9), // "zoomLevel"
QT_MOC_LITERAL(7, 102, 19), // "cameraPresetChanged"
QT_MOC_LITERAL(8, 122, 23), // "Q3DCamera::CameraPreset"
QT_MOC_LITERAL(9, 146, 6), // "preset"
QT_MOC_LITERAL(10, 153, 20), // "wrapXRotationChanged"
QT_MOC_LITERAL(11, 174, 9), // "isEnabled"
QT_MOC_LITERAL(12, 184, 20), // "wrapYRotationChanged"
QT_MOC_LITERAL(13, 205, 13), // "targetChanged"
QT_MOC_LITERAL(14, 219, 6), // "target"
QT_MOC_LITERAL(15, 226, 19), // "minZoomLevelChanged"
QT_MOC_LITERAL(16, 246, 19), // "maxZoomLevelChanged"
QT_MOC_LITERAL(17, 266, 9), // "xRotation"
QT_MOC_LITERAL(18, 276, 9), // "yRotation"
QT_MOC_LITERAL(19, 286, 12), // "cameraPreset"
QT_MOC_LITERAL(20, 299, 12), // "CameraPreset"
QT_MOC_LITERAL(21, 312, 13), // "wrapXRotation"
QT_MOC_LITERAL(22, 326, 13), // "wrapYRotation"
QT_MOC_LITERAL(23, 340, 12), // "minZoomLevel"
QT_MOC_LITERAL(24, 353, 12), // "maxZoomLevel"
QT_MOC_LITERAL(25, 366, 16), // "CameraPresetNone"
QT_MOC_LITERAL(26, 383, 20), // "CameraPresetFrontLow"
QT_MOC_LITERAL(27, 404, 17), // "CameraPresetFront"
QT_MOC_LITERAL(28, 422, 21), // "CameraPresetFrontHigh"
QT_MOC_LITERAL(29, 444, 19), // "CameraPresetLeftLow"
QT_MOC_LITERAL(30, 464, 16), // "CameraPresetLeft"
QT_MOC_LITERAL(31, 481, 20), // "CameraPresetLeftHigh"
QT_MOC_LITERAL(32, 502, 20), // "CameraPresetRightLow"
QT_MOC_LITERAL(33, 523, 17), // "CameraPresetRight"
QT_MOC_LITERAL(34, 541, 21), // "CameraPresetRightHigh"
QT_MOC_LITERAL(35, 563, 21), // "CameraPresetBehindLow"
QT_MOC_LITERAL(36, 585, 18), // "CameraPresetBehind"
QT_MOC_LITERAL(37, 604, 22), // "CameraPresetBehindHigh"
QT_MOC_LITERAL(38, 627, 25), // "CameraPresetIsometricLeft"
QT_MOC_LITERAL(39, 653, 29), // "CameraPresetIsometricLeftHigh"
QT_MOC_LITERAL(40, 683, 26), // "CameraPresetIsometricRight"
QT_MOC_LITERAL(41, 710, 30), // "CameraPresetIsometricRightHigh"
QT_MOC_LITERAL(42, 741, 25), // "CameraPresetDirectlyAbove"
QT_MOC_LITERAL(43, 767, 29), // "CameraPresetDirectlyAboveCW45"
QT_MOC_LITERAL(44, 797, 30), // "CameraPresetDirectlyAboveCCW45"
QT_MOC_LITERAL(45, 828, 22), // "CameraPresetFrontBelow"
QT_MOC_LITERAL(46, 851, 21), // "CameraPresetLeftBelow"
QT_MOC_LITERAL(47, 873, 22), // "CameraPresetRightBelow"
QT_MOC_LITERAL(48, 896, 23), // "CameraPresetBehindBelow"
QT_MOC_LITERAL(49, 920, 25) // "CameraPresetDirectlyBelow"

    },
    "QtDataVisualization::Q3DCamera\0"
    "xRotationChanged\0\0rotation\0yRotationChanged\0"
    "zoomLevelChanged\0zoomLevel\0"
    "cameraPresetChanged\0Q3DCamera::CameraPreset\0"
    "preset\0wrapXRotationChanged\0isEnabled\0"
    "wrapYRotationChanged\0targetChanged\0"
    "target\0minZoomLevelChanged\0"
    "maxZoomLevelChanged\0xRotation\0yRotation\0"
    "cameraPreset\0CameraPreset\0wrapXRotation\0"
    "wrapYRotation\0minZoomLevel\0maxZoomLevel\0"
    "CameraPresetNone\0CameraPresetFrontLow\0"
    "CameraPresetFront\0CameraPresetFrontHigh\0"
    "CameraPresetLeftLow\0CameraPresetLeft\0"
    "CameraPresetLeftHigh\0CameraPresetRightLow\0"
    "CameraPresetRight\0CameraPresetRightHigh\0"
    "CameraPresetBehindLow\0CameraPresetBehind\0"
    "CameraPresetBehindHigh\0CameraPresetIsometricLeft\0"
    "CameraPresetIsometricLeftHigh\0"
    "CameraPresetIsometricRight\0"
    "CameraPresetIsometricRightHigh\0"
    "CameraPresetDirectlyAbove\0"
    "CameraPresetDirectlyAboveCW45\0"
    "CameraPresetDirectlyAboveCCW45\0"
    "CameraPresetFrontBelow\0CameraPresetLeftBelow\0"
    "CameraPresetRightBelow\0CameraPresetBehindBelow\0"
    "CameraPresetDirectlyBelow"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QtDataVisualization__Q3DCamera[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       9,   95, // properties
       1,  140, // enums/sets
       0,    0, // constructors
       0,       // flags
       9,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   68,    2, 0x06 /* Public */,
       4,    1,   71,    2, 0x06 /* Public */,
       5,    1,   74,    2, 0x06 /* Public */,
       7,    1,   77,    2, 0x06 /* Public */,
      10,    1,   80,    2, 0x06 /* Public */,
      12,    1,   83,    2, 0x06 /* Public */,
      13,    1,   86,    2, 0x86 /* Public | MethodRevisioned */,
      15,    1,   89,    2, 0x86 /* Public | MethodRevisioned */,
      16,    1,   92,    2, 0x86 /* Public | MethodRevisioned */,

 // signals: revision
       0,
       0,
       0,
       0,
       0,
       0,
       1,
       1,
       1,

 // signals: parameters
    QMetaType::Void, QMetaType::Float,    3,
    QMetaType::Void, QMetaType::Float,    3,
    QMetaType::Void, QMetaType::Float,    6,
    QMetaType::Void, 0x80000000 | 8,    9,
    QMetaType::Void, QMetaType::Bool,   11,
    QMetaType::Void, QMetaType::Bool,   11,
    QMetaType::Void, QMetaType::QVector3D,   14,
    QMetaType::Void, QMetaType::Float,    6,
    QMetaType::Void, QMetaType::Float,    6,

 // properties: name, type, flags
      17, QMetaType::Float, 0x00495103,
      18, QMetaType::Float, 0x00495103,
       6, QMetaType::Float, 0x00495103,
      19, 0x80000000 | 20, 0x0049510b,
      21, QMetaType::Bool, 0x00495103,
      22, QMetaType::Bool, 0x00495103,
      14, QMetaType::QVector3D, 0x00c95103,
      23, QMetaType::Float, 0x00c95103,
      24, QMetaType::Float, 0x00c95103,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,
       4,
       5,
       6,
       7,
       8,

 // properties: revision
       0,
       0,
       0,
       0,
       0,
       0,
       1,
       1,
       1,

 // enums: name, alias, flags, count, data
      20,   20, 0x0,   25,  145,

 // enum data: key, value
      25, uint(QtDataVisualization::Q3DCamera::CameraPresetNone),
      26, uint(QtDataVisualization::Q3DCamera::CameraPresetFrontLow),
      27, uint(QtDataVisualization::Q3DCamera::CameraPresetFront),
      28, uint(QtDataVisualization::Q3DCamera::CameraPresetFrontHigh),
      29, uint(QtDataVisualization::Q3DCamera::CameraPresetLeftLow),
      30, uint(QtDataVisualization::Q3DCamera::CameraPresetLeft),
      31, uint(QtDataVisualization::Q3DCamera::CameraPresetLeftHigh),
      32, uint(QtDataVisualization::Q3DCamera::CameraPresetRightLow),
      33, uint(QtDataVisualization::Q3DCamera::CameraPresetRight),
      34, uint(QtDataVisualization::Q3DCamera::CameraPresetRightHigh),
      35, uint(QtDataVisualization::Q3DCamera::CameraPresetBehindLow),
      36, uint(QtDataVisualization::Q3DCamera::CameraPresetBehind),
      37, uint(QtDataVisualization::Q3DCamera::CameraPresetBehindHigh),
      38, uint(QtDataVisualization::Q3DCamera::CameraPresetIsometricLeft),
      39, uint(QtDataVisualization::Q3DCamera::CameraPresetIsometricLeftHigh),
      40, uint(QtDataVisualization::Q3DCamera::CameraPresetIsometricRight),
      41, uint(QtDataVisualization::Q3DCamera::CameraPresetIsometricRightHigh),
      42, uint(QtDataVisualization::Q3DCamera::CameraPresetDirectlyAbove),
      43, uint(QtDataVisualization::Q3DCamera::CameraPresetDirectlyAboveCW45),
      44, uint(QtDataVisualization::Q3DCamera::CameraPresetDirectlyAboveCCW45),
      45, uint(QtDataVisualization::Q3DCamera::CameraPresetFrontBelow),
      46, uint(QtDataVisualization::Q3DCamera::CameraPresetLeftBelow),
      47, uint(QtDataVisualization::Q3DCamera::CameraPresetRightBelow),
      48, uint(QtDataVisualization::Q3DCamera::CameraPresetBehindBelow),
      49, uint(QtDataVisualization::Q3DCamera::CameraPresetDirectlyBelow),

       0        // eod
};

void QtDataVisualization::Q3DCamera::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<Q3DCamera *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->xRotationChanged((*reinterpret_cast< float(*)>(_a[1]))); break;
        case 1: _t->yRotationChanged((*reinterpret_cast< float(*)>(_a[1]))); break;
        case 2: _t->zoomLevelChanged((*reinterpret_cast< float(*)>(_a[1]))); break;
        case 3: _t->cameraPresetChanged((*reinterpret_cast< Q3DCamera::CameraPreset(*)>(_a[1]))); break;
        case 4: _t->wrapXRotationChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: _t->wrapYRotationChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 6: _t->targetChanged((*reinterpret_cast< const QVector3D(*)>(_a[1]))); break;
        case 7: _t->minZoomLevelChanged((*reinterpret_cast< float(*)>(_a[1]))); break;
        case 8: _t->maxZoomLevelChanged((*reinterpret_cast< float(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (Q3DCamera::*)(float );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Q3DCamera::xRotationChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (Q3DCamera::*)(float );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Q3DCamera::yRotationChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (Q3DCamera::*)(float );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Q3DCamera::zoomLevelChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (Q3DCamera::*)(Q3DCamera::CameraPreset );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Q3DCamera::cameraPresetChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (Q3DCamera::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Q3DCamera::wrapXRotationChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (Q3DCamera::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Q3DCamera::wrapYRotationChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (Q3DCamera::*)(const QVector3D & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Q3DCamera::targetChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (Q3DCamera::*)(float );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Q3DCamera::minZoomLevelChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (Q3DCamera::*)(float );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Q3DCamera::maxZoomLevelChanged)) {
                *result = 8;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<Q3DCamera *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< float*>(_v) = _t->xRotation(); break;
        case 1: *reinterpret_cast< float*>(_v) = _t->yRotation(); break;
        case 2: *reinterpret_cast< float*>(_v) = _t->zoomLevel(); break;
        case 3: *reinterpret_cast< CameraPreset*>(_v) = _t->cameraPreset(); break;
        case 4: *reinterpret_cast< bool*>(_v) = _t->wrapXRotation(); break;
        case 5: *reinterpret_cast< bool*>(_v) = _t->wrapYRotation(); break;
        case 6: *reinterpret_cast< QVector3D*>(_v) = _t->target(); break;
        case 7: *reinterpret_cast< float*>(_v) = _t->minZoomLevel(); break;
        case 8: *reinterpret_cast< float*>(_v) = _t->maxZoomLevel(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<Q3DCamera *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setXRotation(*reinterpret_cast< float*>(_v)); break;
        case 1: _t->setYRotation(*reinterpret_cast< float*>(_v)); break;
        case 2: _t->setZoomLevel(*reinterpret_cast< float*>(_v)); break;
        case 3: _t->setCameraPreset(*reinterpret_cast< CameraPreset*>(_v)); break;
        case 4: _t->setWrapXRotation(*reinterpret_cast< bool*>(_v)); break;
        case 5: _t->setWrapYRotation(*reinterpret_cast< bool*>(_v)); break;
        case 6: _t->setTarget(*reinterpret_cast< QVector3D*>(_v)); break;
        case 7: _t->setMinZoomLevel(*reinterpret_cast< float*>(_v)); break;
        case 8: _t->setMaxZoomLevel(*reinterpret_cast< float*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QtDataVisualization::Q3DCamera::staticMetaObject = { {
    QMetaObject::SuperData::link<Q3DObject::staticMetaObject>(),
    qt_meta_stringdata_QtDataVisualization__Q3DCamera.data,
    qt_meta_data_QtDataVisualization__Q3DCamera,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QtDataVisualization::Q3DCamera::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtDataVisualization::Q3DCamera::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QtDataVisualization__Q3DCamera.stringdata0))
        return static_cast<void*>(this);
    return Q3DObject::qt_metacast(_clname);
}

int QtDataVisualization::Q3DCamera::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Q3DObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 9;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 9;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 9;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 9;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 9;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 9;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QtDataVisualization::Q3DCamera::xRotationChanged(float _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QtDataVisualization::Q3DCamera::yRotationChanged(float _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QtDataVisualization::Q3DCamera::zoomLevelChanged(float _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QtDataVisualization::Q3DCamera::cameraPresetChanged(Q3DCamera::CameraPreset _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QtDataVisualization::Q3DCamera::wrapXRotationChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void QtDataVisualization::Q3DCamera::wrapYRotationChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void QtDataVisualization::Q3DCamera::targetChanged(const QVector3D & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void QtDataVisualization::Q3DCamera::minZoomLevelChanged(float _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void QtDataVisualization::Q3DCamera::maxZoomLevelChanged(float _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
