/****************************************************************************
** Meta object code from reading C++ file 'abstract3dcontroller_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kde-qtdatavis3d/src/datavisualization/engine/abstract3dcontroller_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'abstract3dcontroller_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QtDataVisualization__Abstract3DController_t {
    QByteArrayData data[94];
    char stringdata0[1740];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QtDataVisualization__Abstract3DController_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QtDataVisualization__Abstract3DController_t qt_meta_stringdata_QtDataVisualization__Abstract3DController = {
    {
QT_MOC_LITERAL(0, 0, 41), // "QtDataVisualization::Abstract..."
QT_MOC_LITERAL(1, 42, 20), // "shadowQualityChanged"
QT_MOC_LITERAL(2, 63, 0), // ""
QT_MOC_LITERAL(3, 64, 31), // "QAbstract3DGraph::ShadowQuality"
QT_MOC_LITERAL(4, 96, 7), // "quality"
QT_MOC_LITERAL(5, 104, 25), // "activeInputHandlerChanged"
QT_MOC_LITERAL(6, 130, 24), // "QAbstract3DInputHandler*"
QT_MOC_LITERAL(7, 155, 12), // "inputHandler"
QT_MOC_LITERAL(8, 168, 18), // "activeThemeChanged"
QT_MOC_LITERAL(9, 187, 9), // "Q3DTheme*"
QT_MOC_LITERAL(10, 197, 11), // "activeTheme"
QT_MOC_LITERAL(11, 209, 20), // "selectionModeChanged"
QT_MOC_LITERAL(12, 230, 32), // "QAbstract3DGraph::SelectionFlags"
QT_MOC_LITERAL(13, 263, 4), // "mode"
QT_MOC_LITERAL(14, 268, 10), // "needRender"
QT_MOC_LITERAL(15, 279, 12), // "axisXChanged"
QT_MOC_LITERAL(16, 292, 16), // "QAbstract3DAxis*"
QT_MOC_LITERAL(17, 309, 4), // "axis"
QT_MOC_LITERAL(18, 314, 12), // "axisYChanged"
QT_MOC_LITERAL(19, 327, 12), // "axisZChanged"
QT_MOC_LITERAL(20, 340, 15), // "elementSelected"
QT_MOC_LITERAL(21, 356, 29), // "QAbstract3DGraph::ElementType"
QT_MOC_LITERAL(22, 386, 4), // "type"
QT_MOC_LITERAL(23, 391, 17), // "measureFpsChanged"
QT_MOC_LITERAL(24, 409, 7), // "enabled"
QT_MOC_LITERAL(25, 417, 17), // "currentFpsChanged"
QT_MOC_LITERAL(26, 435, 3), // "fps"
QT_MOC_LITERAL(27, 439, 22), // "orthoProjectionChanged"
QT_MOC_LITERAL(28, 462, 18), // "aspectRatioChanged"
QT_MOC_LITERAL(29, 481, 5), // "ratio"
QT_MOC_LITERAL(30, 487, 28), // "horizontalAspectRatioChanged"
QT_MOC_LITERAL(31, 516, 24), // "optimizationHintsChanged"
QT_MOC_LITERAL(32, 541, 35), // "QAbstract3DGraph::Optimizatio..."
QT_MOC_LITERAL(33, 577, 5), // "hints"
QT_MOC_LITERAL(34, 583, 12), // "polarChanged"
QT_MOC_LITERAL(35, 596, 24), // "radialLabelOffsetChanged"
QT_MOC_LITERAL(36, 621, 6), // "offset"
QT_MOC_LITERAL(37, 628, 17), // "reflectionChanged"
QT_MOC_LITERAL(38, 646, 19), // "reflectivityChanged"
QT_MOC_LITERAL(39, 666, 12), // "reflectivity"
QT_MOC_LITERAL(40, 679, 13), // "localeChanged"
QT_MOC_LITERAL(41, 693, 6), // "locale"
QT_MOC_LITERAL(42, 700, 27), // "queriedGraphPositionChanged"
QT_MOC_LITERAL(43, 728, 4), // "data"
QT_MOC_LITERAL(44, 733, 13), // "marginChanged"
QT_MOC_LITERAL(45, 747, 6), // "margin"
QT_MOC_LITERAL(46, 754, 15), // "destroyRenderer"
QT_MOC_LITERAL(47, 770, 22), // "handleAxisTitleChanged"
QT_MOC_LITERAL(48, 793, 5), // "title"
QT_MOC_LITERAL(49, 799, 23), // "handleAxisLabelsChanged"
QT_MOC_LITERAL(50, 823, 22), // "handleAxisRangeChanged"
QT_MOC_LITERAL(51, 846, 3), // "min"
QT_MOC_LITERAL(52, 850, 3), // "max"
QT_MOC_LITERAL(53, 854, 29), // "handleAxisSegmentCountChanged"
QT_MOC_LITERAL(54, 884, 5), // "count"
QT_MOC_LITERAL(55, 890, 32), // "handleAxisSubSegmentCountChanged"
QT_MOC_LITERAL(56, 923, 32), // "handleAxisAutoAdjustRangeChanged"
QT_MOC_LITERAL(57, 956, 10), // "autoAdjust"
QT_MOC_LITERAL(58, 967, 28), // "handleAxisLabelFormatChanged"
QT_MOC_LITERAL(59, 996, 6), // "format"
QT_MOC_LITERAL(60, 1003, 25), // "handleAxisReversedChanged"
QT_MOC_LITERAL(61, 1029, 6), // "enable"
QT_MOC_LITERAL(62, 1036, 24), // "handleAxisFormatterDirty"
QT_MOC_LITERAL(63, 1061, 34), // "handleAxisLabelAutoRotationCh..."
QT_MOC_LITERAL(64, 1096, 5), // "angle"
QT_MOC_LITERAL(65, 1102, 32), // "handleAxisTitleVisibilityChanged"
QT_MOC_LITERAL(66, 1135, 7), // "visible"
QT_MOC_LITERAL(67, 1143, 27), // "handleAxisTitleFixedChanged"
QT_MOC_LITERAL(68, 1171, 5), // "fixed"
QT_MOC_LITERAL(69, 1177, 22), // "handleInputViewChanged"
QT_MOC_LITERAL(70, 1200, 34), // "QAbstract3DInputHandler::Inpu..."
QT_MOC_LITERAL(71, 1235, 4), // "view"
QT_MOC_LITERAL(72, 1240, 26), // "handleInputPositionChanged"
QT_MOC_LITERAL(73, 1267, 8), // "position"
QT_MOC_LITERAL(74, 1276, 29), // "handleSeriesVisibilityChanged"
QT_MOC_LITERAL(75, 1306, 28), // "handleThemeColorStyleChanged"
QT_MOC_LITERAL(76, 1335, 20), // "Q3DTheme::ColorStyle"
QT_MOC_LITERAL(77, 1356, 5), // "style"
QT_MOC_LITERAL(78, 1362, 28), // "handleThemeBaseColorsChanged"
QT_MOC_LITERAL(79, 1391, 13), // "QList<QColor>"
QT_MOC_LITERAL(80, 1405, 5), // "color"
QT_MOC_LITERAL(81, 1411, 31), // "handleThemeBaseGradientsChanged"
QT_MOC_LITERAL(82, 1443, 22), // "QList<QLinearGradient>"
QT_MOC_LITERAL(83, 1466, 8), // "gradient"
QT_MOC_LITERAL(84, 1475, 38), // "handleThemeSingleHighlightCol..."
QT_MOC_LITERAL(85, 1514, 41), // "handleThemeSingleHighlightGra..."
QT_MOC_LITERAL(86, 1556, 15), // "QLinearGradient"
QT_MOC_LITERAL(87, 1572, 37), // "handleThemeMultiHighlightColo..."
QT_MOC_LITERAL(88, 1610, 40), // "handleThemeMultiHighlightGrad..."
QT_MOC_LITERAL(89, 1651, 22), // "handleThemeTypeChanged"
QT_MOC_LITERAL(90, 1674, 15), // "Q3DTheme::Theme"
QT_MOC_LITERAL(91, 1690, 5), // "theme"
QT_MOC_LITERAL(92, 1696, 26), // "handleRequestShadowQuality"
QT_MOC_LITERAL(93, 1723, 16) // "updateCustomItem"

    },
    "QtDataVisualization::Abstract3DController\0"
    "shadowQualityChanged\0\0"
    "QAbstract3DGraph::ShadowQuality\0quality\0"
    "activeInputHandlerChanged\0"
    "QAbstract3DInputHandler*\0inputHandler\0"
    "activeThemeChanged\0Q3DTheme*\0activeTheme\0"
    "selectionModeChanged\0"
    "QAbstract3DGraph::SelectionFlags\0mode\0"
    "needRender\0axisXChanged\0QAbstract3DAxis*\0"
    "axis\0axisYChanged\0axisZChanged\0"
    "elementSelected\0QAbstract3DGraph::ElementType\0"
    "type\0measureFpsChanged\0enabled\0"
    "currentFpsChanged\0fps\0orthoProjectionChanged\0"
    "aspectRatioChanged\0ratio\0"
    "horizontalAspectRatioChanged\0"
    "optimizationHintsChanged\0"
    "QAbstract3DGraph::OptimizationHints\0"
    "hints\0polarChanged\0radialLabelOffsetChanged\0"
    "offset\0reflectionChanged\0reflectivityChanged\0"
    "reflectivity\0localeChanged\0locale\0"
    "queriedGraphPositionChanged\0data\0"
    "marginChanged\0margin\0destroyRenderer\0"
    "handleAxisTitleChanged\0title\0"
    "handleAxisLabelsChanged\0handleAxisRangeChanged\0"
    "min\0max\0handleAxisSegmentCountChanged\0"
    "count\0handleAxisSubSegmentCountChanged\0"
    "handleAxisAutoAdjustRangeChanged\0"
    "autoAdjust\0handleAxisLabelFormatChanged\0"
    "format\0handleAxisReversedChanged\0"
    "enable\0handleAxisFormatterDirty\0"
    "handleAxisLabelAutoRotationChanged\0"
    "angle\0handleAxisTitleVisibilityChanged\0"
    "visible\0handleAxisTitleFixedChanged\0"
    "fixed\0handleInputViewChanged\0"
    "QAbstract3DInputHandler::InputView\0"
    "view\0handleInputPositionChanged\0"
    "position\0handleSeriesVisibilityChanged\0"
    "handleThemeColorStyleChanged\0"
    "Q3DTheme::ColorStyle\0style\0"
    "handleThemeBaseColorsChanged\0QList<QColor>\0"
    "color\0handleThemeBaseGradientsChanged\0"
    "QList<QLinearGradient>\0gradient\0"
    "handleThemeSingleHighlightColorChanged\0"
    "handleThemeSingleHighlightGradientChanged\0"
    "QLinearGradient\0handleThemeMultiHighlightColorChanged\0"
    "handleThemeMultiHighlightGradientChanged\0"
    "handleThemeTypeChanged\0Q3DTheme::Theme\0"
    "theme\0handleRequestShadowQuality\0"
    "updateCustomItem"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QtDataVisualization__Abstract3DController[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      48,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      22,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  254,    2, 0x06 /* Public */,
       5,    1,  257,    2, 0x06 /* Public */,
       8,    1,  260,    2, 0x06 /* Public */,
      11,    1,  263,    2, 0x06 /* Public */,
      14,    0,  266,    2, 0x06 /* Public */,
      15,    1,  267,    2, 0x06 /* Public */,
      18,    1,  270,    2, 0x06 /* Public */,
      19,    1,  273,    2, 0x06 /* Public */,
      20,    1,  276,    2, 0x06 /* Public */,
      23,    1,  279,    2, 0x06 /* Public */,
      25,    1,  282,    2, 0x06 /* Public */,
      27,    1,  285,    2, 0x06 /* Public */,
      28,    1,  288,    2, 0x06 /* Public */,
      30,    1,  291,    2, 0x06 /* Public */,
      31,    1,  294,    2, 0x06 /* Public */,
      34,    1,  297,    2, 0x06 /* Public */,
      35,    1,  300,    2, 0x06 /* Public */,
      37,    1,  303,    2, 0x06 /* Public */,
      38,    1,  306,    2, 0x06 /* Public */,
      40,    1,  309,    2, 0x06 /* Public */,
      42,    1,  312,    2, 0x06 /* Public */,
      44,    1,  315,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      46,    0,  318,    2, 0x0a /* Public */,
      47,    1,  319,    2, 0x0a /* Public */,
      49,    0,  322,    2, 0x0a /* Public */,
      50,    2,  323,    2, 0x0a /* Public */,
      53,    1,  328,    2, 0x0a /* Public */,
      55,    1,  331,    2, 0x0a /* Public */,
      56,    1,  334,    2, 0x0a /* Public */,
      58,    1,  337,    2, 0x0a /* Public */,
      60,    1,  340,    2, 0x0a /* Public */,
      62,    0,  343,    2, 0x0a /* Public */,
      63,    1,  344,    2, 0x0a /* Public */,
      65,    1,  347,    2, 0x0a /* Public */,
      67,    1,  350,    2, 0x0a /* Public */,
      69,    1,  353,    2, 0x0a /* Public */,
      72,    1,  356,    2, 0x0a /* Public */,
      74,    1,  359,    2, 0x0a /* Public */,
      75,    1,  362,    2, 0x0a /* Public */,
      78,    1,  365,    2, 0x0a /* Public */,
      81,    1,  368,    2, 0x0a /* Public */,
      84,    1,  371,    2, 0x0a /* Public */,
      85,    1,  374,    2, 0x0a /* Public */,
      87,    1,  377,    2, 0x0a /* Public */,
      88,    1,  380,    2, 0x0a /* Public */,
      89,    1,  383,    2, 0x0a /* Public */,
      92,    1,  386,    2, 0x0a /* Public */,
      93,    0,  389,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 16,   17,
    QMetaType::Void, 0x80000000 | 16,   17,
    QMetaType::Void, 0x80000000 | 16,   17,
    QMetaType::Void, 0x80000000 | 21,   22,
    QMetaType::Void, QMetaType::Bool,   24,
    QMetaType::Void, QMetaType::QReal,   26,
    QMetaType::Void, QMetaType::Bool,   24,
    QMetaType::Void, QMetaType::QReal,   29,
    QMetaType::Void, QMetaType::QReal,   29,
    QMetaType::Void, 0x80000000 | 32,   33,
    QMetaType::Void, QMetaType::Bool,   24,
    QMetaType::Void, QMetaType::Float,   36,
    QMetaType::Void, QMetaType::Bool,   24,
    QMetaType::Void, QMetaType::QReal,   39,
    QMetaType::Void, QMetaType::QLocale,   41,
    QMetaType::Void, QMetaType::QVector3D,   43,
    QMetaType::Void, QMetaType::QReal,   45,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   48,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Float, QMetaType::Float,   51,   52,
    QMetaType::Void, QMetaType::Int,   54,
    QMetaType::Void, QMetaType::Int,   54,
    QMetaType::Void, QMetaType::Bool,   57,
    QMetaType::Void, QMetaType::QString,   59,
    QMetaType::Void, QMetaType::Bool,   61,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Float,   64,
    QMetaType::Void, QMetaType::Bool,   66,
    QMetaType::Void, QMetaType::Bool,   68,
    QMetaType::Void, 0x80000000 | 70,   71,
    QMetaType::Void, QMetaType::QPoint,   73,
    QMetaType::Void, QMetaType::Bool,   66,
    QMetaType::Void, 0x80000000 | 76,   77,
    QMetaType::Void, 0x80000000 | 79,   80,
    QMetaType::Void, 0x80000000 | 82,   83,
    QMetaType::Void, QMetaType::QColor,   80,
    QMetaType::Void, 0x80000000 | 86,   83,
    QMetaType::Void, QMetaType::QColor,   80,
    QMetaType::Void, 0x80000000 | 86,   83,
    QMetaType::Void, 0x80000000 | 90,   91,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void,

       0        // eod
};

void QtDataVisualization::Abstract3DController::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<Abstract3DController *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->shadowQualityChanged((*reinterpret_cast< QAbstract3DGraph::ShadowQuality(*)>(_a[1]))); break;
        case 1: _t->activeInputHandlerChanged((*reinterpret_cast< QAbstract3DInputHandler*(*)>(_a[1]))); break;
        case 2: _t->activeThemeChanged((*reinterpret_cast< Q3DTheme*(*)>(_a[1]))); break;
        case 3: _t->selectionModeChanged((*reinterpret_cast< QAbstract3DGraph::SelectionFlags(*)>(_a[1]))); break;
        case 4: _t->needRender(); break;
        case 5: _t->axisXChanged((*reinterpret_cast< QAbstract3DAxis*(*)>(_a[1]))); break;
        case 6: _t->axisYChanged((*reinterpret_cast< QAbstract3DAxis*(*)>(_a[1]))); break;
        case 7: _t->axisZChanged((*reinterpret_cast< QAbstract3DAxis*(*)>(_a[1]))); break;
        case 8: _t->elementSelected((*reinterpret_cast< QAbstract3DGraph::ElementType(*)>(_a[1]))); break;
        case 9: _t->measureFpsChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 10: _t->currentFpsChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 11: _t->orthoProjectionChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 12: _t->aspectRatioChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 13: _t->horizontalAspectRatioChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 14: _t->optimizationHintsChanged((*reinterpret_cast< QAbstract3DGraph::OptimizationHints(*)>(_a[1]))); break;
        case 15: _t->polarChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 16: _t->radialLabelOffsetChanged((*reinterpret_cast< float(*)>(_a[1]))); break;
        case 17: _t->reflectionChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 18: _t->reflectivityChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 19: _t->localeChanged((*reinterpret_cast< const QLocale(*)>(_a[1]))); break;
        case 20: _t->queriedGraphPositionChanged((*reinterpret_cast< const QVector3D(*)>(_a[1]))); break;
        case 21: _t->marginChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 22: _t->destroyRenderer(); break;
        case 23: _t->handleAxisTitleChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 24: _t->handleAxisLabelsChanged(); break;
        case 25: _t->handleAxisRangeChanged((*reinterpret_cast< float(*)>(_a[1])),(*reinterpret_cast< float(*)>(_a[2]))); break;
        case 26: _t->handleAxisSegmentCountChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 27: _t->handleAxisSubSegmentCountChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 28: _t->handleAxisAutoAdjustRangeChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 29: _t->handleAxisLabelFormatChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 30: _t->handleAxisReversedChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 31: _t->handleAxisFormatterDirty(); break;
        case 32: _t->handleAxisLabelAutoRotationChanged((*reinterpret_cast< float(*)>(_a[1]))); break;
        case 33: _t->handleAxisTitleVisibilityChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 34: _t->handleAxisTitleFixedChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 35: _t->handleInputViewChanged((*reinterpret_cast< QAbstract3DInputHandler::InputView(*)>(_a[1]))); break;
        case 36: _t->handleInputPositionChanged((*reinterpret_cast< const QPoint(*)>(_a[1]))); break;
        case 37: _t->handleSeriesVisibilityChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 38: _t->handleThemeColorStyleChanged((*reinterpret_cast< Q3DTheme::ColorStyle(*)>(_a[1]))); break;
        case 39: _t->handleThemeBaseColorsChanged((*reinterpret_cast< const QList<QColor>(*)>(_a[1]))); break;
        case 40: _t->handleThemeBaseGradientsChanged((*reinterpret_cast< const QList<QLinearGradient>(*)>(_a[1]))); break;
        case 41: _t->handleThemeSingleHighlightColorChanged((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 42: _t->handleThemeSingleHighlightGradientChanged((*reinterpret_cast< const QLinearGradient(*)>(_a[1]))); break;
        case 43: _t->handleThemeMultiHighlightColorChanged((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 44: _t->handleThemeMultiHighlightGradientChanged((*reinterpret_cast< const QLinearGradient(*)>(_a[1]))); break;
        case 45: _t->handleThemeTypeChanged((*reinterpret_cast< Q3DTheme::Theme(*)>(_a[1]))); break;
        case 46: _t->handleRequestShadowQuality((*reinterpret_cast< QAbstract3DGraph::ShadowQuality(*)>(_a[1]))); break;
        case 47: _t->updateCustomItem(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAbstract3DInputHandler* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Q3DTheme* >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAbstract3DAxis* >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAbstract3DAxis* >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAbstract3DAxis* >(); break;
            }
            break;
        case 39:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QColor> >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (Abstract3DController::*)(QAbstract3DGraph::ShadowQuality );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Abstract3DController::shadowQualityChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (Abstract3DController::*)(QAbstract3DInputHandler * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Abstract3DController::activeInputHandlerChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (Abstract3DController::*)(Q3DTheme * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Abstract3DController::activeThemeChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (Abstract3DController::*)(QAbstract3DGraph::SelectionFlags );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Abstract3DController::selectionModeChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (Abstract3DController::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Abstract3DController::needRender)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (Abstract3DController::*)(QAbstract3DAxis * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Abstract3DController::axisXChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (Abstract3DController::*)(QAbstract3DAxis * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Abstract3DController::axisYChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (Abstract3DController::*)(QAbstract3DAxis * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Abstract3DController::axisZChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (Abstract3DController::*)(QAbstract3DGraph::ElementType );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Abstract3DController::elementSelected)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (Abstract3DController::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Abstract3DController::measureFpsChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (Abstract3DController::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Abstract3DController::currentFpsChanged)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (Abstract3DController::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Abstract3DController::orthoProjectionChanged)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (Abstract3DController::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Abstract3DController::aspectRatioChanged)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (Abstract3DController::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Abstract3DController::horizontalAspectRatioChanged)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (Abstract3DController::*)(QAbstract3DGraph::OptimizationHints );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Abstract3DController::optimizationHintsChanged)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (Abstract3DController::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Abstract3DController::polarChanged)) {
                *result = 15;
                return;
            }
        }
        {
            using _t = void (Abstract3DController::*)(float );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Abstract3DController::radialLabelOffsetChanged)) {
                *result = 16;
                return;
            }
        }
        {
            using _t = void (Abstract3DController::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Abstract3DController::reflectionChanged)) {
                *result = 17;
                return;
            }
        }
        {
            using _t = void (Abstract3DController::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Abstract3DController::reflectivityChanged)) {
                *result = 18;
                return;
            }
        }
        {
            using _t = void (Abstract3DController::*)(const QLocale & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Abstract3DController::localeChanged)) {
                *result = 19;
                return;
            }
        }
        {
            using _t = void (Abstract3DController::*)(const QVector3D & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Abstract3DController::queriedGraphPositionChanged)) {
                *result = 20;
                return;
            }
        }
        {
            using _t = void (Abstract3DController::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Abstract3DController::marginChanged)) {
                *result = 21;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject QtDataVisualization::Abstract3DController::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QtDataVisualization__Abstract3DController.data,
    qt_meta_data_QtDataVisualization__Abstract3DController,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QtDataVisualization::Abstract3DController::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtDataVisualization::Abstract3DController::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QtDataVisualization__Abstract3DController.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QtDataVisualization::Abstract3DController::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 48)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 48;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 48)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 48;
    }
    return _id;
}

// SIGNAL 0
void QtDataVisualization::Abstract3DController::shadowQualityChanged(QAbstract3DGraph::ShadowQuality _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QtDataVisualization::Abstract3DController::activeInputHandlerChanged(QAbstract3DInputHandler * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QtDataVisualization::Abstract3DController::activeThemeChanged(Q3DTheme * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QtDataVisualization::Abstract3DController::selectionModeChanged(QAbstract3DGraph::SelectionFlags _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QtDataVisualization::Abstract3DController::needRender()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QtDataVisualization::Abstract3DController::axisXChanged(QAbstract3DAxis * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void QtDataVisualization::Abstract3DController::axisYChanged(QAbstract3DAxis * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void QtDataVisualization::Abstract3DController::axisZChanged(QAbstract3DAxis * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void QtDataVisualization::Abstract3DController::elementSelected(QAbstract3DGraph::ElementType _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void QtDataVisualization::Abstract3DController::measureFpsChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void QtDataVisualization::Abstract3DController::currentFpsChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 10, _a);
}

// SIGNAL 11
void QtDataVisualization::Abstract3DController::orthoProjectionChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 11, _a);
}

// SIGNAL 12
void QtDataVisualization::Abstract3DController::aspectRatioChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 12, _a);
}

// SIGNAL 13
void QtDataVisualization::Abstract3DController::horizontalAspectRatioChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 13, _a);
}

// SIGNAL 14
void QtDataVisualization::Abstract3DController::optimizationHintsChanged(QAbstract3DGraph::OptimizationHints _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 14, _a);
}

// SIGNAL 15
void QtDataVisualization::Abstract3DController::polarChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 15, _a);
}

// SIGNAL 16
void QtDataVisualization::Abstract3DController::radialLabelOffsetChanged(float _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 16, _a);
}

// SIGNAL 17
void QtDataVisualization::Abstract3DController::reflectionChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 17, _a);
}

// SIGNAL 18
void QtDataVisualization::Abstract3DController::reflectivityChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 18, _a);
}

// SIGNAL 19
void QtDataVisualization::Abstract3DController::localeChanged(const QLocale & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 19, _a);
}

// SIGNAL 20
void QtDataVisualization::Abstract3DController::queriedGraphPositionChanged(const QVector3D & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 20, _a);
}

// SIGNAL 21
void QtDataVisualization::Abstract3DController::marginChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 21, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
