/****************************************************************************
** Meta object code from reading C++ file 'q3dtheme.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kde-qtdatavis3d/src/datavisualization/theme/q3dtheme.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'q3dtheme.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QtDataVisualization__Q3DTheme_t {
    QByteArrayData data[73];
    char stringdata0[1250];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QtDataVisualization__Q3DTheme_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QtDataVisualization__Q3DTheme_t qt_meta_stringdata_QtDataVisualization__Q3DTheme = {
    {
QT_MOC_LITERAL(0, 0, 29), // "QtDataVisualization::Q3DTheme"
QT_MOC_LITERAL(1, 30, 11), // "typeChanged"
QT_MOC_LITERAL(2, 42, 0), // ""
QT_MOC_LITERAL(3, 43, 15), // "Q3DTheme::Theme"
QT_MOC_LITERAL(4, 59, 9), // "themeType"
QT_MOC_LITERAL(5, 69, 17), // "baseColorsChanged"
QT_MOC_LITERAL(6, 87, 13), // "QList<QColor>"
QT_MOC_LITERAL(7, 101, 6), // "colors"
QT_MOC_LITERAL(8, 108, 22), // "backgroundColorChanged"
QT_MOC_LITERAL(9, 131, 5), // "color"
QT_MOC_LITERAL(10, 137, 18), // "windowColorChanged"
QT_MOC_LITERAL(11, 156, 21), // "labelTextColorChanged"
QT_MOC_LITERAL(12, 178, 27), // "labelBackgroundColorChanged"
QT_MOC_LITERAL(13, 206, 20), // "gridLineColorChanged"
QT_MOC_LITERAL(14, 227, 27), // "singleHighlightColorChanged"
QT_MOC_LITERAL(15, 255, 26), // "multiHighlightColorChanged"
QT_MOC_LITERAL(16, 282, 17), // "lightColorChanged"
QT_MOC_LITERAL(17, 300, 20), // "baseGradientsChanged"
QT_MOC_LITERAL(18, 321, 22), // "QList<QLinearGradient>"
QT_MOC_LITERAL(19, 344, 9), // "gradients"
QT_MOC_LITERAL(20, 354, 30), // "singleHighlightGradientChanged"
QT_MOC_LITERAL(21, 385, 15), // "QLinearGradient"
QT_MOC_LITERAL(22, 401, 8), // "gradient"
QT_MOC_LITERAL(23, 410, 29), // "multiHighlightGradientChanged"
QT_MOC_LITERAL(24, 440, 20), // "lightStrengthChanged"
QT_MOC_LITERAL(25, 461, 8), // "strength"
QT_MOC_LITERAL(26, 470, 27), // "ambientLightStrengthChanged"
QT_MOC_LITERAL(27, 498, 29), // "highlightLightStrengthChanged"
QT_MOC_LITERAL(28, 528, 25), // "labelBorderEnabledChanged"
QT_MOC_LITERAL(29, 554, 7), // "enabled"
QT_MOC_LITERAL(30, 562, 11), // "fontChanged"
QT_MOC_LITERAL(31, 574, 4), // "font"
QT_MOC_LITERAL(32, 579, 24), // "backgroundEnabledChanged"
QT_MOC_LITERAL(33, 604, 18), // "gridEnabledChanged"
QT_MOC_LITERAL(34, 623, 29), // "labelBackgroundEnabledChanged"
QT_MOC_LITERAL(35, 653, 17), // "colorStyleChanged"
QT_MOC_LITERAL(36, 671, 20), // "Q3DTheme::ColorStyle"
QT_MOC_LITERAL(37, 692, 5), // "style"
QT_MOC_LITERAL(38, 698, 4), // "type"
QT_MOC_LITERAL(39, 703, 5), // "Theme"
QT_MOC_LITERAL(40, 709, 10), // "baseColors"
QT_MOC_LITERAL(41, 720, 15), // "backgroundColor"
QT_MOC_LITERAL(42, 736, 11), // "windowColor"
QT_MOC_LITERAL(43, 748, 14), // "labelTextColor"
QT_MOC_LITERAL(44, 763, 20), // "labelBackgroundColor"
QT_MOC_LITERAL(45, 784, 13), // "gridLineColor"
QT_MOC_LITERAL(46, 798, 20), // "singleHighlightColor"
QT_MOC_LITERAL(47, 819, 19), // "multiHighlightColor"
QT_MOC_LITERAL(48, 839, 10), // "lightColor"
QT_MOC_LITERAL(49, 850, 13), // "baseGradients"
QT_MOC_LITERAL(50, 864, 23), // "singleHighlightGradient"
QT_MOC_LITERAL(51, 888, 22), // "multiHighlightGradient"
QT_MOC_LITERAL(52, 911, 13), // "lightStrength"
QT_MOC_LITERAL(53, 925, 20), // "ambientLightStrength"
QT_MOC_LITERAL(54, 946, 22), // "highlightLightStrength"
QT_MOC_LITERAL(55, 969, 18), // "labelBorderEnabled"
QT_MOC_LITERAL(56, 988, 17), // "backgroundEnabled"
QT_MOC_LITERAL(57, 1006, 11), // "gridEnabled"
QT_MOC_LITERAL(58, 1018, 22), // "labelBackgroundEnabled"
QT_MOC_LITERAL(59, 1041, 10), // "colorStyle"
QT_MOC_LITERAL(60, 1052, 10), // "ColorStyle"
QT_MOC_LITERAL(61, 1063, 17), // "ColorStyleUniform"
QT_MOC_LITERAL(62, 1081, 24), // "ColorStyleObjectGradient"
QT_MOC_LITERAL(63, 1106, 23), // "ColorStyleRangeGradient"
QT_MOC_LITERAL(64, 1130, 7), // "ThemeQt"
QT_MOC_LITERAL(65, 1138, 18), // "ThemePrimaryColors"
QT_MOC_LITERAL(66, 1157, 10), // "ThemeDigia"
QT_MOC_LITERAL(67, 1168, 14), // "ThemeStoneMoss"
QT_MOC_LITERAL(68, 1183, 13), // "ThemeArmyBlue"
QT_MOC_LITERAL(69, 1197, 10), // "ThemeRetro"
QT_MOC_LITERAL(70, 1208, 10), // "ThemeEbony"
QT_MOC_LITERAL(71, 1219, 13), // "ThemeIsabelle"
QT_MOC_LITERAL(72, 1233, 16) // "ThemeUserDefined"

    },
    "QtDataVisualization::Q3DTheme\0typeChanged\0"
    "\0Q3DTheme::Theme\0themeType\0baseColorsChanged\0"
    "QList<QColor>\0colors\0backgroundColorChanged\0"
    "color\0windowColorChanged\0labelTextColorChanged\0"
    "labelBackgroundColorChanged\0"
    "gridLineColorChanged\0singleHighlightColorChanged\0"
    "multiHighlightColorChanged\0lightColorChanged\0"
    "baseGradientsChanged\0QList<QLinearGradient>\0"
    "gradients\0singleHighlightGradientChanged\0"
    "QLinearGradient\0gradient\0"
    "multiHighlightGradientChanged\0"
    "lightStrengthChanged\0strength\0"
    "ambientLightStrengthChanged\0"
    "highlightLightStrengthChanged\0"
    "labelBorderEnabledChanged\0enabled\0"
    "fontChanged\0font\0backgroundEnabledChanged\0"
    "gridEnabledChanged\0labelBackgroundEnabledChanged\0"
    "colorStyleChanged\0Q3DTheme::ColorStyle\0"
    "style\0type\0Theme\0baseColors\0backgroundColor\0"
    "windowColor\0labelTextColor\0"
    "labelBackgroundColor\0gridLineColor\0"
    "singleHighlightColor\0multiHighlightColor\0"
    "lightColor\0baseGradients\0"
    "singleHighlightGradient\0multiHighlightGradient\0"
    "lightStrength\0ambientLightStrength\0"
    "highlightLightStrength\0labelBorderEnabled\0"
    "backgroundEnabled\0gridEnabled\0"
    "labelBackgroundEnabled\0colorStyle\0"
    "ColorStyle\0ColorStyleUniform\0"
    "ColorStyleObjectGradient\0"
    "ColorStyleRangeGradient\0ThemeQt\0"
    "ThemePrimaryColors\0ThemeDigia\0"
    "ThemeStoneMoss\0ThemeArmyBlue\0ThemeRetro\0"
    "ThemeEbony\0ThemeIsabelle\0ThemeUserDefined"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QtDataVisualization__Q3DTheme[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      22,   14, // methods
      22,  190, // properties
       2,  278, // enums/sets
       0,    0, // constructors
       0,       // flags
      22,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  124,    2, 0x06 /* Public */,
       5,    1,  127,    2, 0x06 /* Public */,
       8,    1,  130,    2, 0x06 /* Public */,
      10,    1,  133,    2, 0x06 /* Public */,
      11,    1,  136,    2, 0x06 /* Public */,
      12,    1,  139,    2, 0x06 /* Public */,
      13,    1,  142,    2, 0x06 /* Public */,
      14,    1,  145,    2, 0x06 /* Public */,
      15,    1,  148,    2, 0x06 /* Public */,
      16,    1,  151,    2, 0x06 /* Public */,
      17,    1,  154,    2, 0x06 /* Public */,
      20,    1,  157,    2, 0x06 /* Public */,
      23,    1,  160,    2, 0x06 /* Public */,
      24,    1,  163,    2, 0x06 /* Public */,
      26,    1,  166,    2, 0x06 /* Public */,
      27,    1,  169,    2, 0x06 /* Public */,
      28,    1,  172,    2, 0x06 /* Public */,
      30,    1,  175,    2, 0x06 /* Public */,
      32,    1,  178,    2, 0x06 /* Public */,
      33,    1,  181,    2, 0x06 /* Public */,
      34,    1,  184,    2, 0x06 /* Public */,
      35,    1,  187,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void, QMetaType::QColor,    9,
    QMetaType::Void, QMetaType::QColor,    9,
    QMetaType::Void, QMetaType::QColor,    9,
    QMetaType::Void, QMetaType::QColor,    9,
    QMetaType::Void, QMetaType::QColor,    9,
    QMetaType::Void, QMetaType::QColor,    9,
    QMetaType::Void, QMetaType::QColor,    9,
    QMetaType::Void, QMetaType::QColor,    9,
    QMetaType::Void, 0x80000000 | 18,   19,
    QMetaType::Void, 0x80000000 | 21,   22,
    QMetaType::Void, 0x80000000 | 21,   22,
    QMetaType::Void, QMetaType::Float,   25,
    QMetaType::Void, QMetaType::Float,   25,
    QMetaType::Void, QMetaType::Float,   25,
    QMetaType::Void, QMetaType::Bool,   29,
    QMetaType::Void, QMetaType::QFont,   31,
    QMetaType::Void, QMetaType::Bool,   29,
    QMetaType::Void, QMetaType::Bool,   29,
    QMetaType::Void, QMetaType::Bool,   29,
    QMetaType::Void, 0x80000000 | 36,   37,

 // properties: name, type, flags
      38, 0x80000000 | 39, 0x0049510b,
      40, 0x80000000 | 6, 0x0049510b,
      41, QMetaType::QColor, 0x00495103,
      42, QMetaType::QColor, 0x00495103,
      43, QMetaType::QColor, 0x00495103,
      44, QMetaType::QColor, 0x00495103,
      45, QMetaType::QColor, 0x00495103,
      46, QMetaType::QColor, 0x00495103,
      47, QMetaType::QColor, 0x00495103,
      48, QMetaType::QColor, 0x00495103,
      49, 0x80000000 | 18, 0x0049510b,
      50, 0x80000000 | 21, 0x0049510b,
      51, 0x80000000 | 21, 0x0049510b,
      52, QMetaType::Float, 0x00495103,
      53, QMetaType::Float, 0x00495103,
      54, QMetaType::Float, 0x00495103,
      55, QMetaType::Bool, 0x00495103,
      31, QMetaType::QFont, 0x00495103,
      56, QMetaType::Bool, 0x00495103,
      57, QMetaType::Bool, 0x00495103,
      58, QMetaType::Bool, 0x00495103,
      59, 0x80000000 | 60, 0x0049510b,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,
       4,
       5,
       6,
       7,
       8,
       9,
      10,
      11,
      12,
      13,
      14,
      15,
      16,
      17,
      18,
      19,
      20,
      21,

 // enums: name, alias, flags, count, data
      60,   60, 0x0,    3,  288,
      39,   39, 0x0,    9,  294,

 // enum data: key, value
      61, uint(QtDataVisualization::Q3DTheme::ColorStyleUniform),
      62, uint(QtDataVisualization::Q3DTheme::ColorStyleObjectGradient),
      63, uint(QtDataVisualization::Q3DTheme::ColorStyleRangeGradient),
      64, uint(QtDataVisualization::Q3DTheme::ThemeQt),
      65, uint(QtDataVisualization::Q3DTheme::ThemePrimaryColors),
      66, uint(QtDataVisualization::Q3DTheme::ThemeDigia),
      67, uint(QtDataVisualization::Q3DTheme::ThemeStoneMoss),
      68, uint(QtDataVisualization::Q3DTheme::ThemeArmyBlue),
      69, uint(QtDataVisualization::Q3DTheme::ThemeRetro),
      70, uint(QtDataVisualization::Q3DTheme::ThemeEbony),
      71, uint(QtDataVisualization::Q3DTheme::ThemeIsabelle),
      72, uint(QtDataVisualization::Q3DTheme::ThemeUserDefined),

       0        // eod
};

void QtDataVisualization::Q3DTheme::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<Q3DTheme *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->typeChanged((*reinterpret_cast< Q3DTheme::Theme(*)>(_a[1]))); break;
        case 1: _t->baseColorsChanged((*reinterpret_cast< const QList<QColor>(*)>(_a[1]))); break;
        case 2: _t->backgroundColorChanged((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 3: _t->windowColorChanged((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 4: _t->labelTextColorChanged((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 5: _t->labelBackgroundColorChanged((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 6: _t->gridLineColorChanged((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 7: _t->singleHighlightColorChanged((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 8: _t->multiHighlightColorChanged((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 9: _t->lightColorChanged((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 10: _t->baseGradientsChanged((*reinterpret_cast< const QList<QLinearGradient>(*)>(_a[1]))); break;
        case 11: _t->singleHighlightGradientChanged((*reinterpret_cast< const QLinearGradient(*)>(_a[1]))); break;
        case 12: _t->multiHighlightGradientChanged((*reinterpret_cast< const QLinearGradient(*)>(_a[1]))); break;
        case 13: _t->lightStrengthChanged((*reinterpret_cast< float(*)>(_a[1]))); break;
        case 14: _t->ambientLightStrengthChanged((*reinterpret_cast< float(*)>(_a[1]))); break;
        case 15: _t->highlightLightStrengthChanged((*reinterpret_cast< float(*)>(_a[1]))); break;
        case 16: _t->labelBorderEnabledChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 17: _t->fontChanged((*reinterpret_cast< const QFont(*)>(_a[1]))); break;
        case 18: _t->backgroundEnabledChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 19: _t->gridEnabledChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 20: _t->labelBackgroundEnabledChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 21: _t->colorStyleChanged((*reinterpret_cast< Q3DTheme::ColorStyle(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QColor> >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (Q3DTheme::*)(Q3DTheme::Theme );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Q3DTheme::typeChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (Q3DTheme::*)(const QList<QColor> & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Q3DTheme::baseColorsChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (Q3DTheme::*)(const QColor & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Q3DTheme::backgroundColorChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (Q3DTheme::*)(const QColor & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Q3DTheme::windowColorChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (Q3DTheme::*)(const QColor & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Q3DTheme::labelTextColorChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (Q3DTheme::*)(const QColor & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Q3DTheme::labelBackgroundColorChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (Q3DTheme::*)(const QColor & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Q3DTheme::gridLineColorChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (Q3DTheme::*)(const QColor & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Q3DTheme::singleHighlightColorChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (Q3DTheme::*)(const QColor & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Q3DTheme::multiHighlightColorChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (Q3DTheme::*)(const QColor & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Q3DTheme::lightColorChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (Q3DTheme::*)(const QList<QLinearGradient> & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Q3DTheme::baseGradientsChanged)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (Q3DTheme::*)(const QLinearGradient & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Q3DTheme::singleHighlightGradientChanged)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (Q3DTheme::*)(const QLinearGradient & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Q3DTheme::multiHighlightGradientChanged)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (Q3DTheme::*)(float );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Q3DTheme::lightStrengthChanged)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (Q3DTheme::*)(float );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Q3DTheme::ambientLightStrengthChanged)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (Q3DTheme::*)(float );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Q3DTheme::highlightLightStrengthChanged)) {
                *result = 15;
                return;
            }
        }
        {
            using _t = void (Q3DTheme::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Q3DTheme::labelBorderEnabledChanged)) {
                *result = 16;
                return;
            }
        }
        {
            using _t = void (Q3DTheme::*)(const QFont & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Q3DTheme::fontChanged)) {
                *result = 17;
                return;
            }
        }
        {
            using _t = void (Q3DTheme::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Q3DTheme::backgroundEnabledChanged)) {
                *result = 18;
                return;
            }
        }
        {
            using _t = void (Q3DTheme::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Q3DTheme::gridEnabledChanged)) {
                *result = 19;
                return;
            }
        }
        {
            using _t = void (Q3DTheme::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Q3DTheme::labelBackgroundEnabledChanged)) {
                *result = 20;
                return;
            }
        }
        {
            using _t = void (Q3DTheme::*)(Q3DTheme::ColorStyle );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Q3DTheme::colorStyleChanged)) {
                *result = 21;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QColor> >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<Q3DTheme *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< Theme*>(_v) = _t->type(); break;
        case 1: *reinterpret_cast< QList<QColor>*>(_v) = _t->baseColors(); break;
        case 2: *reinterpret_cast< QColor*>(_v) = _t->backgroundColor(); break;
        case 3: *reinterpret_cast< QColor*>(_v) = _t->windowColor(); break;
        case 4: *reinterpret_cast< QColor*>(_v) = _t->labelTextColor(); break;
        case 5: *reinterpret_cast< QColor*>(_v) = _t->labelBackgroundColor(); break;
        case 6: *reinterpret_cast< QColor*>(_v) = _t->gridLineColor(); break;
        case 7: *reinterpret_cast< QColor*>(_v) = _t->singleHighlightColor(); break;
        case 8: *reinterpret_cast< QColor*>(_v) = _t->multiHighlightColor(); break;
        case 9: *reinterpret_cast< QColor*>(_v) = _t->lightColor(); break;
        case 10: *reinterpret_cast< QList<QLinearGradient>*>(_v) = _t->baseGradients(); break;
        case 11: *reinterpret_cast< QLinearGradient*>(_v) = _t->singleHighlightGradient(); break;
        case 12: *reinterpret_cast< QLinearGradient*>(_v) = _t->multiHighlightGradient(); break;
        case 13: *reinterpret_cast< float*>(_v) = _t->lightStrength(); break;
        case 14: *reinterpret_cast< float*>(_v) = _t->ambientLightStrength(); break;
        case 15: *reinterpret_cast< float*>(_v) = _t->highlightLightStrength(); break;
        case 16: *reinterpret_cast< bool*>(_v) = _t->isLabelBorderEnabled(); break;
        case 17: *reinterpret_cast< QFont*>(_v) = _t->font(); break;
        case 18: *reinterpret_cast< bool*>(_v) = _t->isBackgroundEnabled(); break;
        case 19: *reinterpret_cast< bool*>(_v) = _t->isGridEnabled(); break;
        case 20: *reinterpret_cast< bool*>(_v) = _t->isLabelBackgroundEnabled(); break;
        case 21: *reinterpret_cast< ColorStyle*>(_v) = _t->colorStyle(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<Q3DTheme *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setType(*reinterpret_cast< Theme*>(_v)); break;
        case 1: _t->setBaseColors(*reinterpret_cast< QList<QColor>*>(_v)); break;
        case 2: _t->setBackgroundColor(*reinterpret_cast< QColor*>(_v)); break;
        case 3: _t->setWindowColor(*reinterpret_cast< QColor*>(_v)); break;
        case 4: _t->setLabelTextColor(*reinterpret_cast< QColor*>(_v)); break;
        case 5: _t->setLabelBackgroundColor(*reinterpret_cast< QColor*>(_v)); break;
        case 6: _t->setGridLineColor(*reinterpret_cast< QColor*>(_v)); break;
        case 7: _t->setSingleHighlightColor(*reinterpret_cast< QColor*>(_v)); break;
        case 8: _t->setMultiHighlightColor(*reinterpret_cast< QColor*>(_v)); break;
        case 9: _t->setLightColor(*reinterpret_cast< QColor*>(_v)); break;
        case 10: _t->setBaseGradients(*reinterpret_cast< QList<QLinearGradient>*>(_v)); break;
        case 11: _t->setSingleHighlightGradient(*reinterpret_cast< QLinearGradient*>(_v)); break;
        case 12: _t->setMultiHighlightGradient(*reinterpret_cast< QLinearGradient*>(_v)); break;
        case 13: _t->setLightStrength(*reinterpret_cast< float*>(_v)); break;
        case 14: _t->setAmbientLightStrength(*reinterpret_cast< float*>(_v)); break;
        case 15: _t->setHighlightLightStrength(*reinterpret_cast< float*>(_v)); break;
        case 16: _t->setLabelBorderEnabled(*reinterpret_cast< bool*>(_v)); break;
        case 17: _t->setFont(*reinterpret_cast< QFont*>(_v)); break;
        case 18: _t->setBackgroundEnabled(*reinterpret_cast< bool*>(_v)); break;
        case 19: _t->setGridEnabled(*reinterpret_cast< bool*>(_v)); break;
        case 20: _t->setLabelBackgroundEnabled(*reinterpret_cast< bool*>(_v)); break;
        case 21: _t->setColorStyle(*reinterpret_cast< ColorStyle*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QtDataVisualization::Q3DTheme::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QtDataVisualization__Q3DTheme.data,
    qt_meta_data_QtDataVisualization__Q3DTheme,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QtDataVisualization::Q3DTheme::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtDataVisualization::Q3DTheme::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QtDataVisualization__Q3DTheme.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QtDataVisualization::Q3DTheme::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 22;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 22;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 22;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 22;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 22;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QtDataVisualization::Q3DTheme::typeChanged(Q3DTheme::Theme _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QtDataVisualization::Q3DTheme::baseColorsChanged(const QList<QColor> & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QtDataVisualization::Q3DTheme::backgroundColorChanged(const QColor & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QtDataVisualization::Q3DTheme::windowColorChanged(const QColor & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QtDataVisualization::Q3DTheme::labelTextColorChanged(const QColor & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void QtDataVisualization::Q3DTheme::labelBackgroundColorChanged(const QColor & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void QtDataVisualization::Q3DTheme::gridLineColorChanged(const QColor & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void QtDataVisualization::Q3DTheme::singleHighlightColorChanged(const QColor & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void QtDataVisualization::Q3DTheme::multiHighlightColorChanged(const QColor & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void QtDataVisualization::Q3DTheme::lightColorChanged(const QColor & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void QtDataVisualization::Q3DTheme::baseGradientsChanged(const QList<QLinearGradient> & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 10, _a);
}

// SIGNAL 11
void QtDataVisualization::Q3DTheme::singleHighlightGradientChanged(const QLinearGradient & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 11, _a);
}

// SIGNAL 12
void QtDataVisualization::Q3DTheme::multiHighlightGradientChanged(const QLinearGradient & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 12, _a);
}

// SIGNAL 13
void QtDataVisualization::Q3DTheme::lightStrengthChanged(float _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 13, _a);
}

// SIGNAL 14
void QtDataVisualization::Q3DTheme::ambientLightStrengthChanged(float _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 14, _a);
}

// SIGNAL 15
void QtDataVisualization::Q3DTheme::highlightLightStrengthChanged(float _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 15, _a);
}

// SIGNAL 16
void QtDataVisualization::Q3DTheme::labelBorderEnabledChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 16, _a);
}

// SIGNAL 17
void QtDataVisualization::Q3DTheme::fontChanged(const QFont & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 17, _a);
}

// SIGNAL 18
void QtDataVisualization::Q3DTheme::backgroundEnabledChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 18, _a);
}

// SIGNAL 19
void QtDataVisualization::Q3DTheme::gridEnabledChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 19, _a);
}

// SIGNAL 20
void QtDataVisualization::Q3DTheme::labelBackgroundEnabledChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 20, _a);
}

// SIGNAL 21
void QtDataVisualization::Q3DTheme::colorStyleChanged(Q3DTheme::ColorStyle _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 21, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
